/*
 * Closure methods tests
 * Testing Closure methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
static guint flag = 0;
enum {
    CALLBACK = 1,
    DESTROY
    };

static void test_callback(void)
{
    g_print("Enter test callback\n");
}

static void test_destroy(gpointer data, GClosure *closure)
{
    flag = DESTROY;
}


/*
 * Name: g_cclosure_new
 * Params: GCallback callback_func, gpointer user_data, GClosureNotify destroy_data
 * Returns: GCClosure*
 * Assertion: Creates a new closure which invokes callback_func with user_data as the last parameter.
 *            callback_func :  the function to invoke
 *            user_data : user data to pass to callback_func
 *            destroy_data :  destroy notify to be called when user_data is no longer used
 *            Returns :   a new GCClosure
 * Params Values: GCallback callback_func, gpointer user_data, GClosureNotify destroy_data
 * Expected: GCClosure*
 * 
 * Name: g_closure_ref
 * Params: GClosure *closure
 * Returns: GClosure *closure
 * Assertion: Increments the reference count on a closure to force it staying alive while the caller holds a pointer to it.
 *            closure :    GClosure to increment the reference count on
 *            Returns :   The closure passed in, for convenience
 * Params Values: GClosure *closure
 * Expected: GClosure *closure
 * 
 * Name: g_closure_unref
 * Params: GClosure *closure
 * Returns: void
 * Assertion: Decrements the reference count of a closure after it was previously incremented by the same caller. If no other callers are using the closure, then the closure will be destroyed and freed.
 *            closure :    GClosure to decrement the reference count on
 * Params Values: GClosure *closure
 * Expected: void  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GClosure* gcclosure = g_cclosure_new((GCallback)test_callback, GINT_TO_POINTER(10), (GClosureNotify)test_destroy);
    if (gcclosure == NULL){
        TEST_FAIL("g_cclosure_new: failed", NULL);
    }
    g_closure_ref(gcclosure);
    g_closure_unref(gcclosure);
    g_closure_unref(gcclosure);
    if (flag != DESTROY){
        TEST_FAIL("g_closure_unref: failed", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for Closure ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {NULL,0}
};
