/*
 * GLog methods tests
 * Testing GLog methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
static gint log_flag = 0;

static void my_log_handler(const gchar * domain, GLogLevelFlags level,
            const gchar * message, gpointer user_data)
{
    log_flag = *(gint *)user_data;
}

/*
 * Name: g_log_set_handler
 * Params: const gchar *log_domain, GLogLevelFlags log_levels, GLogFunc log_func, gpointer user_data
 * Returns: guint
 * Assertion: Sets the log handler for a domain and a set of log levels. To handle fatal and recursive messages the log_levels parameter must be combined with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 * Note that since the G_LOG_LEVEL_ERROR log level is always fatal, if you want to set a handler for this log level you must combine it with G_LOG_FLAG_FATAL.  
 *            log_domain :	the log domain, or NULL for the default "" application domain.
 *            log_levels :	the log levels to apply the log handler for. To handle fatal and recursive messages as well, combine the log levels with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 *            log_func : the log handler function.
 *            user_data : data passed to the log handler.
 *            Returns : the id of the new handler.
 * Params Values: log_domain = NULL, log_level = G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, user_data = 1, mask = G_LOG_LEVEL_WARNING
 * Expected: 1  
 */
static void test_purpose_1()
{
    #define DOMAIN    NULL
    #define LEVEL     G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION
    #define USER_DATA 1
    #define MASK      G_LOG_LEVEL_WARNING
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    guint id = 0;
    gint data = USER_DATA;
    
    id = g_log_set_handler(DOMAIN, LEVEL, (GLogFunc)my_log_handler, &data);
    g_log(DOMAIN, MASK, "%s", "Log error");
    if (log_flag != USER_DATA){
        TEST_FAIL("g_log_set_handler: set handler failed", NULL);
        log_flag = 0;
    }
    log_flag = 0;
    
    #undef DOMAIN
    #undef LEVEL
    #undef USER_DATA
    #undef MASK
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_log_set_handler
 * Params: const gchar *log_domain, GLogLevelFlags log_levels, GLogFunc log_func, gpointer user_data
 * Returns: guint
 * Assertion: Sets the log handler for a domain and a set of log levels. To handle fatal and recursive messages the log_levels parameter must be combined with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 * Note that since the G_LOG_LEVEL_ERROR log level is always fatal, if you want to set a handler for this log level you must combine it with G_LOG_FLAG_FATAL.  
 *            log_domain :	the log domain, or NULL for the default "" application domain.
 *            log_levels :	the log levels to apply the log handler for. To handle fatal and recursive messages as well, combine the log levels with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 *            log_func : the log handler function.
 *            user_data : data passed to the log handler.
 *            Returns : the id of the new handler.
 * Params Values: log_domain = "Gtk", log_level = G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, user_data = 1, mask = G_LOG_LEVEL_CRITICAL
 * Expected: 1  
 */
static void test_purpose_2()
{
    #define DOMAIN    "Gtk"
    #define LEVEL     G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION
    #define USER_DATA 1
    #define MASK      G_LOG_LEVEL_CRITICAL
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    guint id = 0;
    gint data = USER_DATA;
    
    id = g_log_set_handler(DOMAIN, LEVEL, (GLogFunc)my_log_handler, &data);
    g_log(DOMAIN, MASK, "%s", "Log error");
    if (log_flag != USER_DATA){
        TEST_FAIL("g_log_set_handler: set handler failed", NULL);
        log_flag = 0;
    }
    log_flag = 0;
    
    #undef DOMAIN
    #undef LEVEL
    #undef USER_DATA
    #undef MASK
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_log_set_handler
 * Params: const gchar *log_domain, GLogLevelFlags log_levels, GLogFunc log_func, gpointer user_data
 * Returns: guint
 * Assertion: Sets the log handler for a domain and a set of log levels. To handle fatal and recursive messages the log_levels parameter must be combined with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 * Note that since the G_LOG_LEVEL_ERROR log level is always fatal, if you want to set a handler for this log level you must combine it with G_LOG_FLAG_FATAL.  
 *            log_domain :	the log domain, or NULL for the default "" application domain.
 *            log_levels :	the log levels to apply the log handler for. To handle fatal and recursive messages as well, combine the log levels with the G_LOG_FLAG_FATAL and G_LOG_FLAG_RECURSION bit flags.
 *            log_func : the log handler function.
 *            user_data : data passed to the log handler.
 *            Returns : the id of the new handler.
 * Params Values: log_domain = "GLib", log_level = G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, user_data = 1, mask = G_LOG_LEVEL_INFO
 * Expected: 1  
 */
static void test_purpose_3()
{
    #define DOMAIN    "GLib"
    #define LEVEL     G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION
    #define USER_DATA 1
    #define MASK      G_LOG_LEVEL_INFO
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    guint id = 0;
    gint data = USER_DATA;
    
    id = g_log_set_handler(DOMAIN, LEVEL, (GLogFunc)my_log_handler, &data);
    g_log(DOMAIN, MASK, "%s", "Log error");
    if (log_flag != USER_DATA){
        TEST_FAIL("g_log_set_handler: set handler failed", NULL);
        log_flag = 0;
    }
    log_flag = 0;
    
    #undef DOMAIN
    #undef LEVEL
    #undef USER_DATA
    #undef MASK
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_log_set_always_fatal
 * Params: GLogLevelFlags fatal_mask 
 * Returns: GLogLevelFlags
 * Assertion: Sets the message levels which are always fatal, in any log domain. When a message with any of these levels is logged the program terminates. You can only set the levels defined by GLib to be fatal. G_LOG_LEVEL_ERROR is always fatal.
 *            fatal_mask :	the mask containing bits set for each level of error which is to be fatal.
 *            Returns : the old fatal mask. 
 * Params Values: fatal_mask = G_LOG_LEVEL_MESSAGE
 * Expected: GLogLevelFlags  
 */
static void test_purpose_4()
{
    #define TBFATAL    G_LOG_LEVEL_MESSAGE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GLogLevelFlags old_flags, new_flags;
    old_flags = g_log_set_always_fatal(TBFATAL);
    new_flags = g_log_set_always_fatal(old_flags);
    if ((new_flags & TBFATAL) == 0){
        TEST_FAIL("g_log_set_always_fatal: failed", NULL);
    }
    else{
        if ((new_flags & G_LOG_LEVEL_ERROR) == 0){
            TEST_FAIL("g_log_set_always_fatal: failed", NULL);
        }
    }
    #undef TBFATAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_log_set_always_fatal
 * Params: GLogLevelFlags fatal_mask 
 * Returns: GLogLevelFlags
 * Assertion: Sets the message levels which are always fatal, in any log domain. When a message with any of these levels is logged the program terminates. You can only set the levels defined by GLib to be fatal. G_LOG_LEVEL_ERROR is always fatal.
 *            fatal_mask :	the mask containing bits set for each level of error which is to be fatal.
 *            Returns : the old fatal mask. 
 * Params Values: fatal_mask = G_LOG_LEVEL_WARNING
 * Expected: GLogLevelFlags  
 */
static void test_purpose_5()
{
    #define TBFATAL    G_LOG_LEVEL_WARNING
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GLogLevelFlags old_flags, new_flags;
    old_flags = g_log_set_always_fatal(TBFATAL);
    new_flags = g_log_set_always_fatal(old_flags);
    if ((new_flags & TBFATAL) == 0){
        TEST_FAIL("g_log_set_always_fatal: failed", NULL);
    }
    else{
        if ((new_flags & G_LOG_LEVEL_ERROR) == 0){
            TEST_FAIL("g_log_set_always_fatal: failed", NULL);
        }
    }
    #undef TBFATAL
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GLog ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {NULL,0}
};
