/*
 * GRandom methods tests
 * Testing GRandom methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name: g_rand_new
 * Params: void
 * Returns: GRand*
 * Assertion: Creates a new random number generator initialized with a seed taken either from /dev/urandom (if existing) or from the current time (as a fallback).
 *            Returns : the new GRand.
 * Params Values: void
 * Expected: GRand*  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    if (g_rand_new() == NULL){
        TEST_FAIL("g_rand_new: failed", NULL);
    }
    g_rand_free(grand);
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_int_range
 * Params: GRand *rand_, gint32 begin, gint32 end
 * Returns: gint32
 * Assertion: Returns the next random gint32 from rand_ equally distributed over the range [begin..end-1].
 *            rand_ : a GRand.
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 10, 100
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_2()
{
    #define BEGIN    10
    #define END      100
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    gint32 rand_n = g_rand_int_range(grand, BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_rand_int_range: failed", NULL);
    }
    g_rand_free(grand);
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_int_range
 * Params: GRand *rand_, gint32 begin, gint32 end
 * Returns: gint32
 * Assertion: Returns the next random gint32 from rand_ equally distributed over the range [begin..end-1].
 *            rand_ : a GRand.
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 1, 100234
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_3()
{
    #define BEGIN    1
    #define END      100234
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    gint32 rand_n = g_rand_int_range(grand, BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_rand_int_range: failed", NULL);
    }
    g_rand_free(grand);
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_int_range
 * Params: gint32 begin, gint32 end
 * Returns: gint32
 * Assertion: Returns a random gint32 equally distributed over the range [begin..end-1].
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 10, 100
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_4()
{
    #define BEGIN    10
    #define END      100
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gint32 rand_n = g_random_int_range(BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_random_int_range: failed", NULL);
    }
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_int_range
 * Params: gint32 begin, gint32 end
 * Returns: gint32
 * Assertion: Returns a random gint32 equally distributed over the range [begin..end-1].
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 1, 100234
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_5()
{
    #define BEGIN    1
    #define END      100234
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gint32 rand_n = g_random_int_range(BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_random_int_range: failed", NULL);
    }
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_double_range
 * Params: gdouble begin, gdouble end
 * Returns: gdouble
 * Assertion: Returns a random gdouble equally distributed over the range [begin..end).
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 10.32, 100.87
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_6()
{
    #define BEGIN    10.32
    #define END      100.87
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    g_random_set_seed(780928);
    gdouble rand_n = g_random_double_range(BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_random_double_range: failed", NULL);
    }
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_double_range
 * Params: gdouble begin, gdouble end
 * Returns: gdouble
 * Assertion: Returns a random gdouble equally distributed over the range [begin..end).
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 1.0, 10023
 * Expected: the return is in [begin..end-1]  
 */
static void test_purpose_7()
{
    #define BEGIN    1.0
    #define END      10023
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    g_random_set_seed(780928);
    gdouble rand_n = g_random_double_range(BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_random_double_range: failed", NULL);
    }
    
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_double
 * Params: void
 * Returns: gdouble
 * Assertion: Returns a random gdouble equally distributed over the range [0..1).
 *            Returns : A random number.
 * Params Values: void
 * Expected: the return is in [0..1)  
 */
static void test_purpose_8()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gdouble rand_n = g_random_double();
    if ((rand_n < 0) || (rand_n >= 1)){
        TEST_FAIL("g_random_double: failed", NULL);
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_int
 * Params: GRand *rand_
 * Returns: guint32
 * Assertion: Returns the next random guint32 from rand_ equally distributed over the range [0..2^32-1].
 *            rand_ : a GRand.
 *            Returns : A random number.
 * Params Values: GRand *rand_
 * Expected: the return is in [0..2^32-1]  
 */
static void test_purpose_9()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    guint32 rand_n = g_rand_int(grand);
    if ((rand_n < 0) || (rand_n > 4294967294)){
        TEST_FAIL("g_rand_int: failed", NULL);
    }
    g_rand_free(grand);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_random_int
 * Params: void
 * Returns: guint32
 * Assertion: Return a random guint32 equally distributed over the range [0..2^32-1]. 
 *            Returns : A random number.
 * Params Values: void
 * Expected: the return is in [0..2^32-1]  
 */
static void test_purpose_10()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    guint32 rand_n = g_random_int();
    if ((rand_n < 0) || (rand_n > 4294967294)){
        TEST_FAIL("g_rand_int: failed", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_double_range
 * Params: GRand *rand_, gdouble begin, gdouble end
 * Returns: gdouble
 * Assertion: Returns the next random gdouble from rand_ equally distributed over the range [begin..end).
 *            rand_ : a GRand 
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 10.32, 100.87
 * Expected: the return is in [begin..end).  
 */
static void test_purpose_11()
{
    #define BEGIN    10.32
    #define END      100.87
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    g_rand_set_seed(grand, 781221);
    gdouble rand_n = g_rand_double_range(grand, BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_rand_double_range: failed", NULL);
    }
    g_rand_free(grand);
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_double_range
 * Params: GRand *rand_, gdouble begin, gdouble end
 * Returns: gdouble
 * Assertion: Returns the next random gdouble from rand_ equally distributed over the range [begin..end).
 *            rand_ : a GRand 
 *            begin : lower closed bound of the interval.
 *            end : upper open bound of the interval.
 *            Returns : A random number.
 * Params Values: 1.0, 10023
 * Expected: the return is in [begin..end).  
 */
static void test_purpose_12()
{
    #define BEGIN    1.0
    #define END      10023
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    g_rand_set_seed(grand, 781221);
    gdouble rand_n = g_rand_double_range(grand, BEGIN, END);
    if ((rand_n < BEGIN) || (rand_n > (END - 1))){
        TEST_FAIL("g_rand_double_range: failed", NULL);
    }
    g_rand_free(grand);
    #undef BEGIN
    #undef END
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_rand_double
 * Params: GRand *rand_
 * Returns: gdouble
 * Assertion: Returns the next random gdouble from rand_ equally distributed over the range [0..1).
 *            rand_ : a GRand.
 *            Returns : A random number.
 * Params Values: GRand *rand_
 * Expected: the return is in [0..1)  
 */
static void test_purpose_13()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GRand *grand = g_rand_new();
    gdouble rand_n = g_rand_double(grand);
    if ((rand_n < 0) || (rand_n >= 1)){
        TEST_FAIL("g_rand_double: failed", NULL);
    }
    g_rand_free(grand);
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GRandom ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {NULL,0}
};
