/*
 * GdkPixbufImageDataInMemory methods tests
 * Testing GdkPixbufImageDataInMemory methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
void call_back_func(guchar * pixels, gpointer data){
	if(pixels)
		free(pixels);
	return;
};

/*
 * Name: gdk_pixbuf_new
 * Params: GdkColorspace colorspace,gboolean has_alpha,int bits_per_sample,int width,int height
 * Returns: GdkPixbuf*
 * Assertion: Creates a new GdkPixbuf structure and allocates a buffer for it. The buffer has an optimal rowstride. Note that the buffer is not cleared; you will have to fill it completely yourself.
 * Params Values: colorspace : Color space for image.  has_alpha : Whether the image should have transparency information.  bits_per_sample : Number of bits per color sample.  width : Width of image in pixels.  height : Height of image in pixels. 
 * Expected:  A newly-created GdkPixbuf with a reference count of 1, or NULL if not enough memory could be allocated for the image buffer.   
 */
static void test_purpose_1()
{
    #define HAS_ALPHA			TRUE
    #define WEITH				1
    #define HEIGTH				1
    #define EXP_RES				TRUE
    
    GdkPixbuf * pixbuf = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,HAS_ALPHA,8,WEITH,HEIGTH);
    	if(EXP_RES == TRUE){
    		if(pixbuf == NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is NULL",NULL);
    			return;
    		}
    		if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is not a valid pixbuf object!",NULL);
    			return;
    		};
    	}else{
    		if(pixbuf != NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":NULL is expected, but...",NULL);
    			return;
    		}
    	}
    
    if(pixbuf)
    	g_object_unref(pixbuf);
    
    #undef HAS_ALPHA
    #undef WEITH
    #undef HEIGTH
    #undef EXP_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_new
 * Params: GdkColorspace colorspace,gboolean has_alpha,int bits_per_sample,int width,int height
 * Returns: GdkPixbuf*
 * Assertion: Creates a new GdkPixbuf structure and allocates a buffer for it. The buffer has an optimal rowstride. Note that the buffer is not cleared; you will have to fill it completely yourself.
 * Params Values: colorspace : Color space for image.  has_alpha : Whether the image should have transparency information.  bits_per_sample : Number of bits per color sample.  width : Width of image in pixels.  height : Height of image in pixels. 
 * Expected:  A newly-created GdkPixbuf with a reference count of 1, or NULL if not enough memory could be allocated for the image buffer.   
 */
static void test_purpose_2()
{
    #define HAS_ALPHA			TRUE
    #define WEITH				1000
    #define HEIGTH				1000
    #define EXP_RES				TRUE
    
    GdkPixbuf * pixbuf = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,HAS_ALPHA,8,WEITH,HEIGTH);
    	if(EXP_RES == TRUE){
    		if(pixbuf == NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is NULL",NULL);
    			return;
    		}
    		if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is not a valid pixbuf object!",NULL);
    			return;
    		};
    	}else{
    		if(pixbuf != NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":NULL is expected, but...",NULL);
    			return;
    		}
    	}
    
    if(pixbuf)
    	g_object_unref(pixbuf);
    
    #undef HAS_ALPHA
    #undef WEITH
    #undef HEIGTH
    #undef EXP_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_new
 * Params: GdkColorspace colorspace,gboolean has_alpha,int bits_per_sample,int width,int height
 * Returns: GdkPixbuf*
 * Assertion: Creates a new GdkPixbuf structure and allocates a buffer for it. The buffer has an optimal rowstride. Note that the buffer is not cleared; you will have to fill it completely yourself.
 * Params Values: colorspace : Color space for image.  has_alpha : Whether the image should have transparency information.  bits_per_sample : Number of bits per color sample.  width : Width of image in pixels.  height : Height of image in pixels. 
 * Expected:  A newly-created GdkPixbuf with a reference count of 1, or NULL if not enough memory could be allocated for the image buffer.   
 */
static void test_purpose_3()
{
    #define HAS_ALPHA			FALSE
    #define WEITH				1
    #define HEIGTH				1
    #define EXP_RES				TRUE
    
    GdkPixbuf * pixbuf = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,HAS_ALPHA,8,WEITH,HEIGTH);
    	if(EXP_RES == TRUE){
    		if(pixbuf == NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is NULL",NULL);
    			return;
    		}
    		if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is not a valid pixbuf object!",NULL);
    			return;
    		};
    	}else{
    		if(pixbuf != NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":NULL is expected, but...",NULL);
    			return;
    		}
    	}
    
    if(pixbuf)
    	g_object_unref(pixbuf);
    
    #undef HAS_ALPHA
    #undef WEITH
    #undef HEIGTH
    #undef EXP_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_new
 * Params: GdkColorspace colorspace,gboolean has_alpha,int bits_per_sample,int width,int height
 * Returns: GdkPixbuf*
 * Assertion: Creates a new GdkPixbuf structure and allocates a buffer for it. The buffer has an optimal rowstride. Note that the buffer is not cleared; you will have to fill it completely yourself.
 * Params Values: colorspace : Color space for image.  has_alpha : Whether the image should have transparency information.  bits_per_sample : Number of bits per color sample.  width : Width of image in pixels.  height : Height of image in pixels. 
 * Expected:  A newly-created GdkPixbuf with a reference count of 1, or NULL if not enough memory could be allocated for the image buffer.   
 */
static void test_purpose_4()
{
    #define HAS_ALPHA			TRUE
    #define WEITH				100000000
    #define HEIGTH				100000000
    #define EXP_RES				FALSE
    
    GdkPixbuf * pixbuf = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,HAS_ALPHA,8,WEITH,HEIGTH);
    	if(EXP_RES == TRUE){
    		if(pixbuf == NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is NULL",NULL);
    			return;
    		}
    		if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    			TEST_FAIL("\"gdk_pixbuf_new\":pixbuf is not a valid pixbuf object!",NULL);
    			return;
    		};
    	}else{
    		if(pixbuf != NULL){
    			TEST_FAIL("\"gdk_pixbuf_new\":NULL is expected, but...",NULL);
    			return;
    		}
    	}
    
    if(pixbuf)
    	g_object_unref(pixbuf);
    
    #undef HAS_ALPHA
    #undef WEITH
    #undef HEIGTH
    #undef EXP_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_copy
 * Params: const GdkPixbuf *pixbuf
 * Returns: GdkPixbuf*
 * Assertion: Creates a new GdkPixbuf with a copy of the information in the specified pixbuf.
 * Params Values: pixbuf : A pixbuf.   
 * Expected: A newly-created pixbuf with a reference count of 1, or NULL if not enough memory could be allocated  
 */
static void test_purpose_5()
{
    #define EXP_RES 	     TRUE
    
    GdkPixbuf * pixbuf = NULL;
    GdkPixbuf * pixbuf_new = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,10,10);
    
    if(pixbuf == NULL){
    	TEST_FAIL("\"gdk_pixbuf_copy\":pixbuf is NULL",NULL);
    	return;
    }
    if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    	TEST_FAIL("\"gdk_pixbuf_copy\":pixbuf is not a valid pixbuf object!",NULL);
    	return;
    };
    pixbuf_new = gdk_pixbuf_copy(pixbuf);
    if(EXP_RES == TRUE){
    	if(pixbuf_new == NULL){
    		TEST_FAIL("\"gdk_pixbuf_copy\":doesnt match expected result!",NULL);
    		return;
    	}
    	if(GDK_IS_PIXBUF(pixbuf_new) != TRUE){
    		TEST_FAIL("\"gdk_pixbuf_copy\":pixbuf_new is not a valid pixbuf object!",NULL);
    		return;
    	};
    }else{
    	if(pixbuf_new != NULL){
    		TEST_FAIL("\"gdk_pixbuf_copy\":doesnt match expected result!",NULL);
    		return;
    	}
    };
    #undef EXP_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_new_subpixbuf
 * Params: GdkPixbuf *src_pixbuf,int src_x,int src_y,int width,int height
 * Returns: GdkPixbuf*
 * Assertion: Creates a new pixbuf which represents a sub-region of src_pixbuf. The new pixbuf shares its pixels with the original pixbuf, so writing to one affects both. The new pixbuf holds a reference to src_pixbuf, so src_pixbuf will not be finalized until the new pixbuf is finalized.
 * Params Values: src_pixbuf, a GdkPixbuf  src_x, X coord in src_pixbuf  src_y, Y coord in src_pixbuf  width, width of region in src_pixbuf  height, height of region in src_pixbuf 
 * Expected: a new pixbuf  
 */
static void test_purpose_6()
{
    GdkPixbuf * pixbuf_ori = NULL;
    GdkPixbuf * pixbuf_new = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixbuf_ori = gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,10,10);
    if(pixbuf_ori == NULL){
    	TEST_FAIL("gdk_pixbuf_new_subpixbuf: pixbuf_ori is NULL",NULL);
    	return;
    };
    if(GDK_IS_PIXBUF(pixbuf_ori) != TRUE){
    	TEST_FAIL("\"gdk_pixbuf_subpixbuf\":pixbuf_ori is not a valid pixbuf object!",NULL);
    	return;
    };
    pixbuf_new = gdk_pixbuf_new_subpixbuf(pixbuf_ori,1,1,2,2);
    if(pixbuf_new == NULL){
    	TEST_FAIL("gdk_pixbuf_new_subpixbuf: pixbuf_new is NULL",NULL);
    	return;
    };
    if(GDK_IS_PIXBUF(pixbuf_new) != TRUE){
    	TEST_FAIL("\"gdk_pixbuf_subpixbuf\":pixbuf_new is not a valid pixbuf object!",NULL);
    	return;
    };
    if(gdk_pixbuf_get_pixels(pixbuf_new) != gdk_pixbuf_get_pixels(pixbuf_ori)+gdk_pixbuf_get_rowstride(pixbuf_ori)+4){
    	TEST_FAIL("gdk_pixbuf_new_subpixbuf: new pixels is not sub-region of original pixels",NULL);
    	return;
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixbuf_new_from_data
 * Params: const guchar *dat,GdkColorspace colorspace,gboolean has_alpha,int bits_per_sample,int width,int height,int rowstride,GdkPixbufDestroyNotify destroy_fn,gpointer destroy_fn_data
 * Return: GdkPixbuf*
 * Assert: Creates a new GdkPixbuf out of in-memory image data. Currently only RGB images with 8 bits per sample are supported.
 * Expect: A newly-created GdkPixbuf structure with a reference count of 1.   
 */
static void test_purpose_7()
{
    guchar * pixels = NULL;
    GdkPixbuf * pixbuf = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixels = (guchar *)malloc(400);
    pixbuf = gdk_pixbuf_new_from_data(pixels,GDK_COLORSPACE_RGB,TRUE,8,10,10,40,call_back_func,"hello");
    if(pixbuf == NULL){
    	TEST_FAIL("gdk_pixbuf_new_from_data: pixbuf is NULL",NULL);
    	return;
    }
    if(GDK_IS_PIXBUF(pixbuf) != TRUE){
    	TEST_FAIL("gdk_pixbuf_new_from_data: not a valide object of GdkPixbuf",NULL);
    	return;
    }
    if(pixbuf)
    	g_object_unref(pixbuf);
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GdkPixbufImageDataInMemory ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {NULL,0}
};
