/*
 * GdkPixmaps methods tests
 * Testing GdkPixmaps methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;


/*
 * Name: gdk_pixmap_new
 * Params: GdkDrawable *drawable, gint width, gint height, gint depth
 * Returns: GdkPixmap*
 * Assertion: Create a new pixmap with a given size and depth.
 * Params Values: drawable : A GdkDrawable, used to determine default values for the new pixmap. Can be NULL if depth is specified. width : The width of the new pixmap in pixels. height : The height of the new pixmap in pixels. depth : The depth (number of bits per pixel) of the new pixmap. If -1, and drawable is not NULL, the depth of the new pixmap will be equal to that of drawable.
 * Expected: the GdkPixmap  
 */
static void test_purpose_1()
{
    #define WIDTH 1
    #define HEIGHT 1
    #define DEPTH 1
    
    GdkPixmap *pixmap;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixmap = gdk_pixmap_new (NULL, WIDTH, HEIGHT, DEPTH);
    
    if (!pixmap)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_DRAWABLE (pixmap))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    g_object_unref (pixmap);
    
    #undef WIDTH
    #undef HEIGHT
    #undef DEPTH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixmap_new
 * Params: GdkDrawable *drawable, gint width, gint height, gint depth
 * Returns: GdkPixmap*
 * Assertion: Create a new pixmap with a given size and depth.
 * Params Values: drawable : A GdkDrawable, used to determine default values for the new pixmap. Can be NULL if depth is specified. width : The width of the new pixmap in pixels. height : The height of the new pixmap in pixels. depth : The depth (number of bits per pixel) of the new pixmap. If -1, and drawable is not NULL, the depth of the new pixmap will be equal to that of drawable.
 * Expected: the GdkPixmap  
 */
static void test_purpose_2()
{
    #define WIDTH 100
    #define HEIGHT 100
    #define DEPTH 8
    
    GdkPixmap *pixmap;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixmap = gdk_pixmap_new (NULL, WIDTH, HEIGHT, DEPTH);
    
    if (!pixmap)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_DRAWABLE (pixmap))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    g_object_unref (pixmap);
    
    #undef WIDTH
    #undef HEIGHT
    #undef DEPTH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixmap_new
 * Params: GdkDrawable *drawable, gint width, gint height, gint depth
 * Returns: GdkPixmap*
 * Assertion: Create a new pixmap with a given size and depth.
 * Params Values: drawable : A GdkDrawable, used to determine default values for the new pixmap. Can be NULL if depth is specified. width : The width of the new pixmap in pixels. height : The height of the new pixmap in pixels. depth : The depth (number of bits per pixel) of the new pixmap. If -1, and drawable is not NULL, the depth of the new pixmap will be equal to that of drawable.
 * Expected: the GdkPixmap  
 */
static void test_purpose_3()
{
    #define WIDTH 10000
    #define HEIGHT 10000
    #define DEPTH 16
    
    GdkPixmap *pixmap;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixmap = gdk_pixmap_new (NULL, WIDTH, HEIGHT, DEPTH);
    
    if (!pixmap)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_DRAWABLE (pixmap))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    g_object_unref (pixmap);
    
    #undef WIDTH
    #undef HEIGHT
    #undef DEPTH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_pixmap_new
 * Params: GdkDrawable *drawable, gint width, gint height, gint depth
 * Returns: GdkPixmap*
 * Assertion: Create a new pixmap with a given size and depth.
 * Params Values: drawable : A GdkDrawable, used to determine default values for the new pixmap. Can be NULL if depth is specified. width : The width of the new pixmap in pixels. height : The height of the new pixmap in pixels. depth : The depth (number of bits per pixel) of the new pixmap. If -1, and drawable is not NULL, the depth of the new pixmap will be equal to that of drawable.
 * Expected: the GdkPixmap  
 */
static void test_purpose_4()
{
    #define WIDTH 65535
    #define HEIGHT 65535
    #define DEPTH 24
    
    GdkPixmap *pixmap;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    pixmap = gdk_pixmap_new (NULL, WIDTH, HEIGHT, DEPTH);
    
    if (!pixmap)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_DRAWABLE (pixmap))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    g_object_unref (pixmap);
    
    #undef WIDTH
    #undef HEIGHT
    #undef DEPTH
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GdkPixmaps ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {NULL,0}
};
