/*
 * GtkIMContext methods tests
 * Testing GtkIMContext methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
gint global_count;

gboolean
global_surrounding  (GtkIMContext *imcontext,gint arg1,gint arg2,gpointer user_data)
{
    global_count++;
}


/*
 * Name: gtk_im_context_set_client_window
 * Params: GtkIMContext *context, GdkWindow *window
 * Returns: void
 * Assertion: Set the client window for the input context; this is the GdkWindow in which the input appears.
 * Params Values: GtkIMContext *context, GdkWindow *window
 * Expected: void  
 */
static void test_purpose_1()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    #define PARENT NULL
    #define MASK 0
    
    GtkIMContext* context;
    GdkWindow *window;
    GdkWindowAttr attributes;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    attributes.window_type = GDK_WINDOW_TOPLEVEL;
    attributes.title = "";
    
    window = gdk_window_new (PARENT, &attributes, MASK);
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_set_client_window (GTK_IM_CONTEXT (context), window);
    
    #undef CONTEXT
    #undef PARENT
    #undef MASK
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_set_use_preedit
 * Params: GtkIMContext *context, gboolean use_preedit
 * Returns: void
 * Assertion: Sets whether the IM context should use the preedit string to display feedback
 * Params Values: GtkIMContext *context, FALSE
 * Expected: void  
 */
static void test_purpose_2()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    #define USE_PREEDIT FALSE
    
    GtkIMContext* context;
    gboolean use_preedit;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_set_use_preedit (GTK_IM_CONTEXT (context), USE_PREEDIT);
    
    #undef CONTEXT
    #undef USE_PREEDIT
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_set_use_preedit
 * Params: GtkIMContext *context, gboolean use_preedit
 * Returns: void
 * Assertion: Sets whether the IM context should use the preedit string to display feedback
 * Params Values: GtkIMContext *context, TRUE
 * Expected: void  
 */
static void test_purpose_3()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    #define USE_PREEDIT TRUE
    
    GtkIMContext* context;
    gboolean use_preedit;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_set_use_preedit (GTK_IM_CONTEXT (context), USE_PREEDIT);
    
    #undef CONTEXT
    #undef USE_PREEDIT
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_set_cursor_location
 * Params: GtkIMContext *context, GdkRectangle *area
 * Returns: void
 * Assertion: Notify the input method that a change in cursor position has been made
 * Params Values: GtkIMContext *context, GdkRectangle *area
 * Expected: void  
 */
static void test_purpose_4()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    #define X 0
    #define Y 0
    #define WIDTH 10
    #define HEIGHT 10
    
    GtkIMContext* context;
    GdkRectangle rect;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    rect.x = X;
    rect.y = Y;
    rect.width = WIDTH;
    rect.height = HEIGHT;
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_set_cursor_location (GTK_IM_CONTEXT (context), &rect);
    
    #undef CONTEXT
    #undef X
    #undef Y
    #undef WIDTH
    #undef HEIGHT
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_get_preedit_string
 * Params: GtkIMContext *context, gchar **str, PangoAttrList **attrs, gint *cursor_pos
 * Returns: void
 * Assertion: Retrieve the current preedit string for the input context, and a list of attributes to apply to the string. 
 * Params Values: GtkIMContext *context, gchar **str, PangoAttrList **attrs, gint *cursor_pos
 * Expected: void  
 */
static void test_purpose_5()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    
    GtkIMContext* context;
    gchar *str;
    PangoAttrList *attr;
    gint cursor_pos;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_get_preedit_string (GTK_IM_CONTEXT (context), &str, &attr, &cursor_pos);
    
    #undef CONTEXT
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_focus_in
 * Params: GtkIMContext *context
 * Returns: void
 * Assertion: Notify the input method that the widget to which this input context corresponds has lost gained. 
 * Params Values: GtkIMContext* context
 * Expected: void  
 */
static void test_purpose_6()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    
    GtkIMContext* context;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    gtk_im_context_focus_in (GTK_IM_CONTEXT (context));
    
    #undef CONTEXT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_focus_out
 * Params: GtkIMContext *context
 * Returns: void
 * Assertion: Notify the input method that the widget to which this input context corresponds has lost focus. 
 * Params Values: GtkIMContext* context
 * Expected: void  
 */
static void test_purpose_7()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    
    GtkIMContext *context;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    gtk_im_context_focus_out (GTK_IM_CONTEXT (context));
    
    #undef CONTEXT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_reset
 * Params: GtkIMContext *context
 * Returns: void
 * Assertion: Notify the input method that a change such as a change in cursor position has been made. 
 * Params Values: GtkIMContext* context
 * Expected: void  
 */
static void test_purpose_8()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    
    GtkIMContext *context;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    
    gtk_im_context_reset (GTK_IM_CONTEXT (context));
    
    #undef CONTEXT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_set_surrounding 
 * Params: GtkIMContext *context,const gchar *text,gint len,gint cursor_index
 * Returns: void
 * Assertion: Sets surrounding context around the insertion point and preedit string.
 * Params Values: GtkIMContext *context, 1
 * Expected: void
 * 
 * Name: gtk_im_context_get_surrounding 
 * Params: GtkIMContext *context,gchar **text,gint* cursor_index
 * Returns: gboolean
 * Assertion: Retrieves context around the insertion point.
 * Params Values: GtkIMContext *context, 1
 * Expected: void  
 */
static void test_purpose_9()
{
    #define TEXT "Text"
    #define LEN 1
    #define INDEX 1
    
    /*  Gtk0bject *context; */
    GtkIMContext *context;
    gint len;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = gtk_im_multicontext_new ();
    
    if (!context)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_im_context_set_surrounding (GTK_IM_CONTEXT (context), TEXT, LEN,INDEX);
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    
    /*  if (strcmp (text,TEXT)) */
    /*  { */
    /*      TEST_FAIL ("Text is %s, should be %s",text,TEXT); */
    /*  } */
    
    /*  if (len != LEN) */
    /*  { */
    /*      TEST_FAIL ("Lenght is %d, should be %d",len,LEN); */
    /*  } */
    
    
    #undef TEXT
    #undef LEN
    #undef INDEX
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_10()
{
    #define OFFSET 0
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_11()
{
    #define OFFSET 1
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_12()
{
    #define OFFSET 100
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_13()
{
    #define OFFSET G_MAXINT
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_14()
{
    #define OFFSET G_MININT
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_delete_surrounding 
 * Params: GtkIMContext *context,gint offset,gint n_chars
 * Returns: gboolean
 * Assertion: Asks the widget that the input context is attached to to delete characters around the cursor position by emitting the GtkIMContext::delete_surrounding signal.
 * Params Values: GtkIMContext* context, gint, gint
 * Expected: gboolean  
 */
static void test_purpose_15()
{
    #define OFFSET -1
    #define CHARS 0
    
    GtkIMContext *context;
    gint index;
    gchar *text;
    gboolean res;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    
    context = gtk_im_multicontext_new ();
    
    res = gtk_im_context_get_surrounding (GTK_IM_CONTEXT(context),&text ,&index);
    
    g_signal_connect (G_OBJECT (context), "delete-surrounding", GTK_SIGNAL_FUNC (global_surrounding), NULL);
    gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (context),OFFSET,CHARS); 
    if (global_count == 0)
    {
        TEST_FAIL ("Signal 'delete-surrounding' is not emitted",NULL);
    }
    
    
    #undef OFFSET
    #undef CHARS
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_filter_keypress 
 * Params: GtkIMContext *context, GdkEventKey *event
 * Returns: gboolean
 * Assertion: Allow an input method to internally handle a key press event.
 * Params Values: GtkIMContext* context, GdkEventKey* event
 * Expected: gboolean  
 */
static void test_purpose_16()
{
    #define CONTEXT gtk_im_multicontext_new ()    
    #define KEY_TYPE GDK_KEY_RELEASE
    
    GtkIMContext *context;
    GdkEventKey* key = (GdkEventKey*)gdk_event_new(KEY_TYPE);
    gboolean res = FALSE;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    key->type = KEY_TYPE;
    
    res = gtk_im_context_filter_keypress (GTK_IM_CONTEXT (context), key); 
    
    #undef CONTEXT
    #undef KEY_TYPE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_im_context_filter_keypress 
 * Params: GtkIMContext *context, GdkEventKey *event
 * Returns: gboolean
 * Assertion: Allow an input method to internally handle a key press event.
 * Params Values: GtkIMContext* context, GdkEventKey* event
 * Expected: gboolean  
 */
static void test_purpose_17()
{
    #define CONTEXT gtk_im_context_simple_new ()    
    #define KEY_TYPE GDK_KEY_RELEASE
    
    GtkIMContext *context;
    GdkEventKey* key = (GdkEventKey*)gdk_event_new(KEY_TYPE);
    gboolean res = FALSE;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    context = CONTEXT;
    key->type = KEY_TYPE;
    
    res = gtk_im_context_filter_keypress (GTK_IM_CONTEXT (context), key); 
    
    #undef CONTEXT
    #undef KEY_TYPE
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkIMContext ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {NULL,0}
};
