/*
 * GtkTreeSortable methods tests
 * Testing GtkTreeSortable methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

static gint global_result;

void  
global_inc_result (GtkWidget *widget, gpointer data)
{
    global_result++;
}  

struct ListSort
{
  gint number1;
  gint number2;
};

struct ListSort data[] =
{
  { 3, 4 },
  { 2, 6 },
  { 0, 49 },
  { 856856, 965 },
  { 1, 34663 },
};

struct ListSort child_data[] =
{
  { 10, 149 },
  { 12, 13 },
  { 1856856, 1965 },
  { 11, 15 },
  { 14, 14 },
};

const gint NUM_TREE_ELEMS = 5;

gint 
global_compare (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
{
    GValue val_a = {0,};
    GValue val_b = {0,};
    guint uint_val_a;
    guint uint_val_b;

    gtk_tree_model_get_value (model, a, 0, &val_a);
    gtk_tree_model_get_value (model, b, 0, &val_b);
    uint_val_a = g_value_get_uint (&val_a);
    uint_val_b = g_value_get_uint (&val_b);
    if (uint_val_a > uint_val_b)
    {
        return (1);
    } else {
        return (0);
    }
}

/*
 * Name:  gtk_tree_sortable_sort_column_changed
 * Params: GtkTreeSortable *sortable
 * Returns: void
 * Assertion: Emits a GtkTreeSortable::sort_column_changed signal on.
 * Params Values: GtkTreeSortable *sortable
 * Expected: void  
 */
static void test_purpose_1()
{
    #define OBJECT gtk_tree_store_new (1, G_TYPE_UINT)
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (object), "sort-column-changed", GTK_SIGNAL_FUNC (global_inc_result), NULL);
    
    gtk_tree_sortable_sort_column_changed (GTK_TREE_SORTABLE (object));
    if (!global_result)
    {
        TEST_FAIL ("'global_result' is 0", NULL);
    }
    
    #undef OBJECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_sort_column_changed
 * Params: GtkTreeSortable *sortable
 * Returns: void
 * Assertion: Emits a GtkTreeSortable::sort_column_changed signal on.
 * Params Values: GtkTreeSortable *sortable
 * Expected: void  
 */
static void test_purpose_2()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_INT, G_TYPE_STRING)
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (object), "sort-column-changed", GTK_SIGNAL_FUNC (global_inc_result), NULL);
    
    gtk_tree_sortable_sort_column_changed (GTK_TREE_SORTABLE (object));
    if (!global_result)
    {
        TEST_FAIL ("'global_result' is 0", NULL);
    }
    
    #undef OBJECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: Fills in sort_column_id and order with the current sort column and the order, if applicable. 
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: If the sort column is not set, then FALSE is returned, and the values in sort_column_id and order are unchanged.
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: Sets the current sort column to be sort_column_id.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_3()
{
    #define OBJECT gtk_tree_store_new (1, G_TYPE_UINT)
    #define COLUMN 0
    #define ORDER GTK_SORT_ASCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), COLUMN, ORDER);
    
    if (!gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (object), &sort_column_id, &order))
    {
        TEST_FAIL ("'gtk_tree_sortable_get_sort_column_id' returns FALSE", NULL);
        return;
    }
    
    if (sort_column_id != COLUMN)
    {
        TEST_FAIL ("sort_column_id != COLUMN", NULL);
    }
    
    if (order != ORDER)
    {
        TEST_FAIL ("order != ORDER", NULL);
    }
    
    #undef OBJECT
    #undef COLUMN 
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: Fills in sort_column_id and order with the current sort column and the order, if applicable. 
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: If the sort column is not set, then FALSE is returned, and the values in sort_column_id and order are unchanged.
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: Sets the current sort column to be sort_column_id.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_4()
{
    #define OBJECT gtk_tree_store_new (1, G_TYPE_UINT)
    #define COLUMN 0
    #define ORDER GTK_SORT_DESCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), COLUMN, ORDER);
    
    if (!gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (object), &sort_column_id, &order))
    {
        TEST_FAIL ("'gtk_tree_sortable_get_sort_column_id' returns FALSE", NULL);
        return;
    }
    
    if (sort_column_id != COLUMN)
    {
        TEST_FAIL ("sort_column_id != COLUMN", NULL);
    }
    
    if (order != ORDER)
    {
        TEST_FAIL ("order != ORDER", NULL);
    }
    
    #undef OBJECT
    #undef COLUMN 
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: Fills in sort_column_id and order with the current sort column and the order, if applicable. 
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: If the sort column is not set, then FALSE is returned, and the values in sort_column_id and order are unchanged.
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: Sets the current sort column to be sort_column_id.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_5()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_INT, G_TYPE_STRING)
    #define COLUMN 0
    #define ORDER GTK_SORT_ASCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), COLUMN, ORDER);
    
    if (!gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (object), &sort_column_id, &order))
    {
        TEST_FAIL ("'gtk_tree_sortable_get_sort_column_id' returns FALSE", NULL);
        return;
    }
    
    if (sort_column_id != COLUMN)
    {
        TEST_FAIL ("sort_column_id != COLUMN", NULL);
    }
    
    if (order != ORDER)
    {
        TEST_FAIL ("order != ORDER", NULL);
    }
    
    #undef OBJECT
    #undef COLUMN 
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: Fills in sort_column_id and order with the current sort column and the order, if applicable. 
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_get_sort_column_id
 * Params: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Returns: void
 * Assertion: If the sort column is not set, then FALSE is returned, and the values in sort_column_id and order are unchanged.
 * Params Values: GtkTreeSortable *sortable, gint *sort_column_id, GtkSortType *order
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: Sets the current sort column to be sort_column_id.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_6()
{
    #define OBJECT gtk_tree_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING)
    #define COLUMN 1
    #define ORDER GTK_SORT_DESCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), COLUMN, ORDER);
    
    if (!gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE (object), &sort_column_id, &order))
    {
        TEST_FAIL ("'gtk_tree_sortable_get_sort_column_id' returns FALSE", NULL);
        return;
    }
    
    if (sort_column_id != COLUMN)
    {
        TEST_FAIL ("sort_column_id != COLUMN", NULL);
    }
    
    if (order != ORDER)
    {
        TEST_FAIL ("order != ORDER", NULL);
    }
    
    #undef OBJECT
    #undef COLUMN 
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: If sort_column_id is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the default sort function will be used, if it is set.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_7()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_INT, G_TYPE_STRING)
    #define ORDER GTK_SORT_ASCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (object), "sort-column-changed", GTK_SIGNAL_FUNC (global_inc_result), NULL);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, ORDER);
    
    if (!global_result)
    {
        TEST_FAIL ("default sort function fails", NULL);
        return;
    }
    
    #undef OBJECT
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: If sort_column_id is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the default sort function will be used, if it is set.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_8()
{
    #define OBJECT gtk_tree_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING)
    #define ORDER GTK_SORT_DESCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (object), "sort-column-changed", GTK_SIGNAL_FUNC (global_inc_result), NULL);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, ORDER);
    
    if (!global_result)
    {
        TEST_FAIL ("default sort function fails", NULL);
        return;
    }
    
    #undef OBJECT
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_sort_column_id
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Returns: void
 * Assertion: If sort_column_id is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the default sort function will be used, if it is set.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkSortType order
 * Expected: void  
 */
static void test_purpose_9()
{
    #define OBJECT gtk_tree_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING)
    #define ORDER GTK_SORT_DESCENDING
    
    GtkTreeStore* object;
    gint sort_column_id;
    GtkSortType order;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (object), "sort-column-changed", GTK_SIGNAL_FUNC (global_inc_result), NULL);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (object), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, ORDER);
    
    if (!global_result)
    {
        TEST_FAIL ("default sort function fails", NULL);
        return;
    }
    
    #undef OBJECT
    #undef ORDER 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_sort_func
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets the comparison function used when sorting to be sort_func. 
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_func
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is the same as sort_column_id, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_10()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT)
    #define COLUMN 1
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (object), COLUMN, (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    
    #undef OBJECT
    #undef COLUMN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_sort_func
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets the comparison function used when sorting to be sort_func. 
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_sort_func
 * Params: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is the same as sort_column_id, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, gint sort_column_id, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_11()
{
    #define OBJECT gtk_tree_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING)
    #define COLUMN 2
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (object), COLUMN, (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    
    #undef OBJECT
    #undef COLUMN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets the default comparison function used when sorting to be sort_func. 
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion:  If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_12()
{
    #define OBJECT gtk_tree_store_new (1, G_TYPE_UINT)
    #define FUNCTION (GtkTreeIterCompareFunc) global_compare
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), FUNCTION, NULL, NULL);
    if (!gtk_tree_sortable_has_default_sort_func (GTK_TREE_SORTABLE (object)))
    {
        if (FUNCTION != NULL)
        {
            TEST_FAIL ("tree sortable should have default sort function", NULL);
            return;
        }
    }
    
    #undef OBJECT
    #undef FUNCTION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets the default comparison function used when sorting to be sort_func. 
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion:  If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_13()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_INT, G_TYPE_STRING)
    #define FUNCTION (GtkTreeIterCompareFunc) global_compare
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), FUNCTION, NULL, NULL);
    if (!gtk_tree_sortable_has_default_sort_func (GTK_TREE_SORTABLE (object)))
    {
        if (FUNCTION != NULL)
        {
            TEST_FAIL ("tree sortable should have default sort function", NULL);
            return;
        }
    }
    
    #undef OBJECT
    #undef FUNCTION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets the default comparison function used when sorting to be sort_func. 
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion:  If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_14()
{
    #define OBJECT gtk_tree_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING)
    #define FUNCTION NULL
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), FUNCTION, NULL, NULL);
    if (!gtk_tree_sortable_has_default_sort_func (GTK_TREE_SORTABLE (object)))
    {
        if (FUNCTION != NULL)
        {
            TEST_FAIL ("tree sortable should have default sort function", NULL);
            return;
        }
    }
    
    #undef OBJECT
    #undef FUNCTION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_15()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_16()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:2"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:2"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:1"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:3"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_17()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, then the model will sort.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_18()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), &global_compare, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * <FUNCTION>
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: This means that once the model has been sorted, it can't go back to the default state.'
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: In this case, when the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, the model will be unsorted.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_19()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), NULL, NULL, NULL);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * <FUNCTION>
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: This means that once the model has been sorted, it can't go back to the default state.'
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: In this case, when the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, the model will be unsorted.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_20()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:2"
    #define CORRECT_CONVERTED_PATH_STR_4 "0:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), NULL, NULL, NULL);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * <FUNCTION>
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: This means that once the model has been sorted, it can't go back to the default state.'
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: In this case, when the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, the model will be unsorted.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_21()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:3"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), NULL, NULL, NULL);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * <FUNCTION>
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If sort_func is NULL, then there will be no default comparison function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: This means that once the model has been sorted, it can't go back to the default state.'
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name:  gtk_tree_sortable_set_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: In this case, when the current sort column id of sortable is GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, the model will be unsorted.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_22()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (smodel), NULL, NULL, NULL);
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_has_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: gboolean
 * Assertion: Returns TRUE if the model has a default sort function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: TRUE  
 */
static void test_purpose_23()
{
    #define OBJECT gtk_tree_store_new (1, G_TYPE_UINT)
    #define HAS TRUE
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    if (HAS)
    {
        gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    }
    
    if (gtk_tree_sortable_has_default_sort_func (GTK_TREE_SORTABLE (object)) != HAS)
    {
        TEST_FAIL ("tree sortable should have default sort function", NULL);
    }
    
    #undef OBJECT
    #undef HAS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_sortable_has_default_sort_func
 * Params: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Returns: gboolean
 * Assertion: Returns TRUE if the model has a default sort function.
 * Params Values: GtkTreeSortable *sortable, GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy
 * Expected: FALSE  
 */
static void test_purpose_24()
{
    #define OBJECT gtk_tree_store_new (2, G_TYPE_INT, G_TYPE_STRING)
    #define HAS FALSE
    
    GtkTreeStore* object;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    object = OBJECT;
    if (!object)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_TREE_SORTABLE (object))
    {
        TEST_FAIL ("'object' is not container", NULL); 
        return;
    }
    
    if (HAS)
    {
        gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (object), (GtkTreeIterCompareFunc) global_compare, NULL, NULL);
    }
    
    if (gtk_tree_sortable_has_default_sort_func (GTK_TREE_SORTABLE (object)) != HAS)
    {
        TEST_FAIL ("tree sortable should have default sort function", NULL);
    }
    
    #undef OBJECT
    #undef HAS
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkTreeSortable ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {NULL,0}
};
