/*
 * GtkViewport methods tests
 * Testing GtkViewport methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name:  gtk_viewport_new
 * Params: GtkAdjustment *hadjustment, GtkAdjustment *vadjustment,
 * Returns: GtkWidget*
 * Assertion: Creates a new GtkViewport with the given adjustments.
 * Params Values: gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) , gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
 * Expected: GtkViewport*  
 */
static void test_purpose_1()
{
    #define    HADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    #define    VADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* hadj;
    GtkAdjustment* vadj;
    GtkAdjustment* ha;
    GtkAdjustment* va;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    hadj = GTK_ADJUSTMENT (HADJ);
    vadj = GTK_ADJUSTMENT (VADJ);
    widget = gtk_viewport_new (hadj, vadj);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_get(GTK_OBJECT(widget), "hadjustment", &ha, NULL);
    if (hadj != ha)
    {
        TEST_FAIL("hadjustment fails", NULL);
    }
    
    g_object_get(GTK_OBJECT(widget), "vadjustment", &va, NULL);
    if (vadj != va)
    {
        TEST_FAIL("vadjustment fails", NULL);
    }
    
    /*  gtk_object_destroy (GTK_OBJECT (ha)); */
    /*  gtk_object_destroy (GTK_OBJECT (va)); */
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  HADJ
    #undef  VADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_new
 * Params: GtkAdjustment *hadjustment, GtkAdjustment *vadjustment,
 * Returns: GtkWidget*
 * Assertion: Creates a new GtkViewport with the given adjustments.
 * Params Values: gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) , gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
 * Expected: GtkViewport*  
 */
static void test_purpose_2()
{
    #define    HADJ gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) 
    #define    VADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* hadj;
    GtkAdjustment* vadj;
    GtkAdjustment* ha;
    GtkAdjustment* va;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    hadj = GTK_ADJUSTMENT (HADJ);
    vadj = GTK_ADJUSTMENT (VADJ);
    widget = gtk_viewport_new (hadj, vadj);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_get(GTK_OBJECT(widget), "hadjustment", &ha, NULL);
    if (hadj != ha)
    {
        TEST_FAIL("hadjustment fails", NULL);
    }
    
    g_object_get(GTK_OBJECT(widget), "vadjustment", &va, NULL);
    if (vadj != va)
    {
        TEST_FAIL("vadjustment fails", NULL);
    }
    
    /*  gtk_object_destroy (GTK_OBJECT (ha)); */
    /*  gtk_object_destroy (GTK_OBJECT (va)); */
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  HADJ
    #undef  VADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_hadjustment
 * Params: GtkViewport *viewport, GtkAdjustment *adjustment
 * Returns: void
 * Assertion: Sets the horizontal adjustment of the viewport.
 * Params Values: GtkViewport *viewport, GtkAdjustment *adjustment
 * Expected: void  
 */
static void test_purpose_3()
{
    #define    ADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* ad;
    GtkAdjustment* adj;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_hadjustment (GTK_VIEWPORT (widget), adj);
    g_object_get(GTK_OBJECT(widget), "hadjustment", &ad, NULL);
    
    if (adj != ad)
    {
        TEST_FAIL("set hadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_hadjustment
 * Params: GtkViewport *viewport, GtkAdjustment *adjustment
 * Returns: void
 * Assertion: Sets the horizontal adjustment of the viewport.
 * Params Values: GtkViewport *viewport, GtkAdjustment *adjustment
 * Expected: void  
 */
static void test_purpose_4()
{
    #define    ADJ gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) 
    
    GtkWidget* widget;
    GtkAdjustment* ad;
    GtkAdjustment* adj;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_hadjustment (GTK_VIEWPORT (widget), adj);
    g_object_get(GTK_OBJECT(widget), "hadjustment", &ad, NULL);
    
    if (adj != ad)
    {
        TEST_FAIL("set hadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_vadjustment
 * Params: GtkViewport *viewport, GtkAdjustment *adjustment
 * Returns: void
 * Assertion: Sets the vertical adjustment of the viewport.
 * Params Values: GtkViewport *viewport, GtkAdjustment *adjustment
 * Expected: void  
 */
static void test_purpose_5()
{
    #define    ADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* ad;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_vadjustment (GTK_VIEWPORT (widget), adj);
    g_object_get(GTK_OBJECT(widget), "vadjustment", &ad, NULL);
    if (adj != ad)
    {
        TEST_FAIL("set vadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_vadjustment
 * Params: GtkViewport *viewport, GtkAdjustment *adjustment
 * Returns: void
 * Assertion: Sets the vertical adjustment of the viewport.
 * Params Values: GtkViewport *viewport, GtkAdjustment *adjustment
 * Expected: void  
 */
static void test_purpose_6()
{
    #define    ADJ gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) 
    
    GtkWidget* widget;
    GtkAdjustment* ad;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_vadjustment (GTK_VIEWPORT (widget), adj);
    g_object_get(GTK_OBJECT(widget), "vadjustment", &ad, NULL);
    if (adj != ad)
    {
        TEST_FAIL("set vadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_hadjustment
 * Params: GtkViewport *viewport
 * Returns: GtkAdjustment*
 * Assertion: Returns the horizontal adjustment of the viewport.
 * Params Values: GtkViewport *viewport
 * Expected: GtkAdjustment*  
 */
static void test_purpose_7()
{
    #define    ADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "hadjustment", adj, NULL);
    if (adj != gtk_viewport_get_hadjustment (GTK_VIEWPORT (widget)))
    {
        TEST_FAIL("get hadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_hadjustment
 * Params: GtkViewport *viewport
 * Returns: GtkAdjustment*
 * Assertion: Returns the horizontal adjustment of the viewport.
 * Params Values: GtkViewport *viewport
 * Expected: GtkAdjustment*  
 */
static void test_purpose_8()
{
    #define    ADJ gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) 
    
    GtkWidget* widget;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "hadjustment", adj, NULL);
    if (adj != gtk_viewport_get_hadjustment (GTK_VIEWPORT (widget)))
    {
        TEST_FAIL("get hadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_vadjustment
 * Params: GtkViewport *viewport
 * Returns: GtkAdjustment*
 * Assertion: Returns the vertical adjustment of the viewport.
 * Params Values: GtkViewport *viewport
 * Expected: GtkAdjustment*  
 */
static void test_purpose_9()
{
    #define    ADJ gtk_adjustment_new (10.0, 1.0, 100.0, 5.0, 20.0, 50.0) 
    
    GtkWidget* widget;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "vadjustment", adj, NULL);
    if (adj != gtk_viewport_get_vadjustment (GTK_VIEWPORT (widget)))
    {
        TEST_FAIL("get vadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_vadjustment
 * Params: GtkViewport *viewport
 * Returns: GtkAdjustment*
 * Assertion: Returns the vertical adjustment of the viewport.
 * Params Values: GtkViewport *viewport
 * Expected: GtkAdjustment*  
 */
static void test_purpose_10()
{
    #define    ADJ gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 5.0, 10.0) 
    
    GtkWidget* widget;
    GtkAdjustment* adj;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    adj = GTK_ADJUSTMENT (ADJ);
    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "vadjustment", adj, NULL);
    if (adj != gtk_viewport_get_vadjustment (GTK_VIEWPORT (widget)))
    {
        TEST_FAIL("get vadjustment fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  ADJ
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_shadow_type
 * Params: GtkViewport *viewport, GtkShadowType type
 * Returns: void
 * Assertion: Sets the shadow type of the viewport.
 * Params Values: GtkViewport *viewport, GTK_SHADOW_NONE
 * Expected: void  
 */
static void test_purpose_11()
{
    #define    SHADOW GTK_SHADOW_NONE
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), SHADOW);
    g_object_get(GTK_OBJECT(widget), "shadow-type", &shadow_type, NULL);
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_shadow_type
 * Params: GtkViewport *viewport, GtkShadowType type
 * Returns: void
 * Assertion: Sets the shadow type of the viewport.
 * Params Values: GtkViewport *viewport, GTK_SHADOW_IN
 * Expected: void  
 */
static void test_purpose_12()
{
    #define    SHADOW GTK_SHADOW_IN
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), SHADOW);
    g_object_get(GTK_OBJECT(widget), "shadow-type", &shadow_type, NULL);
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_shadow_type
 * Params: GtkViewport *viewport, GtkShadowType type
 * Returns: void
 * Assertion: Sets the shadow type of the viewport.
 * Params Values: GtkViewport *viewport, GTK_SHADOW_OUT
 * Expected: void  
 */
static void test_purpose_13()
{
    #define    SHADOW GTK_SHADOW_OUT
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), SHADOW);
    g_object_get(GTK_OBJECT(widget), "shadow-type", &shadow_type, NULL);
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_shadow_type
 * Params: GtkViewport *viewport, GtkShadowType type
 * Returns: void
 * Assertion: Sets the shadow type of the viewport.
 * Params Values: GtkViewport *viewport, GTK_SHADOW_ETCHED_IN
 * Expected: void  
 */
static void test_purpose_14()
{
    #define    SHADOW GTK_SHADOW_ETCHED_IN
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), SHADOW);
    g_object_get(GTK_OBJECT(widget), "shadow-type", &shadow_type, NULL);
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_set_shadow_type
 * Params: GtkViewport *viewport, GtkShadowType type
 * Returns: void
 * Assertion: Sets the shadow type of the viewport.
 * Params Values: GtkViewport *viewport, GTK_SHADOW_ETCHED_OUT
 * Expected: void  
 */
static void test_purpose_15()
{
    #define    SHADOW GTK_SHADOW_ETCHED_OUT
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), SHADOW);
    g_object_get(GTK_OBJECT(widget), "shadow-type", &shadow_type, NULL);
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_shadow_type
 * Params: GtkViewport *viewport
 * Returns: GtkShadowType type
 * Assertion: Gets the shadow type of the GtkViewport.
 * Params Values: GtkViewport *viewport
 * Expected: GTK_SHADOW_NONE  
 */
static void test_purpose_16()
{
    #define    SHADOW GTK_SHADOW_NONE
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "shadow-type", SHADOW, NULL);
    
    shadow_type = gtk_viewport_get_shadow_type (GTK_VIEWPORT (widget));
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_shadow_type
 * Params: GtkViewport *viewport
 * Returns: GtkShadowType type
 * Assertion: Gets the shadow type of the GtkViewport.
 * Params Values: GtkViewport *viewport
 * Expected: GTK_SHADOW_IN  
 */
static void test_purpose_17()
{
    #define    SHADOW GTK_SHADOW_IN
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "shadow-type", SHADOW, NULL);
    
    shadow_type = gtk_viewport_get_shadow_type (GTK_VIEWPORT (widget));
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_shadow_type
 * Params: GtkViewport *viewport
 * Returns: GtkShadowType type
 * Assertion: Gets the shadow type of the GtkViewport.
 * Params Values: GtkViewport *viewport
 * Expected: GTK_SHADOW_OUT  
 */
static void test_purpose_18()
{
    #define    SHADOW GTK_SHADOW_OUT
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "shadow-type", SHADOW, NULL);
    
    shadow_type = gtk_viewport_get_shadow_type (GTK_VIEWPORT (widget));
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_shadow_type
 * Params: GtkViewport *viewport
 * Returns: GtkShadowType type
 * Assertion: Gets the shadow type of the GtkViewport.
 * Params Values: GtkViewport *viewport
 * Expected: GTK_SHADOW_ETCHED_IN  
 */
static void test_purpose_19()
{
    #define    SHADOW GTK_SHADOW_ETCHED_IN
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "shadow-type", SHADOW, NULL);
    
    shadow_type = gtk_viewport_get_shadow_type (GTK_VIEWPORT (widget));
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_viewport_get_shadow_type
 * Params: GtkViewport *viewport
 * Returns: GtkShadowType type
 * Assertion: Gets the shadow type of the GtkViewport.
 * Params Values: GtkViewport *viewport
 * Expected: GTK_SHADOW_ETCHED_OUT  
 */
static void test_purpose_20()
{
    #define    SHADOW GTK_SHADOW_ETCHED_OUT
    
    GtkWidget* widget;
    GtkShadowType shadow_type;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_viewport_new (GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)),
                               GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 2.0, 5.0, 5.0, 20.0, 50.0)));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    } 
    
    if (!GTK_IS_VIEWPORT (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    g_object_set(GTK_OBJECT(widget), "shadow-type", SHADOW, NULL);
    
    shadow_type = gtk_viewport_get_shadow_type (GTK_VIEWPORT (widget));
    if (shadow_type != SHADOW)
    {
        TEST_FAIL("shadow type fails", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef  SHADOW
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkViewport ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {NULL,0}
};
