/*
 * Main_loop_and_Events methods tests
 * Testing Main_loop_and_Events methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "librobot.h"

static gint global_count;
static gint global_result;
static gint global_snooper_count;
static GdkEvent* global_event;

void
global_signal ()
{
    global_result = 1;
}  

void
global_set_event ()
{
    global_event = gtk_get_current_event ();
}  

gint 
global_set_result (GtkWidget *widget, gpointer data)
{
    global_result =  gtk_main_level ();
    global_count++;
    return 0;
}  

gint 
global_set_result_1 (GtkWidget *widget, gpointer data)
{
    global_result++;
    return 0;
}  

gint 
global_set_result_2 (GtkWidget *widget, gpointer data)
{
    global_count++;
    return 0;
}  

gint 
global_timeout (GtkWidget *widget, gpointer data)
{
    global_result += 1;
    if (global_result > 2)
    {
        gtk_main_quit ();
        return 0;
    }
    return 1;
}  


void
global_input (gpointer data, gint source, GdkInputCondition condition)
{
    global_result += 1;
    gtk_main_quit ();
}


gint
global_snooper  (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    global_snooper_count += 1;
    gtk_main_quit ();
}


void
global_snooper_key  (GtkWidget *widget, gpointer data)
{
    robot_key_press_event (widget, GDK_A, 0);
}


gint  
global_init (gpointer data)
{
    global_count++;
    gtk_main_quit ();
    return 0;
}  


void  
global_main_level (gpointer data)
{
    if (gtk_main_level () <= 100)
    {
        gtk_init_add ((GtkFunction)global_main_level, NULL);
        gtk_main ();
    
    }
    gtk_main_quit ();
}  



/*
 * Name: gtk_events_pending
 * Params: void
 * Returns: gint
 * Assertion: Checks if any events are pending.
 * Params Values: void
 * Expected: gint  
 */
static void test_purpose_1()
{
    #define PENDING TRUE
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    if (PENDING)
    {
        widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
        gtk_widget_show_all (widget);
    }
    
    if (gtk_events_pending () != PENDING)
    {
        TEST_FAIL ("'gtk_events_pending' fails: PENDING is %d", PENDING); 
        return;
    }
    
    if (PENDING)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef PENDING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_set_locale
 * Params: void
 * Returns: gchar*
 * Assertion: Initializes internationalization support for GTK+.
 * Params Values: void
 * Expected: gchar*  
 */
static void test_purpose_2()
{
    gchar* locale;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    locale = gtk_set_locale ();
    if (!locale)
    {
        TEST_FAIL ("'gtk_set_locale' returns NULL", NULL); 
        return;
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_disable_setlocale
 * Params: void
 * Returns: void
 * Assertion: Prevents gtk_init() and gtk_init_check() from automatically calling setlocale (LC_ALL, "").
 * Params Values: void
 * Expected: void  
 */
static void test_purpose_3()
{
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gtk_disable_setlocale ();
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_default_language
 * Params: void
 * Returns: PangoLanguage*
 * Assertion: Returns the ISO language code for the default language currently in effect.
 * Params Values: void
 * Expected:  PangoLanguage*  
 */
static void test_purpose_4()
{
    PangoLanguage* lang;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    lang = gtk_get_default_language ();
    if (!lang)
    {
        TEST_FAIL ("'gtk_get_default_language' returns NULL", NULL); 
        return;
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_true
 * Params: void
 * Returns: gboolean
 * Assertion: All this function does it to return TRUE.
 * Params Values: void
 * Expected: gboolean  
 */
static void test_purpose_5()
{
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (!gtk_true ())
    {
        TEST_FAIL ("'gtk_true' returns FALSE", NULL); 
        return;
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_false
 * Params: void
 * Returns: gboolean
 * Assertion: Analogical to gtk_true() this function does nothing but always returns FALSE.
 * Params Values: void
 * Expected: gboolean  
 */
static void test_purpose_6()
{
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (gtk_false ())
    {
        TEST_FAIL ("'gtk_false' returns TRUE", NULL); 
        return;
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Makes widget the current grabbed widget.
 * Params Values: gtk_entry_new ()
 * Expected: void
 * 
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: This means that interaction with other widgets in the same application is blocked and mouse as well as keyboard events are delivered to this widget.
 * Params Values: gtk_entry_new ()
 * Expected: void  
 */
static void test_purpose_7()
{
    #define WIDGET gtk_entry_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_grab_add (widget);
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "key-press-event", G_CALLBACK (global_set_result), NULL);
    robot_key_press_event (widget, GDK_Return, 0);
    if (!global_count)
    {
        TEST_FAIL ("'key-press-event' not catched", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "button-press-event", G_CALLBACK (global_set_result), NULL);
    robot_button_press_event (widget, 1, 1, 1, 0);
    if (!global_count)
    {
        TEST_FAIL ("'button-press-event' not catched", NULL); 
        return;
    }
    
    gtk_grab_remove (widget);
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Makes widget the current grabbed widget.
 * Params Values: gtk_button_new ()
 * Expected: void
 * 
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: This means that interaction with other widgets in the same application is blocked and mouse as well as keyboard events are delivered to this widget.
 * Params Values: gtk_button_new ()
 * Expected: void  
 */
static void test_purpose_8()
{
    #define WIDGET gtk_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_grab_add (widget);
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "key-press-event", G_CALLBACK (global_set_result), NULL);
    robot_key_press_event (widget, GDK_Return, 0);
    if (!global_count)
    {
        TEST_FAIL ("'key-press-event' not catched", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "button-press-event", G_CALLBACK (global_set_result), NULL);
    robot_button_press_event (widget, 1, 1, 1, 0);
    if (!global_count)
    {
        TEST_FAIL ("'button-press-event' not catched", NULL); 
        return;
    }
    
    gtk_grab_remove (widget);
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Makes widget the current grabbed widget.
 * Params Values: gtk_check_button_new ()
 * Expected: void
 * 
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: This means that interaction with other widgets in the same application is blocked and mouse as well as keyboard events are delivered to this widget.
 * Params Values: gtk_check_button_new ()
 * Expected: void  
 */
static void test_purpose_9()
{
    #define WIDGET gtk_check_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_grab_add (widget);
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "key-press-event", G_CALLBACK (global_set_result), NULL);
    robot_key_press_event (widget, GDK_Return, 0);
    if (!global_count)
    {
        TEST_FAIL ("'key-press-event' not catched", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "button-press-event", G_CALLBACK (global_set_result), NULL);
    robot_button_press_event (widget, 1, 1, 1, 0);
    if (!global_count)
    {
        TEST_FAIL ("'button-press-event' not catched", NULL); 
        return;
    }
    
    gtk_grab_remove (widget);
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Makes widget the current grabbed widget.
 * Params Values: gtk_hscale_new_with_range (0, 10, 1)
 * Expected: void
 * 
 * Name: gtk_grab_add
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: This means that interaction with other widgets in the same application is blocked and mouse as well as keyboard events are delivered to this widget.
 * Params Values: gtk_hscale_new_with_range (0, 10, 1)
 * Expected: void  
 */
static void test_purpose_10()
{
    #define WIDGET gtk_hscale_new_with_range (0, 10, 1)
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_grab_add (widget);
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "key-press-event", G_CALLBACK (global_set_result), NULL);
    robot_key_press_event (widget, GDK_Return, 0);
    if (!global_count)
    {
        TEST_FAIL ("'key-press-event' not catched", NULL); 
        return;
    }
    
    global_count = 0;
    g_signal_connect (G_OBJECT (widget), "button-press-event", G_CALLBACK (global_set_result), NULL);
    robot_button_press_event (widget, 1, 1, 1, 0);
    if (!global_count)
    {
        TEST_FAIL ("'button-press-event' not catched", NULL); 
        return;
    }
    
    gtk_grab_remove (widget);
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_remove
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Removes the grab from the given widget.
 * Params Values: gtk_entry_new ()
 * Expected: void  
 */
static void test_purpose_11()
{
    #define WIDGET gtk_entry_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    gtk_grab_add (widget);
    gtk_grab_remove (widget);
    
    if (gtk_grab_get_current () != NULL)
    {
        TEST_FAIL ("current grab should be NULL", NULL); 
        return;
    }
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_remove
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Removes the grab from the given widget.
 * Params Values: gtk_button_new ()
 * Expected: void  
 */
static void test_purpose_12()
{
    #define WIDGET gtk_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    gtk_grab_add (widget);
    gtk_grab_remove (widget);
    
    if (gtk_grab_get_current () != NULL)
    {
        TEST_FAIL ("current grab should be NULL", NULL); 
        return;
    }
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_remove
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Removes the grab from the given widget.
 * Params Values: gtk_check_button_new ()
 * Expected: void  
 */
static void test_purpose_13()
{
    #define WIDGET gtk_check_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    gtk_grab_add (widget);
    gtk_grab_remove (widget);
    
    if (gtk_grab_get_current () != NULL)
    {
        TEST_FAIL ("current grab should be NULL", NULL); 
        return;
    }
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_remove
 * Params: GtkWidget *widget
 * Returns: void
 * Assertion: Removes the grab from the given widget.
 * Params Values: gtk_hscale_new_with_range (0, 10, 1)
 * Expected: void  
 */
static void test_purpose_14()
{
    #define WIDGET gtk_hscale_new_with_range (0, 10, 1)
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_WIDGET_CAN_FOCUS (widget))
    {
        TEST_FAIL ("widget '%s' can't have focus", WIDGET); 
        return;
    } 
    
    gtk_grab_add (widget);
    gtk_grab_remove (widget);
    
    if (gtk_grab_get_current () != NULL)
    {
        TEST_FAIL ("current grab should be NULL", NULL); 
        return;
    }
    
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_get_current
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Queries the current grab.
 * Params Values: void
 * Expected: NULL  
 */
static void test_purpose_15()
{
    #define WIDGET NULL
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (widget)
    {
        gtk_grab_add (widget);
    }
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    if (widget)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_get_current
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Queries the current grab.
 * Params Values: void
 * Expected: gtk_entry_new ()  
 */
static void test_purpose_16()
{
    #define WIDGET gtk_entry_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (widget)
    {
        gtk_grab_add (widget);
    }
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    if (widget)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_get_current
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Queries the current grab.
 * Params Values: void
 * Expected: gtk_button_new ()  
 */
static void test_purpose_17()
{
    #define WIDGET gtk_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (widget)
    {
        gtk_grab_add (widget);
    }
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    if (widget)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_get_current
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Queries the current grab.
 * Params Values: void
 * Expected: gtk_check_button_new ()  
 */
static void test_purpose_18()
{
    #define WIDGET gtk_check_button_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (widget)
    {
        gtk_grab_add (widget);
    }
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    if (widget)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_grab_get_current
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Queries the current grab.
 * Params Values: void
 * Expected: gtk_hscale_new_with_range (0, 10, 1)  
 */
static void test_purpose_19()
{
    #define WIDGET gtk_hscale_new_with_range (0, 10, 1)
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = WIDGET;
    if (widget)
    {
        gtk_grab_add (widget);
    }
    
    if (gtk_grab_get_current () != widget)
    {
        TEST_FAIL ("current grab is not correct", NULL); 
        return;
    }
    
    if (widget)
    {
        gtk_widget_destroy (widget);
    }
    
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_key_snooper_install
 * Params: GtkKeySnoopFunc snooper, gpointer func_data
 * Returns: guint
 * Assertion: Installs a key snooper function, which will get called on all key events before delivering them normally.
 * Params Values: GtkKeySnoopFunc snooper, gpointer func_data
 * Expected: guint  
 */
static void test_purpose_20()
{
    guint snooper_id;
    GtkWidget* window;
    GdkEventKey e;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_snooper_count = 0;
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_show_all (window);
    
    if (!(snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc)global_snooper, NULL)))
    {
        TEST_FAIL ("'gtk_timeout_add' returns NULL", NULL); 
        return;
    }
    
    TEST_VISUALIZE_AND_DO (window, global_snooper_key);
    
    if (!global_snooper_count)
    {
        TEST_FAIL ("global_snooper_count is %i", global_snooper_count); 
    }
    
    gtk_key_snooper_remove (snooper_id);
    
    gtk_widget_destroy (window);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_key_snooper_remove
 * Params: guint snooper_handler_id
 * Returns: void
 * Assertion: Removes the key snooper function with the given id.
 * Params Values: guint snooper_handler_id
 * Expected: void  
 */
static void test_purpose_21()
{
    
    guint snooper_id;
    GtkWidget* window;
    GdkEventKey e;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_snooper_count = 0;
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_show_all (window);
    
    if (!(snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc)global_snooper, NULL)))
    {
        TEST_FAIL ("'gtk_timeout_add' returns NULL", NULL); 
        return;
    }
    
    gtk_key_snooper_remove (snooper_id);
    
    TEST_VISUALIZE_AND_DO (window, global_snooper_key);
    
    if (global_snooper_count)
    {
        TEST_FAIL ("global_snooper_count is %i", global_snooper_count); 
    }
    
    gtk_widget_destroy (window);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main
 * Params: void
 * Returns: void
 * Assertion: Runs the main loop until gtk_main_quit() is called.
 * Params Values: viod
 * Expected: void
 * 
 * Name: gtk_main_quit
 * Params: void
 * Returns: void
 * Assertion: Makes the innermost invocation of the main loop return when it regains control.
 * Params Values: void
 * Expected: void
 * 
 * Name: gtk_init_add
 * Params: GtkFunction function, gpointer data
 * Returns: void
 * Assertion: Registers a function to be called when the mainloop is started.
 * Params Values: GtkFunction function, gpointer data
 * Expected: void  
 */
static void test_purpose_22()
{
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_init, NULL);
    gtk_main ();
    
    if (!global_count)
    {
        TEST_FAIL ("'global_init' not called", NULL); 
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_level
 * Params: void
 * Returns: guint
 * Assertion: Asks for the current nesting level of the main loop.
 * Params Values: void
 * Expected: guint  
 */
static void test_purpose_23()
{
    #define LEVEL 1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_level
 * Params: void
 * Returns: guint
 * Assertion: Asks for the current nesting level of the main loop.
 * Params Values: void
 * Expected: guint  
 */
static void test_purpose_24()
{
    #define LEVEL 3
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_level
 * Params: void
 * Returns: guint
 * Assertion: Asks for the current nesting level of the main loop.
 * Params Values: void
 * Expected: guint  
 */
static void test_purpose_25()
{
    #define LEVEL 75
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_iteration
 * Params: void
 * Returns: gboolean
 * Assertion: Runs a single iteration of the mainloop.
 * Params Values: void
 * Expected: gboolean
 * 
 * Name: gtk_main_iteration
 * Params: void
 * Returns: gboolean
 * Assertion: If no events are waiting to be processed GTK+ will block until the next event is noticed.
 * Params Values: void
 * Expected: gboolean  
 */
static void test_purpose_26()
{
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gtk_widget_show (gtk_window_new (GTK_WINDOW_TOPLEVEL));
    
    if (!gtk_main_iteration ())
    {
        TEST_FAIL ("'gtk_main_iteration' returns FALSE", NULL); 
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_iteration_do
 * Params: gboolean blocking
 * Returns: gboolean
 * Assertion: Runs a single iteration of the mainloop.
 * Params Values: gboolean blocking
 * 
 * Expected: gboolean
 * Name: gtk_main_iteration_do
 * Params: gboolean blocking
 * Returns: gboolean
 * Assertion: If no events are available either return or block dependent on the value of blocking.
 * Params Values: gboolean blocking
 * Expected: gboolean  
 */
static void test_purpose_27()
{
    #define BLOCKING TRUE
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_show (widget);
    
    if (!gtk_main_iteration_do (BLOCKING))
    {
        TEST_FAIL ("'gtk_main_iteration_do' returns FALSE ", NULL); 
    }
    
    gtk_widget_destroy (widget);
    
    #undef BLOCKING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_main_iteration_do
 * Params: gboolean blocking
 * Returns: gboolean
 * Assertion: Runs a single iteration of the mainloop.
 * Params Values: gboolean blocking
 * 
 * Expected: gboolean
 * Name: gtk_main_iteration_do
 * Params: gboolean blocking
 * Returns: gboolean
 * Assertion: If no events are available either return or block dependent on the value of blocking.
 * Params Values: gboolean blocking
 * Expected: gboolean  
 */
static void test_purpose_28()
{
    #define BLOCKING FALSE
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_show (widget);
    
    if (!gtk_main_iteration_do (BLOCKING))
    {
        TEST_FAIL ("'gtk_main_iteration_do' returns FALSE ", NULL); 
    }
    
    gtk_widget_destroy (widget);
    
    #undef BLOCKING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add
 * Params: guint main_level, GtkFunction function, gpointer data
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left. 
 * Params Values: guint main_level, GtkFunction function, gpointer data
 * Expected: guint  
 */
static void test_purpose_29()
{
    #define LEVEL 1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add
 * Params: guint main_level, GtkFunction function, gpointer data
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left. 
 * Params Values: guint main_level, GtkFunction function, gpointer data
 * Expected: guint  
 */
static void test_purpose_30()
{
    #define LEVEL 3
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add
 * Params: guint main_level, GtkFunction function, gpointer data
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left. 
 * Params Values: guint main_level, GtkFunction function, gpointer data
 * Expected: guint  
 */
static void test_purpose_31()
{
    #define LEVEL 50
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add (LEVEL, (GtkFunction)global_set_result, NULL);
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
        return;
    }
    
    #undef LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_destroy
 * Params: guint main_level,  GtkObject *object
 * Returns: void
 * Assertion: Trigger destruction of object in case the mainloop at level main_level is quit.
 * Params Values: guint main_level,  GtkObject *object
 * Expected: void  
 */
static void test_purpose_32()
{
    #define LEVEL 1
    #define WIDGET gtk_label_new ("Label")
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (widget), "destroy", G_CALLBACK (global_set_result_2), NULL);
    
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add_destroy (LEVEL, GTK_OBJECT (widget));
    gtk_main ();
    
    if (!global_count)
    {
        TEST_FAIL ("global_count = %i", global_count); 
        return;
    }
    
    #undef LEVEL
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_destroy
 * Params: guint main_level,  GtkObject *object
 * Returns: void
 * Assertion: Trigger destruction of object in case the mainloop at level main_level is quit.
 * Params Values: guint main_level,  GtkObject *object
 * Expected: void  
 */
static void test_purpose_33()
{
    #define LEVEL 3
    #define WIDGET gtk_button_new () 
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (widget), "destroy", G_CALLBACK (global_set_result_2), NULL);
    
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add_destroy (LEVEL, GTK_OBJECT (widget));
    gtk_main ();
    
    if (!global_count)
    {
        TEST_FAIL ("global_count = %i", global_count); 
        return;
    }
    
    #undef LEVEL
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_destroy
 * Params: guint main_level,  GtkObject *object
 * Returns: void
 * Assertion: Trigger destruction of object in case the mainloop at level main_level is quit.
 * Params Values: guint main_level,  GtkObject *object
 * Expected: void  
 */
static void test_purpose_34()
{
    #define LEVEL 50
    #define WIDGET gtk_entry_new ()
    
    GtkWidget* widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    widget = WIDGET;
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    g_signal_connect (G_OBJECT (widget), "destroy", G_CALLBACK (global_set_result_2), NULL);
    
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    gtk_quit_add_destroy (LEVEL, GTK_OBJECT (widget));
    gtk_main ();
    
    if (!global_count)
    {
        TEST_FAIL ("global_count = %i", global_count); 
        return;
    }
    
    #undef LEVEL
    #undef WIDGET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint
 * 
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: In comparison to gtk_quit_add() this function adds the possibility to pass a marshaller and a function to be called when the quit handler is freed.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint  
 */
static void test_purpose_35()
{
    #define LEVEL 5
    #define FUNCTION (GtkFunction)global_set_result
    #define MARSHAL NULL
    #define DESTROY NULL
    #define CORRECT 1
    
    gpointer data;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    global_result = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    if (!gtk_quit_add_full (LEVEL, FUNCTION, MARSHAL, data, DESTROY))
    {
        TEST_FAIL ("'gtk_quit_add' returns NULL", NULL); 
        return;
    }
    
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
    }
    
    if (global_count != CORRECT)
    {
        TEST_FAIL ("global_count = %i, should be %i", global_count, CORRECT ); 
    }
    
    #undef LEVEL
    #undef FUNCTION
    #undef MARSHAL
    #undef DESTROY
    #undef CORRECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint
 * 
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: In comparison to gtk_quit_add() this function adds the possibility to pass a marshaller and a function to be called when the quit handler is freed.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint  
 */
static void test_purpose_36()
{
    #define LEVEL 5
    #define FUNCTION NULL
    #define MARSHAL (GtkCallbackMarshal)global_set_result
    #define DESTROY (GtkDestroyNotify)global_set_result_2
    #define CORRECT 2
    
    gpointer data;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    global_result = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    if (!gtk_quit_add_full (LEVEL, FUNCTION, MARSHAL, data, DESTROY))
    {
        TEST_FAIL ("'gtk_quit_add' returns NULL", NULL); 
        return;
    }
    
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
    }
    
    if (global_count != CORRECT)
    {
        TEST_FAIL ("global_count = %i, should be %i", global_count, CORRECT ); 
    }
    
    #undef LEVEL
    #undef FUNCTION
    #undef MARSHAL
    #undef DESTROY
    #undef CORRECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint
 * 
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: In comparison to gtk_quit_add() this function adds the possibility to pass a marshaller and a function to be called when the quit handler is freed.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint  
 */
static void test_purpose_37()
{
    #define LEVEL 30
    #define FUNCTION NULL
    #define MARSHAL (GtkCallbackMarshal)global_set_result
    #define DESTROY NULL
    #define CORRECT 1
    
    gpointer data;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    global_result = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    if (!gtk_quit_add_full (LEVEL, FUNCTION, MARSHAL, data, DESTROY))
    {
        TEST_FAIL ("'gtk_quit_add' returns NULL", NULL); 
        return;
    }
    
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
    }
    
    if (global_count != CORRECT)
    {
        TEST_FAIL ("global_count = %i, should be %i", global_count, CORRECT ); 
    }
    
    #undef LEVEL
    #undef FUNCTION
    #undef MARSHAL
    #undef DESTROY
    #undef CORRECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: Registers a function to be called when an instance of the mainloop is left.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint
 * 
 * Name: gtk_quit_add_full
 * Params: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Returns: guint
 * Assertion: In comparison to gtk_quit_add() this function adds the possibility to pass a marshaller and a function to be called when the quit handler is freed.
 * Params Values: guint main_level, GtkFunction function, GtkCallbackMarshal marshal, gpointer data, GtkDestroyNotify destroy
 * Expected: guint  
 */
static void test_purpose_38()
{
    #define LEVEL 10
    #define FUNCTION (GtkFunction)global_set_result
    #define MARSHAL NULL
    #define DESTROY (GtkDestroyNotify)global_set_result_2
    #define CORRECT 2
    
    gpointer data;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    global_result = 0;
    gtk_init_add ((GtkFunction)global_main_level, NULL);
    if (!gtk_quit_add_full (LEVEL, FUNCTION, MARSHAL, data, DESTROY))
    {
        TEST_FAIL ("'gtk_quit_add' returns NULL", NULL); 
        return;
    }
    
    gtk_main ();
    
    if (global_result != LEVEL)
    {
        TEST_FAIL ("global_result = %i, should be %i", global_result, LEVEL); 
    }
    
    if (global_count != CORRECT)
    {
        TEST_FAIL ("global_count = %i, should be %i", global_count, CORRECT ); 
    }
    
    #undef LEVEL
    #undef FUNCTION
    #undef MARSHAL
    #undef DESTROY
    #undef CORRECT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_remove
 * Params: guint quit_handler_id
 * Returns: void
 * Assertion: Removes a quit handler by it's identifier.'
 * Params Values: guint quit_handler_id
 * Expected: void  
 */
static void test_purpose_39()
{
    guint quit_handler_id;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    gtk_init_add ((GtkFunction)global_init, NULL);
    
    quit_handler_id = gtk_quit_add (0, (GtkFunction)global_set_result, NULL);
    
    if (!quit_handler_id)
    {
        TEST_FAIL ("'gtk_quit_add' returns NULL", NULL); 
        return;
    }
    
    gtk_quit_remove (quit_handler_id);
    
    gtk_main ();
    
    if (global_result)
    {
        TEST_FAIL ("global_count = %i", global_count); 
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_quit_remove_by_data
 * Params: gpointer data
 * Returns: void
 * Assertion: Removes a quit handler identified by it's data field.'
 * Params Values: gpointer data
 * Expected: void  
 */
static void test_purpose_40()
{
    gpointer data = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    gtk_init_add ((GtkFunction)global_init, NULL);
    
    gtk_quit_add (0, (GtkFunction)global_set_result, data);
    gtk_quit_remove_by_data (data);
    
    gtk_main ();
    
    if (global_result)
    {
        TEST_FAIL ("global_result = %i", global_result); 
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: Obtains a copy of the event currently being processed by GTK+.
 * Params Values: void
 * Expected: GdkEvent*
 * 
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: If there is no current event, the function returns NULL.
 * Params Values: void
 * Expected: GdkEvent*  
 */
static void test_purpose_41()
{
    #define EVENT_TYPE GdkEventAny
    #define EVENT GDK_DESTROY
    #define SIGNAL "destroy-event"
    #define CATCH TRUE
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    
    if (CATCH)
    {
        gtk_main_do_event ((GdkEvent*)e);
    
    } else {
        
        global_event = gtk_get_current_event ();
    }
    
    if (!global_event && CATCH)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (global_event)
    {
        if ((global_event->any.type != e->type) || (global_event->any.window != widget->window))
        {
            TEST_FAIL ("events not equal", NULL); 
        }
    
        gdk_event_free (global_event);
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef CATCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: Obtains a copy of the event currently being processed by GTK+.
 * Params Values: void
 * Expected: GdkEvent*
 * 
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: If there is no current event, the function returns NULL.
 * Params Values: void
 * Expected: GdkEvent*  
 */
static void test_purpose_42()
{
    #define EVENT_TYPE GdkEventButton
    #define EVENT GDK_BUTTON_RELEASE
    #define SIGNAL "button-release-event"
    #define CATCH TRUE
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    
    if (CATCH)
    {
        gtk_main_do_event ((GdkEvent*)e);
    
    } else {
        
        global_event = gtk_get_current_event ();
    }
    
    if (!global_event && CATCH)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (global_event)
    {
        if ((global_event->any.type != e->type) || (global_event->any.window != widget->window))
        {
            TEST_FAIL ("events not equal", NULL); 
        }
    
        gdk_event_free (global_event);
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef CATCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: Obtains a copy of the event currently being processed by GTK+.
 * Params Values: void
 * Expected: GdkEvent*
 * 
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: If there is no current event, the function returns NULL.
 * Params Values: void
 * Expected: GdkEvent*  
 */
static void test_purpose_43()
{
    #define EVENT_TYPE GdkEventConfigure
    #define EVENT GDK_CONFIGURE
    #define SIGNAL "configure-event"
    #define CATCH TRUE
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    
    if (CATCH)
    {
        gtk_main_do_event ((GdkEvent*)e);
    
    } else {
        
        global_event = gtk_get_current_event ();
    }
    
    if (!global_event && CATCH)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (global_event)
    {
        if ((global_event->any.type != e->type) || (global_event->any.window != widget->window))
        {
            TEST_FAIL ("events not equal", NULL); 
        }
    
        gdk_event_free (global_event);
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef CATCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: Obtains a copy of the event currently being processed by GTK+.
 * Params Values: void
 * Expected: GdkEvent*
 * 
 * Name: gtk_get_current_event
 * Params: void
 * Returns: GdkEvent*
 * Assertion: If there is no current event, the function returns NULL.
 * Params Values: void
 * Expected: GdkEvent*  
 */
static void test_purpose_44()
{
    #define EVENT_TYPE GdkEventMotion
    #define EVENT GDK_MOTION_NOTIFY
    #define SIGNAL "motion-notify-event"
    #define CATCH FALSE
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    
    if (CATCH)
    {
        gtk_main_do_event ((GdkEvent*)e);
    
    } else {
        
        global_event = gtk_get_current_event ();
    }
    
    if (!global_event && CATCH)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (global_event)
    {
        if ((global_event->any.type != e->type) || (global_event->any.window != widget->window))
        {
            TEST_FAIL ("events not equal", NULL); 
        }
    
        gdk_event_free (global_event);
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef CATCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event_time
 * Params: void
 * Returns: guint32
 * Assertion: If there is a current event and it has a timestamp, return that timestamp, otherwise return GDK_CURRENT_TIME.
 * Params Values: void
 * Expected: 734627  
 */
static void test_purpose_45()
{
    #define EVENT_TYPE GdkEventButton
    #define EVENT GDK_BUTTON_RELEASE
    #define SIGNAL "button-release-event"
    #define TIME 734627
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    e->time = TIME;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    gtk_main_do_event ((GdkEvent*)e);
    
    if (!global_event)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (((EVENT_TYPE*)global_event)->time != TIME)
    {
        TEST_FAIL ("timestamp not equal", NULL); 
    }
    
    gdk_event_free (global_event);
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef TIME
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_current_event_state
 * Params: GdkModifierType *state
 * Returns: gboolean
 * Assertion:If there is a current event and it has a state field, place that state field in state and return TRUE, otherwise return FALSE.
 * Params Values: GdkModifierType *state
 * Expected: gboolean  
 */
static void test_purpose_46()
{
    #define EVENT_TYPE GdkEventButton
    #define EVENT GDK_BUTTON_RELEASE
    #define SIGNAL "button-release-event"
    #define STATE GDK_LOCK_MASK
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_event = NULL;
    widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    e->state = STATE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_set_event), NULL);
    gtk_main_do_event ((GdkEvent*)e);
    
    if (!global_event)
    {
        TEST_FAIL ("'event' is NULL", NULL); 
        return;
    }
    
    if (((EVENT_TYPE*)global_event)->state != STATE)
    {
        TEST_FAIL ("timestamp not equal", NULL); 
    }
    
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    #undef STATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_event_widget
 * Params: GdkEvent*
 * Returns: GtkWidget*
 * Assertion: If event is NULL or the event was not associated with any widget, returns NULL, otherwise returns the widget that received the event originally.
 * Params Values: GdkEvent*
 * Expected:  GtkWidget*  
 */
static void test_purpose_47()
{
    #define WIDGET gtk_dialog_new ()
    #define ASSOCIATE TRUE
    
    GtkWidget* widget;
    GdkEventAny *e = (GdkEventAny*)gdk_event_new (GDK_EXPOSE);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = GDK_EXPOSE;
    
    if (ASSOCIATE)
    {
        e->window = widget->window;
    }
    
    if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) == NULL))
    {
        TEST_FAIL ("event should be associated with the widget", NULL); 
        return;
    }
    
    if (widget != gtk_get_event_widget ((GdkEvent*)e))
    {
        if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) != NULL))
            TEST_FAIL ("widgets not equal", NULL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef ASSOCIATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_event_widget
 * Params: GdkEvent*
 * Returns: GtkWidget*
 * Assertion: If event is NULL or the event was not associated with any widget, returns NULL, otherwise returns the widget that received the event originally.
 * Params Values: GdkEvent*
 * Expected:  GtkWidget*  
 */
static void test_purpose_48()
{
    #define WIDGET gtk_dialog_new ()
    #define ASSOCIATE FALSE
    
    GtkWidget* widget;
    GdkEventAny *e = (GdkEventAny*)gdk_event_new (GDK_EXPOSE);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = GDK_EXPOSE;
    
    if (ASSOCIATE)
    {
        e->window = widget->window;
    }
    
    if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) == NULL))
    {
        TEST_FAIL ("event should be associated with the widget", NULL); 
        return;
    }
    
    if (widget != gtk_get_event_widget ((GdkEvent*)e))
    {
        if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) != NULL))
            TEST_FAIL ("widgets not equal", NULL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef ASSOCIATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_event_widget
 * Params: GdkEvent*
 * Returns: GtkWidget*
 * Assertion: If event is NULL or the event was not associated with any widget, returns NULL, otherwise returns the widget that received the event originally.
 * Params Values: GdkEvent*
 * Expected:  GtkWidget*  
 */
static void test_purpose_49()
{
    #define WIDGET gtk_window_new (GTK_WINDOW_TOPLEVEL)
    #define ASSOCIATE TRUE
    
    GtkWidget* widget;
    GdkEventAny *e = (GdkEventAny*)gdk_event_new (GDK_EXPOSE);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = GDK_EXPOSE;
    
    if (ASSOCIATE)
    {
        e->window = widget->window;
    }
    
    if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) == NULL))
    {
        TEST_FAIL ("event should be associated with the widget", NULL); 
        return;
    }
    
    if (widget != gtk_get_event_widget ((GdkEvent*)e))
    {
        if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) != NULL))
            TEST_FAIL ("widgets not equal", NULL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef ASSOCIATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_get_event_widget
 * Params: GdkEvent*
 * Returns: GtkWidget*
 * Assertion: If event is NULL or the event was not associated with any widget, returns NULL, otherwise returns the widget that received the event originally.
 * Params Values: GdkEvent*
 * Expected:  GtkWidget*  
 */
static void test_purpose_50()
{
    #define WIDGET gtk_window_new (GTK_WINDOW_TOPLEVEL)
    #define ASSOCIATE FALSE
    
    GtkWidget* widget;
    GdkEventAny *e = (GdkEventAny*)gdk_event_new (GDK_EXPOSE);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = GDK_EXPOSE;
    
    if (ASSOCIATE)
    {
        e->window = widget->window;
    }
    
    if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) == NULL))
    {
        TEST_FAIL ("event should be associated with the widget", NULL); 
        return;
    }
    
    if (widget != gtk_get_event_widget ((GdkEvent*)e))
    {
        if (ASSOCIATE && (gtk_get_event_widget ((GdkEvent*)e) != NULL))
            TEST_FAIL ("widgets not equal", NULL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef ASSOCIATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_propagate_event
 * Params: GtkWidget *widget, GdkEvent *event
 * Returns: void
 * Assertion: Sends an event to a widget, propagating the event to parent widgets if the event remains unhandled. 
 * Params Values: GtkWidget *widget, GdkEvent *event
 * Expected: void  
 */
static void test_purpose_51()
{
    #define WIDGET gtk_dialog_new ()
    #define EVENT_TYPE GdkEventKey
    #define EVENT GDK_KEY_PRESS
    #define SIGNAL "key-press-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_show_all (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_propagate_event (widget, (GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_propagate_event
 * Params: GtkWidget *widget, GdkEvent *event
 * Returns: void
 * Assertion: Sends an event to a widget, propagating the event to parent widgets if the event remains unhandled. 
 * Params Values: GtkWidget *widget, GdkEvent *event
 * Expected: void  
 */
static void test_purpose_52()
{
    #define WIDGET gtk_dialog_new ()
    #define EVENT_TYPE GdkEventButton
    #define EVENT GDK_BUTTON_RELEASE
    #define SIGNAL "button-release-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_show_all (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_propagate_event (widget, (GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_propagate_event
 * Params: GtkWidget *widget, GdkEvent *event
 * Returns: void
 * Assertion: Sends an event to a widget, propagating the event to parent widgets if the event remains unhandled. 
 * Params Values: GtkWidget *widget, GdkEvent *event
 * Expected: void  
 */
static void test_purpose_53()
{
    #define WIDGET gtk_window_new (GTK_WINDOW_TOPLEVEL)
    #define EVENT_TYPE GdkEventConfigure
    #define EVENT GDK_CONFIGURE
    #define SIGNAL "configure-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_show_all (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_propagate_event (widget, (GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_propagate_event
 * Params: GtkWidget *widget, GdkEvent *event
 * Returns: void
 * Assertion: Sends an event to a widget, propagating the event to parent widgets if the event remains unhandled. 
 * Params Values: GtkWidget *widget, GdkEvent *event
 * Expected: void  
 */
static void test_purpose_54()
{
    #define WIDGET gtk_window_new (GTK_WINDOW_TOPLEVEL)
    #define EVENT_TYPE GdkEventMotion
    #define EVENT GDK_MOTION_NOTIFY
    #define SIGNAL "motion-notify-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_show_all (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_propagate_event (widget, (GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_main_do_event
 * Params: GdkEvent *event
 * Returns: void
 * Assertion: Processes a single GDK event. 
 * Params Values: GdkEvent *event
 * Expected: void  
 */
static void test_purpose_55()
{
    #define WIDGET gtk_dialog_new ()
    #define EVENT_TYPE GdkEventButton
    #define EVENT GDK_BUTTON_RELEASE
    #define SIGNAL "button-release-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    gboolean result;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_main_do_event ((GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_main_do_event
 * Params: GdkEvent *event
 * Returns: void
 * Assertion: Processes a single GDK event. 
 * Params Values: GdkEvent *event
 * Expected: void  
 */
static void test_purpose_56()
{
    #define WIDGET gtk_window_new (GTK_WINDOW_TOPLEVEL)
    #define EVENT_TYPE GdkEventConfigure
    #define EVENT GDK_CONFIGURE
    #define SIGNAL "configure-event"
    
    GtkWidget* widget;
    EVENT_TYPE *e = (EVENT_TYPE*)gdk_event_new (EVENT);
    gboolean result;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_result = 0;
    widget = WIDGET;
    gtk_widget_realize (widget);
    
    e->type = EVENT;
    e->window = widget->window;
    e->send_event = TRUE;
    
    g_signal_connect (G_OBJECT (widget), SIGNAL, G_CALLBACK (global_signal), NULL);
    
    gtk_main_do_event ((GdkEvent*)e);
    
    if (!global_result)
    {
        TEST_FAIL ("signal '%s' not catched", SIGNAL); 
    }
    
    gdk_event_free ((GdkEvent*)e);
    gtk_widget_destroy (widget);
    
    #undef WIDGET
    #undef EVENT_TYPE
    #undef EVENT
    #undef SIGNAL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_init
 * Params: void
 * Returns: void
 * Assertion: It will initialize everything needed to operate the toolkit and parses some standard command line options. 
 * Params Values: void
 * Expected: void  
 */
static void test_purpose_57()
{
    #define ARGC 0
    #define ARGV NULL
    
    int argc;
    gchar* argv;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    argc = ARGC;
    argv = ARGV;
    
    gtk_init (&argc, (char***)&argv);
    
    #undef ARGC
    #undef ARGV
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_init
 * Params: void
 * Returns: void
 * Assertion: It will initialize everything needed to operate the toolkit and parses some standard command line options. 
 * Params Values: void
 * Expected: void  
 */
static void test_purpose_58()
{
    #define ARGC 2
    #define ARGV "test 1"
    
    int argc;
    gchar* argv;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    argc = ARGC;
    argv = ARGV;
    
    gtk_init (&argc, (char***)&argv);
    
    #undef ARGC
    #undef ARGV
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_init_check
 * Params: void
 * Returns: gboolean
 * Assertion: This function does the same work as gtk_init() with only a single change: It does not terminate the program if the GUI can't be initialized.'
 * Params Values: void
 * Expected: gboolean  
 */
static void test_purpose_59()
{
    #define ARGC 0
    #define ARGV NULL
    
    int argc;
    gchar* argv;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    argc = ARGC;
    argv = ARGV;
    
    if (!gtk_init_check (&argc, (char***)&argv))
    {
        TEST_FAIL ("'gtk_init_check' returns FALSE", NULL);
    }
    
    
    #undef ARGC
    #undef ARGV
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_init_check
 * Params: void
 * Returns: gboolean
 * Assertion: This function does the same work as gtk_init() with only a single change: It does not terminate the program if the GUI can't be initialized.'
 * Params Values: void
 * Expected: gboolean  
 */
static void test_purpose_60()
{
    #define ARGC 2
    #define ARGV "test 1"
    
    int argc;
    gchar* argv;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    argc = ARGC;
    argv = ARGV;
    
    if (!gtk_init_check (&argc, (char***)&argv))
    {
        TEST_FAIL ("'gtk_init_check' returns FALSE", NULL);
    }
    
    
    #undef ARGC
    #undef ARGV
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for Main_loop_and_Events ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {test_purpose_41, 1},
    {test_purpose_42, 1},
    {test_purpose_43, 1},
    {test_purpose_44, 1},
    {test_purpose_45, 1},
    {test_purpose_46, 1},
    {test_purpose_47, 1},
    {test_purpose_48, 1},
    {test_purpose_49, 1},
    {test_purpose_50, 1},
    {test_purpose_51, 1},
    {test_purpose_52, 1},
    {test_purpose_53, 1},
    {test_purpose_54, 1},
    {test_purpose_55, 1},
    {test_purpose_56, 1},
    {test_purpose_57, 1},
    {test_purpose_58, 1},
    {test_purpose_59, 1},
    {test_purpose_60, 1},
    {NULL,0}
};
