/*
 * Param_Value methods tests
 * Testing Param_Value methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
static guint flag = 0;

#define TEST_TYPE   (test_get_type())
#define TEST(obj)   (G_TYPE_CHECK_INSTANCE_CAST((obj), TEST_TYPE, Test))

typedef struct _Test        Test;
typedef struct _TestClass   TestClass;

struct _Test {
    GObject parent;
};

struct _TestClass {
    GObjectClass parent_class;
};

static void test_init(Test* test)
{
}

static void test_class_init(TestClass* testclass)
{
}

GType test_get_type(void)
{
    static GType test_type = 0;
    if (!test_type){
        static const GTypeInfo test_info = {
            sizeof(TestClass),
            NULL, NULL,
            (GClassInitFunc)test_class_init,
            NULL, NULL,
            sizeof(Test),
            0,
            (GInstanceInitFunc)test_init
        };
        test_type = g_type_register_static(G_TYPE_OBJECT, "Test", &test_info, 0);
    }
    return test_type;
}

Test* test_new(void)
{
    Test* test = g_object_new(TEST_TYPE, NULL);
    
    return test;
}

#define TEST_BOXED_TYPE     (test_boxed_get_type())

typedef struct _TestStruct  TestStruct;

struct _TestStruct {
    int ref_count;
};

TestStruct* test_boxed_ref(TestStruct* ts)
{
    if (ts != NULL)
        ts->ref_count ++;
    
    return ts;
}

void test_boxed_unref(TestStruct* ts)
{
    if (ts == NULL)
        return;
    g_return_if_fail(ts->ref_count > 0);
    ts->ref_count --;
    if (ts->ref_count != 0)
        return;
    g_free(ts);
}

GType test_boxed_get_type()
{
    static GType boxed_type = 0;
    if (boxed_type == 0){
        boxed_type = g_boxed_type_register_static(
                            "TestBoxed",
                            (GBoxedCopyFunc)test_boxed_ref,
                            (GBoxedFreeFunc)test_boxed_unref
                     ); 
    }
    return boxed_type;
}

enum {
    PROP_STRING = 1,
    PROP_INT,
    PROP_BOOLEAN,
    PROP_UINT,
    PROP_POINTER,
    PROP_OBJ,
    PROP_DOUBLE,
    PROP_BOXED,
    PROP_FLOAT,
    PROP_INT64
};
#define TEST_PROPERTY_TYPE  (test_property_get_type())
#define TEST_PROPERTY(obj)  (G_TYPE_CHECK_INSTANCE_CAST((obj), TEST_PROPERTY_TYPE, TestProperty))

typedef struct _TestProperty        TestProperty;
typedef struct _TestPropertyClass   TestPropertyClass;

struct _TestProperty {
    GObject parent;
};

struct _TestPropertyClass {
    GObjectClass parent_class;
};

static void test_set_property(GObject *object, guint prop_id, GValue const *value, GParamSpec *pspec)
{
    flag = prop_id;
}

static void test_get_property(GObject *object, guint prop_id, GValue const *value, GParamSpec *pspec)
{
    flag = prop_id;
}

static void test_property_init(TestProperty* testproperty)
{
}

static void test_property_class_init(GObjectClass *gobject_class)
{
    TestPropertyClass* testpropertyclass = (TestPropertyClass *)gobject_class;
    gobject_class->set_property = test_set_property;
    gobject_class->get_property = test_get_property;
    
    g_object_class_install_property(gobject_class,
                                    PROP_STRING,
                                    g_param_spec_string("SpecString", "SString", "The string spec test", "", G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_INT,
                                    g_param_spec_int("SpecInt", "SInt", "The int spec test", 1, 100, 1, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_BOOLEAN,
                                    g_param_spec_boolean("SpecBoolean", "SBoolean", "The boolean spec test", TRUE, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_UINT,
                                    g_param_spec_uint("SpecUint", "SUint", "The uint spec test", 1, 25000, 100, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class, 
                                    PROP_POINTER,
                                    g_param_spec_pointer("SpecPointer", "SPointer", "The pointer spec test", G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_OBJ,
                                    g_param_spec_object("SpecObject", "SObject", "The object spec test", TEST_TYPE, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_DOUBLE,
                                    g_param_spec_double("SpecDouble", "SDouble", "The double spec test", -12.0, 23.0, 0, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_BOXED,
                                    g_param_spec_boxed("SpecBoxed", "SBoxed", "The boxed spec test", TEST_BOXED_TYPE, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_FLOAT,
                                    g_param_spec_float("SpecFloat", "SFloat", "The float spec test", -0.5, 0.5, 0, G_PARAM_READWRITE)
                                    );
    g_object_class_install_property(gobject_class,
                                    PROP_INT64,
                                    g_param_spec_int64("SpecInt64", "SInt64", "The int64 spec test", 1, 100000, 50000, G_PARAM_READWRITE)
                                    );
}

GType test_property_get_type(void)
{
    static GType test_property_type = 0;
    if (!test_property_type){
        static const GTypeInfo test_property_info = {
            sizeof(TestPropertyClass),
            NULL, NULL,
            (GClassInitFunc)test_property_class_init,
            NULL, NULL,
            sizeof(TestProperty),
            0,
            (GInstanceInitFunc)test_property_init
        };
        test_property_type = g_type_register_static(G_TYPE_OBJECT, "TestProperty", &test_property_info, 0);
    }
    return test_property_type;
}

TestProperty* test_property_new(void)
{
    TestProperty* testproperty = g_object_new(TEST_PROPERTY_TYPE, NULL);

    return testproperty;
}


/*
 * Name: g_value_set_string  
 * Params: GValue *value, const gchar *v_string
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_STRING GValue to v_string.
 *            value :   a valid GValue of type G_TYPE_STRING
 *            v_string :  string to be set
 * 
 * Name: g_value_get_string
 * Params: GValue *value
 * Returns: const gchar*
 * Assertion: Get the contents of a G_TYPE_STRING GValue.
 *            value :   a valid GValue of type G_TYPE_STRING
 *            Returns :   string content of value
 * 
 * Name: g_value_take_string
 * Params: GValue *value, gchar *v_string
 * Returns: void
 * Assertion: Sets the contents of a G_TYPE_STRING GValue to v_string.
 *            value :   a valid GValue of type G_TYPE_STRING
 *            v_string :  duplicated unowned string to be set
 * Params Values: "Hello, LSB"
 * Expected: void  
 */
static void test_purpose_1()
{
    #define STRING  "Hello, LSB"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_STRING);
    g_value_set_string(&val_str, STRING);
    if (strcmp(g_value_get_string(&val_str), STRING) != 0){
        TEST_FAIL("g_value_set_string & g_value_get_string: failed", NULL);
    }
    {
        GString *gs = g_string_new(STRING);
        gs = g_string_append(gs, "Michael");
        g_value_take_string(&val_str, gs->str);
        if (strcmp(g_value_get_string(&val_str), gs->str) != 0){
            TEST_FAIL("g_value_take_string: failed", NULL);
        }
        g_string_free(gs, TRUE);
    }
    #undef STRING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_boolean
 * Params: GValue *value, gboolean v_boolean
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_BOOLEAN GValue to v_boolean.
 *            value :   a valid GValue of type G_TYPE_BOOLEAN
 *            v_boolean :  boolean value to be set
 * 
 * Name: g_value_get_boolean
 * Params: const GValue *value
 * Returns: gboolean
 * Assertion: Get the contents of a G_TYPE_BOOLEAN GValue.
 *            value :   a valid GValue of type G_TYPE_BOOLEAN
 *            Returns :   boolean contents of value
 * 
 * Params Values: TRUE
 * Expected: void  
 */
static void test_purpose_2()
{
    #define BOOLEAN  TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_BOOLEAN);
    g_value_set_boolean(&val_str, BOOLEAN);
    if (g_value_get_boolean(&val_str) != BOOLEAN){
        TEST_FAIL("g_value_set_boolean & g_value_get_boolean: failed", NULL);
    }
    #undef BOOLEAN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_int
 * Params: GValue *value, gint v_int
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_INT GValue to v_int.
 *            value :   a valid GValue of type G_TYPE_INT
 *            v_int :  integer value to be set
 * 
 * Name: g_value_get_int
 * Params: const GValue *value
 * Returns: gint
 * Assertion: Get the contents of a G_TYPE_INT GValue.
 *            value :   a valid GValue of type G_TYPE_INT
 *            Returns :   integer contents of value
 * 
 * Params Values: 100
 * Expected: void  
 */
static void test_purpose_3()
{
    #define INTEGER  100
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_INT);
    g_value_set_int(&val_str, INTEGER);
    if (g_value_get_int(&val_str) != INTEGER){
        TEST_FAIL("g_value_set_int & g_value_get_int: failed", NULL);
    }
    #undef INTEGER
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_double
 * Params: GValue *value, gdouble v_double
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_DOUBLE GValue to v_double.
 *            value :   a valid GValue of type G_TYPE_DOUBLE
 *            v_int :  double value to be set
 * 
 * Name: g_value_get_double
 * Params: const GValue *value
 * Returns: gint
 * Assertion: Get the contents of a G_TYPE_DOUBLE GValue.
 *            value :   a valid GValue of type G_TYPE_DOUBLE
 *            Returns :   ouble contents of value
 * 
 * Params Values: 13.12323
 * Expected: void  
 */
static void test_purpose_4()
{
    #define DOUBLE  13.12323
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_DOUBLE);
    g_value_set_double(&val_str, DOUBLE);
    if (g_value_get_double(&val_str) != DOUBLE){
        TEST_FAIL("g_value_set_double & g_value_get_double: failed", NULL);
    }
    #undef DOUBLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_float
 * Params: GValue *value, gfloat v_float
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_FLOAT GValue to v_float.
 *            value :   a valid GValue of type G_TYPE_FLOAT
 *            v_float :    float value to be set
 * 
 * Name: g_value_get_float
 * Params: const GValue *value
 * Returns: gfloat
 * Assertion: Get the contents of a G_TYPE_FLOAT GValue.
 *            value :   a valid GValue of type G_TYPE_FLOAT
 *            Returns :    float contents of value
 * 
 * Params Values: 13.1
 * Expected: void  
 */
static void test_purpose_5()
{
    #define FLOAT  13.1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_FLOAT);
    g_value_set_float(&val_str, FLOAT);
    if ((gint)g_value_get_float(&val_str) != (gint)FLOAT){
        TEST_FAIL("g_value_set_float & g_value_get_float: failed", NULL);
    }
    #undef FLOAT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_int64
 * Params: GValue *value, gint64 v_int64
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_INT64 GValue to v_int64.
 *            value :   a valid GValue of type G_TYPE_INT64
 *            v_int64 :    64bit integer value to be set
 * 
 * Name: g_value_get_int64
 * Params: const GValue *value
 * Returns: gint64
 * Assertion: Get the contents of a G_TYPE_INT64 GValue.
 *            value :   a valid GValue of type G_TYPE_INT64
 *            Returns :    64bit integer contents of value
 * 
 * Params Values: 0xabcdef01
 * Expected: void  
 */
static void test_purpose_6()
{
    #define INT64  0xabcdef01
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_INT64);
    g_value_set_int64(&val_str, INT64);
    if (g_value_get_int64(&val_str) != INT64){
        TEST_FAIL("g_value_set_int64 & g_value_get_int64: failed", NULL);
    }
    #undef INT64
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_uint64
 * Params: GValue *value, guint64 v_uint64
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_UINT64 GValue to v_uint64.
 *            value :   a valid GValue of type G_TYPE_UINT64
 *            v_uint64 :   unsigned 64bit integer value to be set
 * 
 * Name: g_value_get_uint64
 * Params: const GValue *value
 * Returns: guint64
 * Assertion: Get the contents of a G_TYPE_UINT64 GValue.
 *            value :   a valid GValue of type G_TYPE_UINT64
 *            Returns :    unsigned 64bit integer contents of value
 * 
 * Params Values: 0xabcdef01
 * Expected: void  
 */
static void test_purpose_7()
{
    #define _UINT64  0xabcdef01
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_UINT64);
    g_value_set_uint64(&val_str, _UINT64);
    if (g_value_get_uint64(&val_str) != _UINT64){
        TEST_FAIL("g_value_set_uint64 & g_value_get_uint64: failed", NULL);
    }
    #undef _UINT64
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_ulong
 * Params: GValue *value, gulong v_ulong
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_ULONG GValue to v_ulong.
 *            value :   a valid GValue of type G_TYPE_ULONG
 *            v_ulong :    unsigned long integer value to be set
 * 
 * Name: g_value_get_ulong
 * Params: const GValue *value
 * Returns: gulong
 * Assertion: Get the contents of a G_TYPE_ULONG GValue.
 *            value :   a valid GValue of type G_TYPE_ULONG
 *            Returns :    unsigned long integer contents of value
 * 
 * Params Values: 0xabcdef01
 * Expected: void  
 */
static void test_purpose_8()
{
    #define _ULONG  0xabcdef01
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_ULONG);
    g_value_set_ulong(&val_str, _ULONG);
    if (g_value_get_ulong(&val_str) != _ULONG){
        TEST_FAIL("g_value_set_ulong & g_value_get_ulong: failed", NULL);
    }
    #undef _ULONG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_long
 * Params: GValue *value, glong v_long
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_LONG GValue to v_long.
 *            value :   a valid GValue of type G_TYPE_LONG
 *            v_long : long integer value to be set
 * 
 * Name: g_value_get_long
 * Params: const GValue *value
 * Returns: glong
 * Assertion: Get the contents of a G_TYPE_LONG GValue.
 *            value :   a valid GValue of type G_TYPE_LONG
 *            Returns :    long integer contents of value
 * 
 * Params Values: 0xabcdef01
 * Expected: void  
 */
static void test_purpose_9()
{
    #define _LONG  0xabcdef01
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_LONG);
    g_value_set_long(&val_str, _LONG);
    if (g_value_get_long(&val_str) != _LONG){
        TEST_FAIL("g_value_set_long & g_value_get_long: failed", NULL);
    }
    #undef _LONG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_uint
 * Params: GValue *value, guint v_uint
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_UINT GValue to v_uint.
 *            value :   a valid GValue of type G_TYPE_UINT
 *            v_uint : unsigned integer value to be set
 * 
 * Name: g_value_get_uint
 * Params: const GValue *value
 * Returns: guint
 * Assertion: Get the contents of a G_TYPE_UINT GValue.
 *            value :   a valid GValue of type G_TYPE_UINT
 *            Returns :    unsigned integer contents of value
 * 
 * Params Values: 0xabcdef01
 * Expected: void  
 */
static void test_purpose_10()
{
    #define _UINT  0xabcdef01
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_UINT);
    g_value_set_uint(&val_str, _UINT);
    if (g_value_get_uint(&val_str) != _UINT){
        TEST_FAIL("g_value_set_uint & g_value_get_uint: failed", NULL);
    }
    #undef _UINT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_uchar
 * Params: GValue *value, guchar v_uchar
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_UCHAR GValue to v_uchar.
 *            value :   a valid GValue of type G_TYPE_UCHAR
 *            v_uchar :    unsigned character value to be set
 * Params Values: 0x88
 * Expected: void  
 */
static void test_purpose_11()
{
    #define _UCHAR  0x88
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_UCHAR);
    g_value_set_uchar(&val_str, _UCHAR);
    if (g_value_get_uchar(&val_str) != _UCHAR){
        TEST_FAIL("g_value_set_uchar: failed", NULL);
    }
    #undef _UCHAR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_char
 * Params: GValue *value, guchar v_char
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_CHAR GValue to v_char.
 *            value :   a valid GValue of type G_TYPE_CHAR
 *            v_uchar :    character value to be set
 * Params Values: 'C'
 * Expected: void  
 */
static void test_purpose_12()
{
    #define _CHAR  'C'
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_CHAR);
    g_value_set_char(&val_str, _CHAR);
    if (g_value_get_char(&val_str) != _CHAR){
        TEST_FAIL("g_value_set_char: failed", NULL);
    }
    #undef _CHAR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_pointer
 * Params: GValue *value, gpointer v_pointer
 * Returns: void
 * Assertion: Set the contents of a pointer GValue to v_pointer.
 *            value :   a valid GValue of G_TYPE_POINTER
 *            v_pointer : pointer value to be set
 * 
 * Name: g_value_get_pointer
 * Params: const GValue *value
 * Returns: gpointer
 * Assertion: Get the contents of a pointer GValue.
 *            value :   a valid GValue of G_TYPE_POINTER
 *            Returns :   pointer contents of value
 * 
 * Params Values: gpointer v_pointer
 * Expected: void  
 */
static void test_purpose_13()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gint i;
    
    i = 10;
    GValue val_str = {0, };
    
    g_value_init(&val_str, G_TYPE_POINTER);
    g_value_set_pointer(&val_str, &i);
    if (g_value_get_pointer(&val_str) != &i){
        TEST_FAIL("g_value_set_pointer & g_value_get_pointer: failed", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_object
 * Params: GValue *value, gpointer v_object
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_OBJECT derived GValue to v_object.
 *            value :   a valid GValue of G_TYPE_OBJECT derived type
 *            v_object :  object value to be set
 * 
 * Name: g_value_get_object
 * Params: const GValue *value
 * Returns: gpointer
 * Assertion: Get the contents of a G_TYPE_OBJECT derived GValue.
 *            value :   a valid GValue of G_TYPE_OBJECT derived type
 *            Returns :   object contents of value
 * 
 * Params Values: gpointer v_object
 * Expected: void  
 */
static void test_purpose_14()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    Test *test = NULL;
    GValue val_obj = {0, };
    
    g_type_init();
    g_value_init(&val_obj, G_TYPE_OBJECT);
    
    test = test_new();
    g_value_set_object(&val_obj, TEST(test));
    if (g_value_get_object(&val_obj) != TEST(test)){
        TEST_FAIL("g_value_set_object & g_value_get_object: failed", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_STRING, "Hello", g_value_set_string, "SpecString"
 * Expected: PROP_STRING  
 */
static void test_purpose_15()
{
    #define VAL_TYPE    G_TYPE_STRING
    #define VAL_VAL     "Hello"
    #define PROP_NAME   "SpecString"
    #define G_VAL_FUNC  g_value_set_string
    #define EXP_FLAG    PROP_STRING
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_INT, 20, g_value_set_int, "SpecInt"
 * Expected: PROP_INT  
 */
static void test_purpose_16()
{
    #define VAL_TYPE    G_TYPE_INT
    #define VAL_VAL     20
    #define PROP_NAME   "SpecInt"
    #define G_VAL_FUNC  g_value_set_int
    #define EXP_FLAG    PROP_INT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_BOOLEAN, FALSE, g_value_set_boolean, "SpecBoolean"
 * Expected: PROP_BOOLEAN  
 */
static void test_purpose_17()
{
    #define VAL_TYPE    G_TYPE_BOOLEAN
    #define VAL_VAL     FALSE
    #define PROP_NAME   "SpecBoolean"
    #define G_VAL_FUNC  g_value_set_boolean
    #define EXP_FLAG    PROP_BOOLEAN
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_UINT, 30, g_value_set_uint, "SpecUint"
 * Expected: PROP_UINT  
 */
static void test_purpose_18()
{
    #define VAL_TYPE    G_TYPE_UINT
    #define VAL_VAL     30
    #define PROP_NAME   "SpecUint"
    #define G_VAL_FUNC  g_value_set_uint
    #define EXP_FLAG    PROP_UINT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_POINTER, GINT_TO_POINTER(10), g_value_set_pointer, "SpecPointer"
 * Expected: PROP_POINTER  
 */
static void test_purpose_19()
{
    #define VAL_TYPE    G_TYPE_POINTER
    #define VAL_VAL     GINT_TO_POINTER(10)
    #define PROP_NAME   "SpecPointer"
    #define G_VAL_FUNC  g_value_set_pointer
    #define EXP_FLAG    PROP_POINTER
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_OBJECT, TEST_TYPE, g_value_set_object, "SpecObject"
 * Expected: PROP_OBJ  
 */
static void test_purpose_20()
{
    #define VAL_TYPE    G_TYPE_OBJECT
    #define VAL_VAL     TEST_TYPE
    #define PROP_NAME   "SpecObject"
    #define G_VAL_FUNC  g_value_set_object
    #define EXP_FLAG    PROP_OBJ
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_param_spec_int64
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint64 minimum, gint64 maximum, gint64 default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt64 instance specifying a G_TYPE_INT64 property.
 *            name :    canonical name of the property specified
 *            nick :  nick name for the property specified
 *            blurb : description of the property specified
 *            minimum :   minimum value for the property specified
 *            maximum :   maximum value for the property specified
 *            default_value : default value for the property specified
 *            flags : flags for the property specified
 *            Returns :   a newly created parameter specification
 * 
 * Name: g_param_spec_double
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gdouble minimum, gdouble maximum, gdouble default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecDouble instance specifying a G_TYPE_DOUBLE property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_float
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gfloat minimum, gfloat maximum, gfloat default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a G_TYPE_FLOAT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_pointer
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecFloat instance specifying a pointer property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_boolean
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gboolean default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoolean instance specifying a G_TYPE_BOOLEAN property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_int
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, gint minimum, gint maximum, gint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecInt instance specifying a G_TYPE_INT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_uint
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, guint minimum, guint maximum, guint default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecUInt instance specifying a G_TYPE_UINT property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            minimum :    minimum value for the property specified
 *            maximum :    maximum value for the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 * 
 * Name: g_param_spec_string
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, const gchar *default_value, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecString instance
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            default_value :  default value for the property specified
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_boxed
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType boxed_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_BOXED derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            boxed_type :  G_TYPE_BOXED derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *            
 * Name: g_param_spec_object
 * Params: const gchar *name, const gchar *nick, const gchar *blurb, GType object_type, GParamFlags flags
 * Returns: GParamSpec*
 * Assertion: Creates a new GParamSpecBoxed instance specifying a G_TYPE_OBJECT derived property.
 *            name :    canonical name of the property specified
 *            nick :   nick name for the property specified
 *            blurb :  description of the property specified
 *            object_type :     G_TYPE_OBJECT derived type of this property
 *            flags :  flags for the property specified
 *            Returns :    a newly created parameter specification
 *                                                                                         
 * Params Values: G_TYPE_INT64, 45000, g_value_set_int64, "SpecInt64"
 * Expected: PROP_INT64  
 */
static void test_purpose_21()
{
    #define VAL_TYPE    G_TYPE_INT64
    #define VAL_VAL     45000
    #define PROP_NAME   "SpecInt64"
    #define G_VAL_FUNC  g_value_set_int64
    #define EXP_FLAG    PROP_INT64
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, VAL_TYPE);
    G_VAL_FUNC(&g_val, VAL_VAL);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, PROP_NAME, &g_val);
    if (flag != EXP_FLAG){
        TEST_FAIL("g_param_spec_*: shoulde be %d", EXP_FLAG);
    }
    #undef VAL_TYPE
    #undef VAL_VAL
    #undef PROP_NAME
    #undef G_VAL_FUNC
    #undef EXP_FLAG
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * The description is above  
 */
static void test_purpose_22()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, G_TYPE_DOUBLE);
    g_value_set_double(&g_val, 10.5);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, "SpecDouble", &g_val);
    if (flag != PROP_DOUBLE){
        TEST_FAIL("g_param_spec_*: shoulde be %d", PROP_DOUBLE);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * The description is above  
 */
static void test_purpose_23()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue g_val = {0, };
    g_value_init(&g_val, G_TYPE_FLOAT);
    g_value_set_double(&g_val, 0.1);
    TestProperty* tp = test_property_new();
    g_object_set_property((GObject *)tp, "SpecFloat", &g_val);
    if (flag != PROP_FLOAT){
        TEST_FAIL("g_param_spec_*: shoulde be %d", PROP_FLOAT);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_dup_string
 * Params: const GValue *value
 * Returns: gchar*
 * Assertion: Get a copy the contents of a G_TYPE_STRING GValue.
 *            value :   a valid GValue of type G_TYPE_STRING
 *            Returns :   a newly allocated copy of the string content of value
 * Params Values: "Hello, LSB"
 * Expected: "Hello, LSB"  
 */
static void test_purpose_24()
{
    #define STR     "Hello, LSB"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GValue val = {0, };
    g_value_init(&val, G_TYPE_STRING);
    g_value_set_string(&val, STR);
    if (strcmp(g_value_dup_string(&val), STR) != 0){
        TEST_FAIL("g_value_dup_string: should be %s", STR);
    }
    #undef STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_value_set_static_string
 * Params: GValue *value, const gchar *v_string
 * Returns: void
 * Assertion: Set the contents of a G_TYPE_STRING GValue to v_string. The string is assumed to be static, and is thus not duplicated when setting the GValue.
 *            value :   a valid GValue of type G_TYPE_STRING
 *            v_string :  static string to be set
 * Params Values: "Hello, LSB"
 * Expected: "Hello, LSB"  
 */
static void test_purpose_25()
{
    #define STR     "Hello, LSB"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *test_string = g_strdup(STR);
    GValue val = {0, };
    g_value_init(&val, G_TYPE_STRING);
    g_value_set_static_string(&val, test_string);
    if (strcmp(g_value_get_string(&val), STR) != 0){
        TEST_FAIL("g_value_set_static_string: set failed", NULL);
    }
    else{
        test_string = g_ascii_strup(test_string, -1);
        if (strcmp(g_value_get_string(&val), STR) != 0){
            TEST_FAIL("g_value_set_static_string: static failed", NULL);
        }
    }
    #undef STR
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for Param_Value ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {NULL,0}
};
