/*
 * SXMLParser methods tests
 * Testing SXMLParser methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name: g_markup_escape_text
 * Params: const gchar *text, gssize length
 * Returns: gchar*
 * Assertion: Escapes text so that the markup parser will parse it verbatim. Less than, greater than, ampersand, etc. are replaced with the corresponding entities. This function would typically be used when writing out a file to be parsed with the markup parser.
 * Note that this function doesn't protect whitespace and line endings from being processed according to the XML rules for normalization of line endings and attribute values.
 *            text : some valid UTF-8 text
 *            length : length of text in bytes
 *            Returns : escaped text 
 * Params Values: text = "\" & < > \'"
 * Expected: "&quot; &amp; &lt; &gt; &apos;"  
 */
static void test_purpose_1()
{
    #define TEXT    "\" & < > \'"
    #define RETS    "&quot; &amp; &lt; &gt; &apos;"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *rets = NULL;
    gchar text[] = TEXT;
    
    rets = g_markup_escape_text(text, sizeof(text));
    if (strcmp(rets, RETS) != 0){
        TEST_FAIL("g_markup_escape_text: should be %s, now is %s", RETS, rets);
    }
    g_free(rets);
    
    #undef TEXT
    #undef RETS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_markup_escape_text
 * Params: const gchar *text, gssize length
 * Returns: gchar*
 * Assertion: Escapes text so that the markup parser will parse it verbatim. Less than, greater than, ampersand, etc. are replaced with the corresponding entities. This function would typically be used when writing out a file to be parsed with the markup parser.
 * Note that this function doesn't protect whitespace and line endings from being processed according to the XML rules for normalization of line endings and attribute values.
 *            text : some valid UTF-8 text
 *            length : length of text in bytes
 *            Returns : escaped text 
 * Params Values: text = "Hello, LSB"
 * Expected: "Hello, LSB"  
 */
static void test_purpose_2()
{
    #define TEXT    "Hello, LSB"
    #define RETS    "Hello, LSB"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *rets = NULL;
    gchar text[] = TEXT;
    
    rets = g_markup_escape_text(text, sizeof(text));
    if (strcmp(rets, RETS) != 0){
        TEST_FAIL("g_markup_escape_text: should be %s, now is %s", RETS, rets);
    }
    g_free(rets);
    
    #undef TEXT
    #undef RETS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_markup_printf_escaped
 * Params: const char *format, ...
 * Returns: gchar*
 * Assertion: Formats arguments according to format, escaping all string and character arguments in the fashion of g_markup_escape_text(). This is useful when you want to insert literal strings into XML-style markup output, without having to worry that the strings might themselves contain markup.
 *            format : printf() style format string
 *            ... : the arguments to insert in the format string
 *            Returns : newly allocated result from formatting operation. Free with g_free(). 
 * Params Values: format = "<result>%s</result>", "\" & < > \'"
 * Expected: "<result>&quot; &amp; &lt; &gt; &apos;</result>"  
 */
static void test_purpose_3()
{
    #define FORMAT    "<result>%s</result>", "\" & < > \'"
    #define RETS      "<result>&quot; &amp; &lt; &gt; &apos;</result>"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *rets = NULL;
    
    rets = g_markup_printf_escaped(FORMAT);
    if (strcmp(rets, RETS) != 0){
        TEST_FAIL("g_markup_printf_escaped: should be %s, now is %s", RETS, rets);
    }
    g_free(rets);
    #undef FORMAT
    #undef RETS
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for SXMLParser ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {NULL,0}
};
