/*
 * GtkTreeStore methods tests
 * Testing GtkTreeStore methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <float.h>
struct ListSort
{
  gint number1;
  gint number2;
};

struct ListSort data[] =
{
  { 3, 4 },
  { 2, 6 },
  { 0, 49 },
  { 856856, 965 },
  { 1, 34663 },
};

struct ListSort child_data[] =
{
  { 10, 149 },
  { 12, 13 },
  { 1856856, 1965 },
  { 11, 15 },
  { 14, 14 },
};

const gint NUM_TREE_ELEMS = 5;

va_list var_args;

void global_var_args_init (int prav_param, ...)
{
    va_start (var_args, prav_param);
}

global_valist (GtkTreeStore* tree_store, GtkTreeIter* iter, ...)
{
    va_list list;

    va_start (list, iter);
    gtk_tree_store_set_valist (tree_store, iter, list);
    va_end (list);
}


/*
 * Name: gtk_tree_store_new
 * Params: gint n_columns, ...
 * Returns: GtkTreeStore*
 * Assertion: Creates a new tree store as with n_columns columns each of the types passed in.
 * Params Values: 1, G_TYPE_UINT  
 * Expected: GtkTreeStore*  
 */
static void test_purpose_1()
{
    #define N_COLUMNS 1
    #define TYPES G_TYPE_UINT 
    
    GtkTreeStore *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_new (N_COLUMNS, TYPES);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef N_COLUMNS
    #undef TYPES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_new
 * Params: gint n_columns, ...
 * Returns: GtkTreeStore*
 * Assertion: Creates a new tree store as with n_columns columns each of the types passed in.
 * Params Values: 9, G_TYPE_UINT, G_TYPE_INT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING 
 * Expected: GtkTreeStore*  
 */
static void test_purpose_2()
{
    #define N_COLUMNS 9
    #define TYPES G_TYPE_UINT, G_TYPE_INT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING
    
    GtkTreeStore *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_new (N_COLUMNS, TYPES);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef N_COLUMNS
    #undef TYPES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_new
 * Params: gint n_columns, ...
 * Returns: GtkTreeStore*
 * Assertion: As an example, gtk_tree_store_new (3, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF); will create a new GtkTreeStore with three columns, of type int, string and GdkPixbuf respectively.
 * Params Values: 3, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF
 * Expected: GtkTreeStore*  
 */
static void test_purpose_3()
{
    GtkTreeStore *widget;
    GType type;
    G_CONST_RETURN gchar* type_name;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_new (3, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    type = GTK_TREE_STORE (widget)->column_headers[0];
    type_name = g_type_name (type);
    if (strcmp (type_name, "gint"))
    {
        TEST_FAIL ("0: type_name is '%s', should be 'gint'", type_name);
    }
    
    type = GTK_TREE_STORE (widget)->column_headers[1];
    type_name = g_type_name (type);
    if (strcmp (type_name, "gchararray"))
    {
        TEST_FAIL ("1: type_name is '%s', should be 'gchararray'", type_name);
    }
    
    type = GTK_TREE_STORE (widget)->column_headers[2];
    type_name = g_type_name (type);
    if (strcmp (type_name, "GdkPixbuf"))
    {
        TEST_FAIL ("2: type_name is '%s', should be 'GdkPixbuf'", type_name);
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_newv
 * Params: gint n_columns, GType *types
 * Returns: GtkTreeStore*
 * Assertion: Non vararg creation function.
 * Params Values: 1, {G_TYPE_UINT} 
 * Expected: GtkTreeStore*  
 */
static void test_purpose_4()
{
    #define N_COLUMNS 1
    #define TYPES {G_TYPE_UINT}
    
    GtkTreeStore *widget;
    GType types[] = TYPES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_newv (N_COLUMNS, types);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef N_COLUMNS
    #undef TYPES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_newv
 * Params: gint n_columns, GType *types
 * Returns: GtkTreeStore*
 * Assertion: Non vararg creation function.
 * Params Values: 9, {G_TYPE_UINT, G_TYPE_INT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING}    
 * Expected: GtkTreeStore*  
 */
static void test_purpose_5()
{
    #define N_COLUMNS 9
    #define TYPES {G_TYPE_UINT, G_TYPE_INT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING}   
    
    GtkTreeStore *widget;
    GType types[] = TYPES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_newv (N_COLUMNS, types);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef N_COLUMNS
    #undef TYPES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_value
 * Params: GtkTreeStore*, GtkTreeIter *iter, gint column, GValue *value
 * Returns: void
 * Assertion: Sets the data in the cell specified by iter and column.
 * Params Values: GtkTreeStore*, GtkTreeIter *iter, gint column, GValue *value 
 * Expected: void  
 */
static void test_purpose_6()
{
    #define CREATE_PARAMS 12, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF
    #define PATH "0:0:0"
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define COL_N 1
    #define VALUE_TYPE G_TYPE_STRING
    #define VALUE "str"
    
    GtkTreeStore *model;
    GtkTreeStore *widget;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint n_columns;
    GValue val = {0,};
    gint i;
    gint j;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_new (CREATE_PARAMS);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (widget, &iter, NULL);
            } else {
                gtk_tree_store_append (widget, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL (widget), &iter, gtk_tree_path_new_from_string(PATH)))
    {
        TEST_FAIL ("can't obtain iter", NULL); 
        return;
    }
    
    g_value_init(&val, VALUE_TYPE);
    if (VALUE_TYPE == G_TYPE_STRING)
    {
        g_value_set_string (&val, (char*) VALUE);
    } else if (VALUE_TYPE == G_TYPE_INT) {
        g_value_set_int (&val, (int) VALUE);
    } else {
        TEST_FAIL ("unexpected VALUE_TYPE, please extend the test", NULL);
    }
    
    gtk_tree_store_set_value(GTK_TREE_STORE (widget), &iter, COL_N, &val);
    
    #undef CREATE_PARAMS
    #undef COL_N
    #undef PATH
    #undef DEPTH
    #undef INDICES
    #undef VALUE_TYPE
    #undef VALUE
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_value
 * Params: GtkTreeStore*, GtkTreeIter *iter, gint column, GValue *value
 * Returns: void
 * Assertion: Sets the data in the cell specified by iter and column.
 * Params Values: GtkTreeStore*, GtkTreeIter *iter, gint column, GValue *value 
 * Expected: void  
 */
static void test_purpose_7()
{
    #define CREATE_PARAMS 3, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF
    #define PATH "5:7:3"
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define COL_N 0
    #define VALUE_TYPE G_TYPE_INT
    #define VALUE 49
    
    GtkTreeStore *model;
    GtkTreeStore *widget;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint n_columns;
    GValue val = {0,};
    gint i;
    gint j;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_store_new (CREATE_PARAMS);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (widget, &iter, NULL);
            } else {
                gtk_tree_store_append (widget, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL (widget), &iter, gtk_tree_path_new_from_string(PATH)))
    {
        TEST_FAIL ("can't obtain iter", NULL); 
        return;
    }
    
    g_value_init(&val, VALUE_TYPE);
    if (VALUE_TYPE == G_TYPE_STRING)
    {
        g_value_set_string (&val, (char*) VALUE);
    } else if (VALUE_TYPE == G_TYPE_INT) {
        g_value_set_int (&val, (int) VALUE);
    } else {
        TEST_FAIL ("unexpected VALUE_TYPE, please extend the test", NULL);
    }
    
    gtk_tree_store_set_value(GTK_TREE_STORE (widget), &iter, COL_N, &val);
    
    #undef CREATE_PARAMS
    #undef COL_N
    #undef PATH
    #undef DEPTH
    #undef INDICES
    #undef VALUE_TYPE
    #undef VALUE
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_remove
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: void
 * Assertion: Removes iter from tree_store. 
 * Params Values: newly created GtkTreeStore*, iter that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_remove
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: void
 * Assertion: After being removed, iter is set to the next valid row at that level, or invalidated if it previously pointed to the last one.
 * Params Values: newly created GtkTreeStore*, iter that is tree_store node 
 * Expected: void  
 */
static void test_purpose_8()
{
    #define N_ROWS 2
    #define ROW_NUM 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter saved_iter;
    GValue next_val = {0,};
    GValue val = {0,};
    guint uint_value;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    g_value_init (&next_val, G_TYPE_UINT);
    g_value_set_uint (&next_val, UINT_VALUE);
    for (i = 0; i < N_ROWS; i++)
    {
        gtk_tree_store_append (tree_store, &iter, NULL); 
        if (i == ROW_NUM)
        {
            saved_iter = iter;
        };
        if (i == (ROW_NUM + 1))
        {
            gtk_tree_store_set_value (tree_store, &iter, 0, &next_val);
        };
    }
    gtk_tree_store_remove (tree_store, &saved_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &saved_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("gtk_tree_store_remove () set iter in iter with value = %i, instead of %i.", uint_value, UINT_VALUE); 
    }
    
    #undef N_ROWS
    #undef ROW_NUM
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_remove
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: void
 * Assertion: Removes iter from tree_store. 
 * Params Values: newly created GtkTreeStore*, iter that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_remove
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: void
 * Assertion: After being removed, iter is set to the next valid row at that level, or invalidated if it previously pointed to the last one.
 * Params Values: newly created GtkTreeStore*, iter that is tree_store node 
 * Expected: void  
 */
static void test_purpose_9()
{
    #define N_ROWS 11
    #define ROW_NUM 7
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter saved_iter;
    GValue next_val = {0,};
    GValue val = {0,};
    guint uint_value;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    g_value_init (&next_val, G_TYPE_UINT);
    g_value_set_uint (&next_val, UINT_VALUE);
    for (i = 0; i < N_ROWS; i++)
    {
        gtk_tree_store_append (tree_store, &iter, NULL); 
        if (i == ROW_NUM)
        {
            saved_iter = iter;
        };
        if (i == (ROW_NUM + 1))
        {
            gtk_tree_store_set_value (tree_store, &iter, 0, &next_val);
        };
    }
    gtk_tree_store_remove (tree_store, &saved_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &saved_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("gtk_tree_store_remove () set iter in iter with value = %i, instead of %i.", uint_value, UINT_VALUE); 
    }
    
    #undef N_ROWS
    #undef ROW_NUM
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Creates a new row at position.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 0
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Otherwise, the row will be created at the toplevel. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 0
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: If position is larger than the number of rows at that level, then the new row will be inserted to the end of the list.
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 0
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: iter will be changed to point to this new row. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 0
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion:The row will be empty after this function is called.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 0
 * Expected: void  
 */
static void test_purpose_10()
{
    #define N_ROWS 1
    #define ROW_NUM 0
    #define UINT_VALUE 53184995
    
    GtkTreeModel *model;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter saved_iter;
    GValue inserted_val = {0,};
    GValue val = {0,};
    guint uint_value;
    guint i, row_num;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    row_num = (ROW_NUM >= N_ROWS)?(N_ROWS - 1):(ROW_NUM);
    for (i = 0; i < N_ROWS; i++)
    {
        gtk_tree_store_append (tree_store, &iter, NULL); 
        if (i == (row_num - 1))
        {
            saved_iter = iter;
        };
    }
    gtk_tree_store_insert (tree_store, &iter, NULL, ROW_NUM); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The inserted row is not empty!", NULL); 
    }
    g_value_unset (&val);
    
    gtk_tree_store_set_value (tree_store, &iter, 0, &inserted_val);
    if (row_num != 0)
    {
        gtk_tree_model_iter_next (GTK_TREE_MODEL (tree_store), &saved_iter);
    } else {
        gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &saved_iter, 
                                 gtk_tree_path_new_from_string ("0"));
    }
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &saved_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", row_num, uint_value, UINT_VALUE); 
    }
    #undef N_ROWS
    #undef ROW_NUM
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Creates a new row at position.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 7
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Otherwise, the row will be created at the toplevel. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 7
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: If position is larger than the number of rows at that level, then the new row will be inserted to the end of the list.
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 7
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: iter will be changed to point to this new row. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 7
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion:The row will be empty after this function is called.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 7
 * Expected: void  
 */
static void test_purpose_11()
{
    #define N_ROWS 11
    #define ROW_NUM 7
    #define UINT_VALUE 53184995
    
    GtkTreeModel *model;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter saved_iter;
    GValue inserted_val = {0,};
    GValue val = {0,};
    guint uint_value;
    guint i, row_num;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    row_num = (ROW_NUM >= N_ROWS)?(N_ROWS - 1):(ROW_NUM);
    for (i = 0; i < N_ROWS; i++)
    {
        gtk_tree_store_append (tree_store, &iter, NULL); 
        if (i == (row_num - 1))
        {
            saved_iter = iter;
        };
    }
    gtk_tree_store_insert (tree_store, &iter, NULL, ROW_NUM); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The inserted row is not empty!", NULL); 
    }
    g_value_unset (&val);
    
    gtk_tree_store_set_value (tree_store, &iter, 0, &inserted_val);
    if (row_num != 0)
    {
        gtk_tree_model_iter_next (GTK_TREE_MODEL (tree_store), &saved_iter);
    } else {
        gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &saved_iter, 
                                 gtk_tree_path_new_from_string ("0"));
    }
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &saved_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", row_num, uint_value, UINT_VALUE); 
    }
    #undef N_ROWS
    #undef ROW_NUM
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Creates a new row at position.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 6
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: Otherwise, the row will be created at the toplevel. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 6
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: If position is larger than the number of rows at that level, then the new row will be inserted to the end of the list.
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 6
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: iter will be changed to point to this new row. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 6
 * Expected: void
 * 
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion:The row will be empty after this function is called.  
 * Params Values: newly created GtkTreeStore*, uninitialized iter, iter that is tree_store node, 6
 * Expected: void  
 */
static void test_purpose_12()
{
    #define N_ROWS 7
    #define ROW_NUM 6
    #define UINT_VALUE 53184995
    
    GtkTreeModel *model;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter saved_iter;
    GValue inserted_val = {0,};
    GValue val = {0,};
    guint uint_value;
    guint i, row_num;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    row_num = (ROW_NUM >= N_ROWS)?(N_ROWS - 1):(ROW_NUM);
    for (i = 0; i < N_ROWS; i++)
    {
        gtk_tree_store_append (tree_store, &iter, NULL); 
        if (i == (row_num - 1))
        {
            saved_iter = iter;
        };
    }
    gtk_tree_store_insert (tree_store, &iter, NULL, ROW_NUM); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The inserted row is not empty!", NULL); 
    }
    g_value_unset (&val);
    
    gtk_tree_store_set_value (tree_store, &iter, 0, &inserted_val);
    if (row_num != 0)
    {
        gtk_tree_model_iter_next (GTK_TREE_MODEL (tree_store), &saved_iter);
    } else {
        gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &saved_iter, 
                                 gtk_tree_path_new_from_string ("0"));
    }
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &saved_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", row_num, uint_value, UINT_VALUE); 
    }
    #undef N_ROWS
    #undef ROW_NUM
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: If parent is non-NULL, then the row will be made a child of parent. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, 0
 * Expected: void  
 */
static void test_purpose_13()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define PARENT_LEVEL 0
    #define POSITION 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert (tree_store, &inserted_iter, &parent_iter, POSITION); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, gint position
 * Returns: void
 * Assertion: If parent is non-NULL, then the row will be made a child of parent. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, 1 
 * Expected: void  
 */
static void test_purpose_14()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_LEVEL 1
    #define POSITION 1 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert (tree_store, &inserted_iter, &parent_iter, POSITION); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: Inserts a new row before sibling. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void  
 */
static void test_purpose_15()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define PARENT_LEVEL 0
    #define POSITION 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
            if ((i == PARENT_LEVEL + 1) && (j == POSITION))
            {
                sibling_iter = child_iter;
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, &parent_iter, &sibling_iter); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: Inserts a new row before sibling. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void  
 */
static void test_purpose_16()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_LEVEL 1
    #define POSITION 1 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
            if ((i == PARENT_LEVEL + 1) && (j == POSITION))
            {
                sibling_iter = child_iter;
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, &parent_iter, &sibling_iter); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If sibling is NULL, then the row will be appended to parent 's children.'
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, NULL 
 * Expected: void  
 */
static void test_purpose_17()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_ITER_PATH_STR "1"
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter inserted_iter;
    GtkTreePath* parent_iter_path;
    GtkTreePath* inserted_iter_path;
    gint parent_iter_path_depth;
    gint inserted_iter_path_depth;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint indices[] = INDICES;
    gint i;
    gint j;
    gint uint_value;
    gchar* path_str;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (tree_store), &parent_iter, PARENT_ITER_PATH_STR))
    {
        TEST_FAIL ("can't get iter with path \"%s\"", PARENT_ITER_PATH_STR);
        return;
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, &parent_iter, NULL); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    parent_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &parent_iter);
    inserted_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &inserted_iter);
    if (!gtk_tree_path_is_descendant (inserted_iter_path, parent_iter_path))
    {
        TEST_FAIL ("inserted iter is not a descendant of parent iter", NULL);
        return;
    }
    
    parent_iter_path_depth = gtk_tree_path_get_depth (parent_iter_path);
    inserted_iter_path_depth = gtk_tree_path_get_depth (inserted_iter_path);
    if (parent_iter_path_depth + 1 != inserted_iter_path_depth)
    {
        TEST_FAIL ("parent_iter_path_depth = %d, inserted_iter_path_depth = %d, the correct equation is parent_iter_path_depth + 1 = inserted_iter_path_depth", parent_iter_path_depth, inserted_iter_path_depth);
    }
    
    #undef DEPTH
    #undef INDICES
    #undef PARENT_ITER_PATH_STR
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If sibling is NULL, then the row will be appended to parent 's children.'
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, NULL 
 * Expected: void  
 */
static void test_purpose_18()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_ITER_PATH_STR "5"
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter inserted_iter;
    GtkTreePath* parent_iter_path;
    GtkTreePath* inserted_iter_path;
    gint parent_iter_path_depth;
    gint inserted_iter_path_depth;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint indices[] = INDICES;
    gint i;
    gint j;
    gint uint_value;
    gchar* path_str;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (tree_store), &parent_iter, PARENT_ITER_PATH_STR))
    {
        TEST_FAIL ("can't get iter with path \"%s\"", PARENT_ITER_PATH_STR);
        return;
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, &parent_iter, NULL); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    parent_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &parent_iter);
    inserted_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &inserted_iter);
    if (!gtk_tree_path_is_descendant (inserted_iter_path, parent_iter_path))
    {
        TEST_FAIL ("inserted iter is not a descendant of parent iter", NULL);
        return;
    }
    
    parent_iter_path_depth = gtk_tree_path_get_depth (parent_iter_path);
    inserted_iter_path_depth = gtk_tree_path_get_depth (inserted_iter_path);
    if (parent_iter_path_depth + 1 != inserted_iter_path_depth)
    {
        TEST_FAIL ("parent_iter_path_depth = %d, inserted_iter_path_depth = %d, the correct equation is parent_iter_path_depth + 1 = inserted_iter_path_depth", parent_iter_path_depth, inserted_iter_path_depth);
    }
    
    #undef DEPTH
    #undef INDICES
    #undef PARENT_ITER_PATH_STR
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If sibling is NULL, then the row will be appended to parent 's children.'
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, NULL 
 * Expected: void  
 */
static void test_purpose_19()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_ITER_PATH_STR "5:7"
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter inserted_iter;
    GtkTreePath* parent_iter_path;
    GtkTreePath* inserted_iter_path;
    gint parent_iter_path_depth;
    gint inserted_iter_path_depth;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint indices[] = INDICES;
    gint i;
    gint j;
    gint uint_value;
    gchar* path_str;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (tree_store), &parent_iter, PARENT_ITER_PATH_STR))
    {
        TEST_FAIL ("can't get iter with path \"%s\"", PARENT_ITER_PATH_STR);
        return;
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, &parent_iter, NULL); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    parent_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &parent_iter);
    inserted_iter_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &inserted_iter);
    if (!gtk_tree_path_is_descendant (inserted_iter_path, parent_iter_path))
    {
        TEST_FAIL ("inserted iter is not a descendant of parent iter", NULL);
        return;
    }
    
    parent_iter_path_depth = gtk_tree_path_get_depth (parent_iter_path);
    inserted_iter_path_depth = gtk_tree_path_get_depth (inserted_iter_path);
    if (parent_iter_path_depth + 1 != inserted_iter_path_depth)
    {
        TEST_FAIL ("parent_iter_path_depth = %d, inserted_iter_path_depth = %d, the correct equation is parent_iter_path_depth + 1 = inserted_iter_path_depth", parent_iter_path_depth, inserted_iter_path_depth);
    }
    
    #undef DEPTH
    #undef INDICES
    #undef PARENT_ITER_PATH_STR
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_before
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If parent and sibling are NULL, then the row will be appended to the toplevel.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, NULL, NULL 
 * Expected: void  
 */
static void test_purpose_20()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define CORRECT_PATH_STR "6"
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gchar* path_str;
    gchar* buf = calloc (1, 255);
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_before (tree_store, &inserted_iter, NULL, NULL); 
    
    path_str = gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &inserted_iter));
    if (strcmp (path_str, CORRECT_PATH_STR))
    {
        TEST_FAIL ("the row was not added to the toplevel, path = \"%s\", should be \"%s\"", path_str, CORRECT_PATH_STR);
    }
    
    #undef DEPTH
    #undef INDICES
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: Inserts a new row after sibling.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If sibling is NULL, then the row will be prepended to parent 's children.'
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void  
 */
static void test_purpose_21()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define PARENT_LEVEL 0
    #define POSITION 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
            if ((i == PARENT_LEVEL + 1) && (j == POSITION))
            {
                sibling_iter = child_iter;
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_after (tree_store, &inserted_iter, &parent_iter, &sibling_iter); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION + 1);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION + 1, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: Inserts a new row after sibling.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void
 * 
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If sibling is NULL, then the row will be prepended to parent 's children.'
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node, GtkTreeIter* that is child of *parent 
 * Expected: void  
 */
static void test_purpose_22()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_LEVEL 1
    #define POSITION 1 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
            if ((i == PARENT_LEVEL + 1) && (j == POSITION))
            {
                sibling_iter = child_iter;
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_after (tree_store, &inserted_iter, &parent_iter, &sibling_iter); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, POSITION + 1);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", POSITION + 1, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef POSITION
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_insert_after
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling 
 * Returns: void
 * Assertion: If parent and sibling are NULL, then the row will be prepended to the toplevel.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, NULL, NULL 
 * Expected: void  
 */
static void test_purpose_23()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter inserted_iter;
    GValue inserted_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    g_value_init (&inserted_val, G_TYPE_UINT);
    g_value_set_uint (&inserted_val, UINT_VALUE);
    gtk_tree_store_insert_after (tree_store, &inserted_iter, NULL, NULL); 
    gtk_tree_store_set_value (tree_store, &inserted_iter, 0, &inserted_val);
    
    gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &iter, 
                             gtk_tree_path_new_from_string ("0"));
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", 0, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: Prepends a new row to tree_store.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will prepend the new row before the first child of parent, otherwise it will prepend a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: The row will be empty after this function is called. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void  
 */
static void test_purpose_24()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define PARENT_LEVEL 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter prepended_iter;
    GValue prepended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    gtk_tree_store_prepend (tree_store, &prepended_iter, &parent_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &prepended_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The prepended row is not empty!", 0, uint_value, UINT_VALUE); 
    }
    g_value_unset (&val);
    
    g_value_init (&prepended_val, G_TYPE_UINT);
    g_value_set_uint (&prepended_val, UINT_VALUE);
    gtk_tree_store_set_value (tree_store, &prepended_iter, 0, &prepended_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, 0);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", 0, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: Prepends a new row to tree_store.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will prepend the new row before the first child of parent, otherwise it will prepend a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: The row will be empty after this function is called. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void  
 */
static void test_purpose_25()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_LEVEL 2
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter prepended_iter;
    GValue prepended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    gtk_tree_store_prepend (tree_store, &prepended_iter, &parent_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &prepended_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The prepended row is not empty!", 0, uint_value, UINT_VALUE); 
    }
    g_value_unset (&val);
    
    g_value_init (&prepended_val, G_TYPE_UINT);
    g_value_set_uint (&prepended_val, UINT_VALUE);
    gtk_tree_store_set_value (tree_store, &prepended_iter, 0, &prepended_val);
    
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, 0);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", 0, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_prepend
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will prepend the new row before the first child of parent, otherwise it will prepend a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, NULL
 * Expected: void  
 */
static void test_purpose_26()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter prepended_iter;
    GValue prepended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    g_value_init (&prepended_val, G_TYPE_UINT);
    g_value_set_uint (&prepended_val, UINT_VALUE);
    gtk_tree_store_prepend (tree_store, &prepended_iter, NULL); 
    gtk_tree_store_set_value (tree_store, &prepended_iter, 0, &prepended_val);
    
    gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &iter, 
                             gtk_tree_path_new_from_string ("0"));
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", 0, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: Appends a new row to tree_store.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will append the new row after the last child of parent, otherwise it will append a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: The row will be empty after this function is called.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void  
 */
static void test_purpose_27()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define PARENT_LEVEL 0
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter appended_iter;
    GValue appended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint children_num;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    
    gtk_tree_store_append (tree_store, &appended_iter, &parent_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &appended_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The appended row is not empty!", NULL); 
    }
    g_value_unset (&val);
    
    g_value_init (&appended_val, G_TYPE_UINT);
    g_value_set_uint (&appended_val, UINT_VALUE);
    gtk_tree_store_set_value (tree_store, &appended_iter, 0, &appended_val);
    
    children_num = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store), 
                                                   &parent_iter);
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, 
                                   children_num - 1);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", children_num, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: Appends a new row to tree_store.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will append the new row after the last child of parent, otherwise it will append a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: iter will be changed to point to this new row.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void
 * 
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: The row will be empty after this function is called.
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, GtkTreeIter* that is tree_store node 
 * Expected: void  
 */
static void test_purpose_28()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PARENT_LEVEL 2
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter parent_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter appended_iter;
    GValue appended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint children_num;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == PARENT_LEVEL)
        {
            parent_iter = iter;
        }
    }
    
    gtk_tree_store_append (tree_store, &appended_iter, &parent_iter); 
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &appended_iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    if (uint_value != 0)
    {
        TEST_FAIL ("The appended row is not empty!", NULL); 
    }
    g_value_unset (&val);
    
    g_value_init (&appended_val, G_TYPE_UINT);
    g_value_set_uint (&appended_val, UINT_VALUE);
    gtk_tree_store_set_value (tree_store, &appended_iter, 0, &appended_val);
    
    children_num = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store), 
                                                   &parent_iter);
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, &parent_iter, 
                                   children_num - 1);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", children_num, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef PARENT_LEVEL
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_append
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *parent 
 * Returns: void
 * Assertion: If parent is non-NULL, then it will append the new row after the last child of parent, otherwise it will append a row to the top level. 
 * Params Values: GtkTreeStore*, uninitialized GtkTreeIter*, NULL
 * Expected: void  
 */
static void test_purpose_29()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define UINT_VALUE 53184995
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter sibling_iter;
    GtkTreeIter appended_iter;
    GValue appended_val = {0,};
    GValue val = {0, };
    gint i;
    gint j;
    gint uint_value;
    gint children_num;
    gint indices[] = INDICES;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    g_value_init (&appended_val, G_TYPE_UINT);
    g_value_set_uint (&appended_val, UINT_VALUE);
    gtk_tree_store_append (tree_store, &appended_iter, NULL); 
    gtk_tree_store_set_value (tree_store, &appended_iter, 0, &appended_val);
    
    children_num = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store), NULL);
    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree_store), &iter, NULL, 
                                   children_num - 1);
    gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store), &iter, 0, &val);
    uint_value = g_value_get_uint (&val);
    
    if (uint_value != UINT_VALUE)
    {
        TEST_FAIL ("At position %i iter value = %i, instead of %i.", 0, uint_value, UINT_VALUE); 
    }
    #undef DEPTH
    #undef INDICES
    #undef UINT_VALUE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_is_ancestor
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *descendant 
 * Returns: gboolean
 * Assertion: Returns TRUE if iter is an ancestor of descendant.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*, initialized GtkTreeIter* 
 * Expected: TRUE   
 */
static void test_purpose_30()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define ANCESTOR_LEVEL 0
    #define DESCENDANT_LEVEL 2
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter ancestor_iter;
    GtkTreeIter descendant_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gboolean is_ancestor;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ANCESTOR_LEVEL)
        {
            ancestor_iter = iter;
        }
    
        if (i == DESCENDANT_LEVEL)
        {
            descendant_iter = iter;
        }
    }
    is_ancestor = gtk_tree_store_is_ancestor (tree_store, &ancestor_iter, &descendant_iter);
    if (!is_ancestor)
    {
        TEST_FAIL ("gtk_tree_store_is_ancestor () return FALSE instead of TRUE", NULL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ANCESTOR_LEVEL
    #undef DESCENDANT_LEVEL 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_is_ancestor
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *descendant 
 * Returns: gboolean
 * Assertion: Returns TRUE if iter is an ancestor of descendant.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*, initialized GtkTreeIter* 
 * Expected: TRUE   
 */
static void test_purpose_31()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define ANCESTOR_LEVEL 1
    #define DESCENDANT_LEVEL 2
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter ancestor_iter;
    GtkTreeIter descendant_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gboolean is_ancestor;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ANCESTOR_LEVEL)
        {
            ancestor_iter = iter;
        }
    
        if (i == DESCENDANT_LEVEL)
        {
            descendant_iter = iter;
        }
    }
    is_ancestor = gtk_tree_store_is_ancestor (tree_store, &ancestor_iter, &descendant_iter);
    if (!is_ancestor)
    {
        TEST_FAIL ("gtk_tree_store_is_ancestor () return FALSE instead of TRUE", NULL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ANCESTOR_LEVEL
    #undef DESCENDANT_LEVEL 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_is_ancestor
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *descendant 
 * Returns: gboolean
 * Assertion: Returns TRUE if iter is an ancestor of descendant.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*, initialized GtkTreeIter* 
 * Expected: FALSE   
 */
static void test_purpose_32()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define ANCESTOR_LEVEL 2
    #define DESCENDANT_LEVEL 0
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter ancestor_iter;
    GtkTreeIter descendant_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gboolean is_ancestor;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ANCESTOR_LEVEL)
        {
            ancestor_iter = iter;
        }
    
        if (i == DESCENDANT_LEVEL)
        {
            descendant_iter = iter;
        }
    }
    is_ancestor = gtk_tree_store_is_ancestor (tree_store, &ancestor_iter, &descendant_iter);
    if (is_ancestor)
    {
        TEST_FAIL ("gtk_tree_store_is_ancestor () return TRUE instead of FALSE.", NULL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ANCESTOR_LEVEL
    #undef DESCENDANT_LEVEL 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_is_ancestor
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, GtkTreeIter *descendant 
 * Returns: gboolean
 * Assertion: Returns TRUE if iter is an ancestor of descendant.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*, initialized GtkTreeIter* 
 * Expected: FALSE   
 */
static void test_purpose_33()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define ANCESTOR_LEVEL 2
    #define DESCENDANT_LEVEL 1
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter ancestor_iter;
    GtkTreeIter descendant_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gboolean is_ancestor;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ANCESTOR_LEVEL)
        {
            ancestor_iter = iter;
        }
    
        if (i == DESCENDANT_LEVEL)
        {
            descendant_iter = iter;
        }
    }
    is_ancestor = gtk_tree_store_is_ancestor (tree_store, &ancestor_iter, &descendant_iter);
    if (is_ancestor)
    {
        TEST_FAIL ("gtk_tree_store_is_ancestor () return TRUE instead of FALSE.", NULL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ANCESTOR_LEVEL
    #undef DESCENDANT_LEVEL 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_depth
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter
 * Returns: gint
 * Assertion: Returns the depth of iter.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*
 * Expected: 0  
 */
static void test_purpose_34()
{
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define ITER_LEVEL 0
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter saved_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gint iter_depth;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ITER_LEVEL)
        {
            saved_iter = iter;
        }
    }
    iter_depth = gtk_tree_store_iter_depth (tree_store, &saved_iter);
    if (iter_depth != ITER_LEVEL)
    {
        TEST_FAIL ("gtk_tree_store_iter_depth () return %i instead of %i.", iter_depth, ITER_LEVEL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ITER_LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_depth
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter
 * Returns: gint
 * Assertion: Returns the depth of iter.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter*
 * Expected: 2  
 */
static void test_purpose_35()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define ITER_LEVEL 2
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter saved_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gint iter_depth;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
        if (i == ITER_LEVEL)
        {
            saved_iter = iter;
        }
    }
    iter_depth = gtk_tree_store_iter_depth (tree_store, &saved_iter);
    if (iter_depth != ITER_LEVEL)
    {
        TEST_FAIL ("gtk_tree_store_iter_depth () return %i instead of %i.", iter_depth, ITER_LEVEL); 
    }
    #undef DEPTH
    #undef INDICES
    #undef ITER_LEVEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_clear
 * Params: GtkTreeStore* tree_store
 * Returns: void
 * Assertion: Removes all rows from tree_store
 * Params Values: newly created GtkTreeStore*
 * Expected: void  
 */
static void test_purpose_36()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint i;
    gint j;
    gint indices[] = INDICES;
    gint root_children_num;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    } 
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    gtk_tree_store_clear (tree_store);
    root_children_num = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store), NULL);
    if (root_children_num > 0)
    {
        TEST_FAIL ("gtk_tree_store_clear () doesn`t work.", NULL); 
    }
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_column_types
 * Params: GtkTreeStore *tree_store, gint n_columns, GType *types
 * Returns: void
 * Assertion: This function is meant primarily for GObjects that inherit from GtkTreeStore, and should only be used when constructing a new GtkTreeStore.
 * Params Values: a newly created GtkTreeStore*, 1, "gchar"
 * Expected: void  
 */
static void test_purpose_37()
{
    #define COLUMNS_N_AND_TYPES 2, G_TYPE_DOUBLE,G_TYPE_CHAR
    #define COLUMNS_TYPES "G_TYPE_DOUBLE,G_TYPE_CHAR"
    #define NEW_COLUMNS_N 1
    #define NEW_COLUMNS_TYPES "gchar"
    #define NEW_COLUMNS_TYPES_STR "G_TYPE_CHAR"
    
    GtkTreeStore* store;
    gchar* tp;
    gint i = 0;
    GType new_types[NEW_COLUMNS_N];
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* new_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    gchar* new_real_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (new_columns_types, NEW_COLUMNS_TYPES, strlen (NEW_COLUMNS_TYPES_STR));
    memcpy (new_real_columns_types, NEW_COLUMNS_TYPES_STR, strlen (NEW_COLUMNS_TYPES_STR));
    
    store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    tp = (char*)strtok (new_columns_types, ",");
    for (i = 0; tp != NULL && i <= NEW_COLUMNS_N; i++)
    {  
        new_types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    gtk_tree_store_set_column_types (store, NEW_COLUMNS_N, &new_types[0]);
    
    tp = (char*)strtok (new_real_columns_types, ",");
    for (i = 0; tp != NULL; i++)
    {  
        GType col_type 
            = gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), i);
    
        if (col_type == G_TYPE_CHAR)
        {
            if (strcmp (tp, "G_TYPE_CHAR"))
            {
                TEST_FAIL ("i = %d. col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_DOUBLE) {
            if (strcmp (tp, "G_TYPE_DOUBLE"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_BOOLEAN) {
            if (strcmp (tp, "G_TYPE_BOOLEAN"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else {
            TEST_FAIL ("unexpected col_type, please extend the test", NULL);
        }
    
        tp = (char*) strtok (NULL, ",");
    }
    
    #undef COLUMNS_N_AND_TYPES
    #undef COLUMNS_TYPES
    #undef NEW_COLUMNS_N
    #undef NEW_COLUMNS_TYPES
    #undef NEW_COLUMNS_TYPES_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_column_types
 * Params: GtkTreeStore *tree_store, gint n_columns, GType *types
 * Returns: void
 * Assertion: This function is meant primarily for GObjects that inherit from GtkTreeStore, and should only be used when constructing a new GtkTreeStore.
 * Params Values: a newly created GtkTreeStore*, 2, "gchar,gdouble"
 * Expected: void  
 */
static void test_purpose_38()
{
    #define COLUMNS_N_AND_TYPES 2, G_TYPE_DOUBLE,G_TYPE_CHAR
    #define COLUMNS_TYPES "G_TYPE_DOUBLE,G_TYPE_CHAR"
    #define NEW_COLUMNS_N 2
    #define NEW_COLUMNS_TYPES "gchar,gdouble"
    #define NEW_COLUMNS_TYPES_STR "G_TYPE_CHAR,G_TYPE_DOUBLE"
    
    GtkTreeStore* store;
    gchar* tp;
    gint i = 0;
    GType new_types[NEW_COLUMNS_N];
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* new_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    gchar* new_real_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (new_columns_types, NEW_COLUMNS_TYPES, strlen (NEW_COLUMNS_TYPES_STR));
    memcpy (new_real_columns_types, NEW_COLUMNS_TYPES_STR, strlen (NEW_COLUMNS_TYPES_STR));
    
    store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    tp = (char*)strtok (new_columns_types, ",");
    for (i = 0; tp != NULL && i <= NEW_COLUMNS_N; i++)
    {  
        new_types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    gtk_tree_store_set_column_types (store, NEW_COLUMNS_N, &new_types[0]);
    
    tp = (char*)strtok (new_real_columns_types, ",");
    for (i = 0; tp != NULL; i++)
    {  
        GType col_type 
            = gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), i);
    
        if (col_type == G_TYPE_CHAR)
        {
            if (strcmp (tp, "G_TYPE_CHAR"))
            {
                TEST_FAIL ("i = %d. col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_DOUBLE) {
            if (strcmp (tp, "G_TYPE_DOUBLE"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_BOOLEAN) {
            if (strcmp (tp, "G_TYPE_BOOLEAN"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else {
            TEST_FAIL ("unexpected col_type, please extend the test", NULL);
        }
    
        tp = (char*) strtok (NULL, ",");
    }
    
    #undef COLUMNS_N_AND_TYPES
    #undef COLUMNS_TYPES
    #undef NEW_COLUMNS_N
    #undef NEW_COLUMNS_TYPES
    #undef NEW_COLUMNS_TYPES_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_column_types
 * Params: GtkTreeStore *tree_store, gint n_columns, GType *types
 * Returns: void
 * Assertion: This function is meant primarily for GObjects that inherit from GtkTreeStore, and should only be used when constructing a new GtkTreeStore.
 * Params Values: a newly created GtkTreeStore*, 3, "gchar,gdouble,gboolean"
 * Expected: void  
 */
static void test_purpose_39()
{
    #define COLUMNS_N_AND_TYPES 2, G_TYPE_DOUBLE,G_TYPE_CHAR
    #define COLUMNS_TYPES "G_TYPE_DOUBLE,G_TYPE_CHAR"
    #define NEW_COLUMNS_N 3
    #define NEW_COLUMNS_TYPES "gchar,gdouble,gboolean"
    #define NEW_COLUMNS_TYPES_STR "G_TYPE_CHAR,G_TYPE_DOUBLE,G_TYPE_BOOLEAN"
    
    GtkTreeStore* store;
    gchar* tp;
    gint i = 0;
    GType new_types[NEW_COLUMNS_N];
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* new_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    gchar* new_real_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (new_columns_types, NEW_COLUMNS_TYPES, strlen (NEW_COLUMNS_TYPES_STR));
    memcpy (new_real_columns_types, NEW_COLUMNS_TYPES_STR, strlen (NEW_COLUMNS_TYPES_STR));
    
    store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    tp = (char*)strtok (new_columns_types, ",");
    for (i = 0; tp != NULL && i <= NEW_COLUMNS_N; i++)
    {  
        new_types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    gtk_tree_store_set_column_types (store, NEW_COLUMNS_N, &new_types[0]);
    
    tp = (char*)strtok (new_real_columns_types, ",");
    for (i = 0; tp != NULL; i++)
    {  
        GType col_type 
            = gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), i);
    
        if (col_type == G_TYPE_CHAR)
        {
            if (strcmp (tp, "G_TYPE_CHAR"))
            {
                TEST_FAIL ("i = %d. col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_DOUBLE) {
            if (strcmp (tp, "G_TYPE_DOUBLE"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_BOOLEAN) {
            if (strcmp (tp, "G_TYPE_BOOLEAN"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else {
            TEST_FAIL ("unexpected col_type, please extend the test", NULL);
        }
    
        tp = (char*) strtok (NULL, ",");
    }
    
    #undef COLUMNS_N_AND_TYPES
    #undef COLUMNS_TYPES
    #undef NEW_COLUMNS_N
    #undef NEW_COLUMNS_TYPES
    #undef NEW_COLUMNS_TYPES_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_column_types
 * Params: GtkTreeStore *tree_store, gint n_columns, GType *types
 * Returns: void
 * Assertion: This function is meant primarily for GObjects that inherit from GtkTreeStore, and should only be used when constructing a new GtkTreeStore.
 * Params Values: a newly created GtkTreeStore*, 2, "gchar,gboolean"
 * Expected: void  
 */
static void test_purpose_40()
{
    #define COLUMNS_N_AND_TYPES 3, G_TYPE_DOUBLE,G_TYPE_CHAR, G_TYPE_INT
    #define COLUMNS_TYPES "G_TYPE_DOUBLE,G_TYPE_CHAR,G_TYPE_INT"
    #define NEW_COLUMNS_N 2
    #define NEW_COLUMNS_TYPES "gchar,gboolean"
    #define NEW_COLUMNS_TYPES_STR "G_TYPE_CHAR,G_TYPE_BOOLEAN"
    
    GtkTreeStore* store;
    gchar* tp;
    gint i = 0;
    GType new_types[NEW_COLUMNS_N];
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* new_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    gchar* new_real_columns_types = calloc (1, strlen (NEW_COLUMNS_TYPES_STR) + 1);
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (new_columns_types, NEW_COLUMNS_TYPES, strlen (NEW_COLUMNS_TYPES_STR));
    memcpy (new_real_columns_types, NEW_COLUMNS_TYPES_STR, strlen (NEW_COLUMNS_TYPES_STR));
    
    store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    tp = (char*)strtok (new_columns_types, ",");
    for (i = 0; tp != NULL && i <= NEW_COLUMNS_N; i++)
    {  
        new_types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    gtk_tree_store_set_column_types (store, NEW_COLUMNS_N, &new_types[0]);
    
    tp = (char*)strtok (new_real_columns_types, ",");
    for (i = 0; tp != NULL; i++)
    {  
        GType col_type 
            = gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), i);
    
        if (col_type == G_TYPE_CHAR)
        {
            if (strcmp (tp, "G_TYPE_CHAR"))
            {
                TEST_FAIL ("i = %d. col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_DOUBLE) {
            if (strcmp (tp, "G_TYPE_DOUBLE"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else if (col_type == G_TYPE_BOOLEAN) {
            if (strcmp (tp, "G_TYPE_BOOLEAN"))
            {
                TEST_FAIL ("i = %d, col_type is %d, tp is %s", 
                           i - 1, col_type, tp);
            }
        } else {
            TEST_FAIL ("unexpected col_type, please extend the test", NULL);
        }
    
        tp = (char*) strtok (NULL, ",");
    }
    
    #undef COLUMNS_N_AND_TYPES
    #undef COLUMNS_TYPES
    #undef NEW_COLUMNS_N
    #undef NEW_COLUMNS_TYPES
    #undef NEW_COLUMNS_TYPES_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, ...
 * Returns: void
 * Assertion: Sets the value of one or more cells in the row referenced by iter. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, "G_TYPE_UINT",GValue*
 * Expected: void  
 */
static void test_purpose_41()
{
    #define COLUMNS_N 1
    #define COLUMNS_TYPES "G_TYPE_UINT"
    #define COLUMNS_N_AND_TYPES 1,G_TYPE_UINT 
    #define DATA 0, 100, -1
    #define COLUMNS_VALUES "100"
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GValue val = {0,};
    GValue nval = {0,};
    guint i;
    gchar* tp;
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* columns_values = calloc (1, strlen (COLUMNS_VALUES) + 1);
    GType types[COLUMNS_N];
    GValue values[COLUMNS_N];
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (columns_types, COLUMNS_TYPES, strlen (COLUMNS_TYPES));
    memcpy (columns_values, COLUMNS_VALUES, strlen (COLUMNS_VALUES));
    
    
    tree_store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!tree_store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_tree_store_append (tree_store, &iter, NULL); 
    
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    tp = (char*)strtok (columns_values, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            g_value_set_int (&values[i],atoi(tp));
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            g_value_set_double (&values[i],atof(tp));
           
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            g_value_set_long (&values[i],atol(tp));
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            g_value_set_string (&values[i],tp);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            g_value_set_uint (&values[i],(u_int)atoi(tp));
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            g_value_set_char (&values[i],atoi(tp));
        } 
        tp = (char*) strtok (NULL, ",");
    }
    
    
    gtk_tree_store_set (tree_store, &iter, DATA);
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
    
            if (g_value_get_int (&val) != g_value_get_int (&values[i]))
            {
                TEST_FAIL ("Int Value is '%d',should be '%d'",g_value_get_int (&val),g_value_get_int (&values[i]));
               
            }
            g_value_unset (&val);
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("long Value is '%d',should be '%d'",g_value_get_long (&val),g_value_get_long (&values[i]));
               
            }
            g_value_unset (&val);
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("Long Value is '%d',should be '%d'",g_value_get_double (&val),g_value_get_double (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_string (&val) != g_value_get_string (&values[i]))
            {
                TEST_FAIL ("String Value is '%d',should be '%d'",g_value_get_string (&val),g_value_get_string (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_uint (&val) != g_value_get_uint (&values[i]))
            {
                TEST_FAIL ("Uint Value is '%d',should be '%d'",g_value_get_uint (&val),g_value_get_uint (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_char (&val) != g_value_get_char (&values[i]))
            {
                TEST_FAIL ("Char Value is '%d',should be '%d'",g_value_get_char (&val),g_value_get_char (&values[i]));
               
            }
            g_value_unset (&val);
            
        } 
        
    }
    
    #undef COLUMNS_N
    #undef COLUMNS_TYPES 
    #undef COLUMNS_N_AND_TYPES
    #undef DATA 
    #undef COLUMNS_VALUES 
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, ...
 * Returns: void
 * Assertion: Sets the value of one or more cells in the row referenced by iter. 
 * Params Values: newly created GtkTreeStore*, uninitialized iter, "G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING",GValue*
 * Expected: void  
 */
static void test_purpose_42()
{
    #define COLUMNS_N 6
    #define COLUMNS_TYPES "G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING"
    #define COLUMNS_N_AND_TYPES 6,G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING
    #define DATA 0, G_MAXINT, 1, '\0', 2, TRUE, 3, G_MINFLOAT, 4, G_MAXDOUBLE, 5, "\n\t_b", -1
    #define COLUMNS_VALUES "G_MAXINT,'\0',TRUE,G_MINFLOAT,G_MAXDOUBLE,\"\n\t_b\""
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GValue val = {0,};
    GValue nval = {0,};
    guint i;
    gchar* tp;
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* columns_values = calloc (1, strlen (COLUMNS_VALUES) + 1);
    GType types[COLUMNS_N];
    GValue values[COLUMNS_N];
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (columns_types, COLUMNS_TYPES, strlen (COLUMNS_TYPES));
    memcpy (columns_values, COLUMNS_VALUES, strlen (COLUMNS_VALUES));
    
    
    tree_store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!tree_store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_tree_store_append (tree_store, &iter, NULL); 
    
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    tp = (char*)strtok (columns_values, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            g_value_set_int (&values[i],atoi(tp));
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            g_value_set_double (&values[i],atof(tp));
           
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            g_value_set_long (&values[i],atol(tp));
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            g_value_set_string (&values[i],tp);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            g_value_set_uint (&values[i],(u_int)atoi(tp));
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            g_value_set_char (&values[i],atoi(tp));
        } 
        tp = (char*) strtok (NULL, ",");
    }
    
    
    gtk_tree_store_set (tree_store, &iter, DATA);
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
    
            if (g_value_get_int (&val) != g_value_get_int (&values[i]))
            {
                TEST_FAIL ("Int Value is '%d',should be '%d'",g_value_get_int (&val),g_value_get_int (&values[i]));
               
            }
            g_value_unset (&val);
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("long Value is '%d',should be '%d'",g_value_get_long (&val),g_value_get_long (&values[i]));
               
            }
            g_value_unset (&val);
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("Long Value is '%d',should be '%d'",g_value_get_double (&val),g_value_get_double (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_string (&val) != g_value_get_string (&values[i]))
            {
                TEST_FAIL ("String Value is '%d',should be '%d'",g_value_get_string (&val),g_value_get_string (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_uint (&val) != g_value_get_uint (&values[i]))
            {
                TEST_FAIL ("Uint Value is '%d',should be '%d'",g_value_get_uint (&val),g_value_get_uint (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_char (&val) != g_value_get_char (&values[i]))
            {
                TEST_FAIL ("Char Value is '%d',should be '%d'",g_value_get_char (&val),g_value_get_char (&values[i]));
               
            }
            g_value_unset (&val);
            
        } 
        
    }
    
    #undef COLUMNS_N
    #undef COLUMNS_TYPES 
    #undef COLUMNS_N_AND_TYPES
    #undef DATA 
    #undef COLUMNS_VALUES 
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_valist
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, va_list var_args
 * Returns: void
 * Assertion: See gtk_tree_store_set(); this version takes a va_list for use by language bindings.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter *iter, 1,G_TYPE_UINT 
 * Expected: void  
 */
static void test_purpose_43()
{
    #define COLUMNS_N 1
    #define COLUMNS_TYPES "G_TYPE_UINT"
    #define COLUMNS_N_AND_TYPES 1,G_TYPE_UINT 
    #define DATA 0, 100, -1
    #define COLUMNS_VALUES "100"
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GValue val = {0,};
    GValue nval = {0,};
    guint i;
    gchar* tp;
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* columns_values = calloc (1, strlen (COLUMNS_VALUES) + 1);
    GType types[COLUMNS_N];
    GValue values[COLUMNS_N];
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (columns_types, COLUMNS_TYPES, strlen (COLUMNS_TYPES));
    memcpy (columns_values, COLUMNS_VALUES, strlen (COLUMNS_VALUES));
    
    
    tree_store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!tree_store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_tree_store_append (tree_store, &iter, NULL); 
    
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    tp = (char*)strtok (columns_values, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            g_value_set_int (&values[i],atoi(tp));
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            g_value_set_double (&values[i],atof(tp));
           
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            g_value_set_long (&values[i],atol(tp));
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            g_value_set_string (&values[i],tp);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            g_value_set_uint (&values[i],(u_int)atoi(tp));
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            g_value_set_char (&values[i],atoi(tp));
        } 
        tp = (char*) strtok (NULL, ",");
    }
    
    
    global_valist (tree_store, &iter, DATA);
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
    
            if (g_value_get_int (&val) != g_value_get_int (&values[i]))
            {
                TEST_FAIL ("Int Value is '%d',should be '%d'",g_value_get_int (&val),g_value_get_int (&values[i]));
               
            }
            g_value_unset (&val);
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("long Value is '%d',should be '%d'",g_value_get_long (&val),g_value_get_long (&values[i]));
               
            }
            g_value_unset (&val);
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("Long Value is '%d',should be '%d'",g_value_get_double (&val),g_value_get_double (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_string (&val) != g_value_get_string (&values[i]))
            {
                TEST_FAIL ("String Value is '%d',should be '%d'",g_value_get_string (&val),g_value_get_string (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_uint (&val) != g_value_get_uint (&values[i]))
            {
                TEST_FAIL ("Uint Value is '%d',should be '%d'",g_value_get_uint (&val),g_value_get_uint (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_char (&val) != g_value_get_char (&values[i]))
            {
                TEST_FAIL ("Char Value is '%d',should be '%d'",g_value_get_char (&val),g_value_get_char (&values[i]));
               
            }
            g_value_unset (&val);
            
        } 
        
    }
    
    #undef COLUMNS_N
    #undef COLUMNS_TYPES 
    #undef COLUMNS_N_AND_TYPES
    #undef DATA 
    #undef COLUMNS_VALUES 
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_set_valist
 * Params: GtkTreeStore* tree_store, GtkTreeIter *iter, va_list var_args
 * Returns: void
 * Assertion: See gtk_tree_store_set(); this version takes a va_list for use by language bindings.
 * Params Values: newly created GtkTreeStore*, initialized GtkTreeIter *iter, 6,G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING
 * Expected: void  
 */
static void test_purpose_44()
{
    #define COLUMNS_N 6
    #define COLUMNS_TYPES "G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING"
    #define COLUMNS_N_AND_TYPES 6,G_TYPE_INT, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING
    #define DATA 0, G_MAXINT, 1, '\0', 2, TRUE, 3, G_MINFLOAT, 4, G_MAXDOUBLE, 5, "\n\t_b", -1
    #define COLUMNS_VALUES "G_MAXINT,'\0',TRUE,G_MINFLOAT,G_MAXDOUBLE,\"\n\t_b\""
    
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GValue val = {0,};
    GValue nval = {0,};
    guint i;
    gchar* tp;
    gchar* columns_types = calloc (1, strlen (COLUMNS_TYPES) + 1);
    gchar* columns_values = calloc (1, strlen (COLUMNS_VALUES) + 1);
    GType types[COLUMNS_N];
    GValue values[COLUMNS_N];
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    memcpy (columns_types, COLUMNS_TYPES, strlen (COLUMNS_TYPES));
    memcpy (columns_values, COLUMNS_VALUES, strlen (COLUMNS_VALUES));
    
    
    tree_store = gtk_tree_store_new (COLUMNS_N_AND_TYPES);
    if (!tree_store) 
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    gtk_tree_store_append (tree_store, &iter, NULL); 
    
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        types [i] = g_type_from_name (tp);
        tp = (char*) strtok (NULL, ",");
    }
    
    tp = (char*)strtok (columns_values, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            g_value_set_int (&values[i],atoi(tp));
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            g_value_set_double (&values[i],atof(tp));
           
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            g_value_set_long (&values[i],atol(tp));
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            g_value_set_string (&values[i],tp);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            g_value_set_uint (&values[i],(u_int)atoi(tp));
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            g_value_set_char (&values[i],atoi(tp));
        } 
        tp = (char*) strtok (NULL, ",");
    }
    
    
    global_valist (tree_store, &iter, DATA);
    
    tp = (char*)strtok (columns_types, ",");
    for (i = 0; tp != NULL && i < COLUMNS_N; i++)
    {  
        if (types [i] == G_TYPE_INT)
        {
            g_value_init(&val, G_TYPE_INT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
    
            if (g_value_get_int (&val) != g_value_get_int (&values[i]))
            {
                TEST_FAIL ("Int Value is '%d',should be '%d'",g_value_get_int (&val),g_value_get_int (&values[i]));
               
            }
            g_value_unset (&val);
    
        } else if (types [i] == G_TYPE_DOUBLE)
        {
            g_value_init(&val, G_TYPE_DOUBLE);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("long Value is '%d',should be '%d'",g_value_get_long (&val),g_value_get_long (&values[i]));
               
            }
            g_value_unset (&val);
        } else if (types [i] == G_TYPE_LONG)
        {
            g_value_init(&val, G_TYPE_LONG);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_long (&val) != g_value_get_long (&values[i]))
            {
                TEST_FAIL ("Long Value is '%d',should be '%d'",g_value_get_double (&val),g_value_get_double (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_STRING)
        {
            g_value_init(&val, G_TYPE_STRING);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_string (&val) != g_value_get_string (&values[i]))
            {
                TEST_FAIL ("String Value is '%d',should be '%d'",g_value_get_string (&val),g_value_get_string (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_UINT)
        {
            g_value_init(&val, G_TYPE_UINT);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_uint (&val) != g_value_get_uint (&values[i]))
            {
                TEST_FAIL ("Uint Value is '%d',should be '%d'",g_value_get_uint (&val),g_value_get_uint (&values[i]));
               
            }
            g_value_unset (&val);
            
        } else if (types [i] == G_TYPE_CHAR)
        {
            g_value_init(&val, G_TYPE_CHAR);
            gtk_tree_model_get_value (GTK_TREE_MODEL (tree_store),&iter,i,&val);
            if (g_value_get_char (&val) != g_value_get_char (&values[i]))
            {
                TEST_FAIL ("Char Value is '%d',should be '%d'",g_value_get_char (&val),g_value_get_char (&values[i]));
               
            }
            g_value_unset (&val);
            
        } 
        
    }
    
    #undef COLUMNS_N
    #undef COLUMNS_TYPES 
    #undef COLUMNS_N_AND_TYPES
    #undef DATA 
    #undef COLUMNS_VALUES 
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "0:0"
 * Expected: TRUE  
 */
static void test_purpose_45()
{
    #define PATH_STR "0:0"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "2:1"
 * Expected: TRUE  
 */
static void test_purpose_46()
{
    #define PATH_STR "2:1"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "4:4"
 * Expected: TRUE  
 */
static void test_purpose_47()
{
    #define PATH_STR "4:4"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "4:2"
 * Expected: TRUE  
 */
static void test_purpose_48()
{
    #define PATH_STR "4:2"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "0:2"
 * Expected: TRUE  
 */
static void test_purpose_49()
{
    #define PATH_STR "0:2"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "3:4"
 * Expected: TRUE  
 */
static void test_purpose_50()
{
    #define PATH_STR "3:4"
    #define CORRECT_RES TRUE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "5:0"
 * Expected: FALSE  
 */
static void test_purpose_51()
{
    #define PATH_STR "5:0"
    #define CORRECT_RES FALSE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "5:5"
 * Expected: FALSE  
 */
static void test_purpose_52()
{
    #define PATH_STR "5:5"
    #define CORRECT_RES FALSE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "1:5"
 * Expected: FALSE  
 */
static void test_purpose_53()
{
    #define PATH_STR "1:5"
    #define CORRECT_RES FALSE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_iter_is_valid
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeStore.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointed to "11:5"
 * Expected: FALSE  
 */
static void test_purpose_54()
{
    #define PATH_STR "11:5"
    #define CORRECT_RES FALSE
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                         &test_iter, PATH_STR);
    
    if (iter_valid != CORRECT_RES)
    {
        TEST_FAIL ("iter_valid = %d, should be %d (path = '%s')", iter_valid, CORRECT_RES, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_RES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:0"
 * Expected: void  
 */
static void test_purpose_55()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:0"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:1"
 * Expected: void  
 */
static void test_purpose_56()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:1"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:4"
 * Expected: void  
 */
static void test_purpose_57()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "1:2", GtkTreeIter* pointing to "1:4"
 * Expected: void  
 */
static void test_purpose_58()
{
    #define ITER_PATH_STR "1:2"
    #define POSITION_PATH_STR "1:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "3:3", GtkTreeIter* pointing to "3:2"
 * Expected: void  
 */
static void test_purpose_59()
{
    #define ITER_PATH_STR "3:3"
    #define POSITION_PATH_STR "3:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:2", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_60()
{
    #define ITER_PATH_STR "4:2"
    #define POSITION_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:2"
 * Expected: void  
 */
static void test_purpose_61()
{
    #define ITER_PATH_STR "4:3"
    #define POSITION_PATH_STR "4:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_62()
{
    #define ITER_PATH_STR "4:3"
    #define POSITION_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_63()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "1:1", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_64()
{
    #define ITER_PATH_STR "1:1"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "2:2", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_65()
{
    #define ITER_PATH_STR "2:2"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "3:3", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_66()
{
    #define ITER_PATH_STR "3:3"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_before
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position before position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:4", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_67()
{
    #define ITER_PATH_STR "4:4"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_before (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos > ipiter)
        {
            sprintf (ppos + 1, "%d", ippos - 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = strdup (iter_path_str);
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", sizeof (child_data) / sizeof (struct ListSort) - 1);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    free (iter_path_str);
    free (position_path_str);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:0"
 * Expected: void  
 */
static void test_purpose_68()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:0"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:1"
 * Expected: void  
 */
static void test_purpose_69()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:1"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:4"
 * Expected: void  
 */
static void test_purpose_70()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR "0:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "1:2", GtkTreeIter* pointing to "1:4"
 * Expected: void  
 */
static void test_purpose_71()
{
    #define ITER_PATH_STR "1:2"
    #define POSITION_PATH_STR "1:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "3:3", GtkTreeIter* pointing to "3:2"
 * Expected: void  
 */
static void test_purpose_72()
{
    #define ITER_PATH_STR "3:3"
    #define POSITION_PATH_STR "3:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:2", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_73()
{
    #define ITER_PATH_STR "4:2"
    #define POSITION_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:2"
 * Expected: void  
 */
static void test_purpose_74()
{
    #define ITER_PATH_STR "4:3"
    #define POSITION_PATH_STR "4:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_75()
{
    #define ITER_PATH_STR "4:3"
    #define POSITION_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_76()
{
    #define ITER_PATH_STR "0:0"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "1:1", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_77()
{
    #define ITER_PATH_STR "1:1"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "2:2", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_78()
{
    #define ITER_PATH_STR "2:2"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "3:3", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_79()
{
    #define ITER_PATH_STR "3:3"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_move_after
 * Params: GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *position
 * Returns: void
 * Assertion: Moves iter in tree_store to the position after position.
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:4", GtkTreeIter* pointing to NULL
 * Expected: void  
 */
static void test_purpose_80()
{
    #define ITER_PATH_STR "4:4"
    #define POSITION_PATH_STR NULL
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter;
    GtkTreeIter target_iter;
    GtkTreeIter pos_test_iter;
    GtkTreePath* iter_path;
    gint i;
    gint j;
    GValue iter_val = {0,};
    GValue pos_val = {0,};
    guint iter_uint_value;
    guint pos_uint_value;
    gboolean iter_valid;
    gchar* iter_path_str = strdup (ITER_PATH_STR);
    gchar* position_path_str = POSITION_PATH_STR ? strdup (POSITION_PATH_STR) : NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &test_iter, iter_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", iter_path_str);
        return;
    }
    
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &test_iter, 0, &iter_val);
    iter_uint_value = g_value_get_uint (&iter_val);
    
    
    if (position_path_str) {
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &target_iter, position_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
            return;
        }
    }
    
    gtk_tree_store_move_after (store, &test_iter, position_path_str ? &target_iter : NULL);
    
    
    if (position_path_str) {
        /* we expect correct parameters passed to the function */
        gchar* ppos;
        gchar* piter;
        int ippos;
        int ipiter;
    
        ppos = strrchr (position_path_str, ':');
        piter = strrchr (iter_path_str, ':');
    
        if (!ppos)
        {
            ppos = position_path_str;
            piter = iter_path_str;
        }
    
        ippos = atoi (ppos + 1);
        ipiter = atoi (piter + 1);
        
        if (ippos < ipiter)
        {
            sprintf (ppos + 1, "%d", ippos + 1);
        }
    } else {
        gchar* pcolon;
    
        position_path_str = iter_path_str;
        pcolon = strrchr (position_path_str, ':');
    
        sprintf (pcolon + 1, "%d", 0);
    }
     
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &pos_test_iter, 
                                                      position_path_str);
    if (!iter_valid)
    {
        TEST_FAIL ("can't get iter pointing to '%s'", position_path_str);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &pos_test_iter, 
                              0, &pos_val);
    pos_uint_value = g_value_get_uint (&pos_val);
    
    if (iter_uint_value != pos_uint_value)
    {
        TEST_FAIL ("value of iter pointing to '%s' is %d, should be %d", 
                   position_path_str, pos_uint_value, iter_uint_value);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef POSITION_PATH_STR
    #undef ITER_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:0"
 * Expected: void  
 */
static void test_purpose_81()
{
    #define A_PATH_STR "0:0"
    #define B_PATH_STR "0:0"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:1"
 * Expected: void  
 */
static void test_purpose_82()
{
    #define A_PATH_STR "0:0"
    #define B_PATH_STR "0:1"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "0:0", GtkTreeIter* pointing to "0:4"
 * Expected: void  
 */
static void test_purpose_83()
{
    #define A_PATH_STR "0:0"
    #define B_PATH_STR "0:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "1:2", GtkTreeIter* pointing to "1:4"
 * Expected: void  
 */
static void test_purpose_84()
{
    #define A_PATH_STR "1:2"
    #define B_PATH_STR "1:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "3:3", GtkTreeIter* pointing to "3:2"
 * Expected: void  
 */
static void test_purpose_85()
{
    #define A_PATH_STR "3:3"
    #define B_PATH_STR "3:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:2", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_86()
{
    #define A_PATH_STR "4:2"
    #define B_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:2"
 * Expected: void  
 */
static void test_purpose_87()
{
    #define A_PATH_STR "4:3"
    #define B_PATH_STR "4:2"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:3", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_88()
{
    #define A_PATH_STR "4:3"
    #define B_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_swap
 * Params: GtkTreeStore *tree_store, GtkTreeIter *a, GtkTreeIter *b
 * Returns: void
 * Assertion: Swaps a and b in the same level of tree_store. 
 * Params Values: newly created GtkTreeStore*, GtkTreeIter* pointing to "4:4", GtkTreeIter* pointing to "4:4"
 * Expected: void  
 */
static void test_purpose_89()
{
    #define A_PATH_STR "4:4"
    #define B_PATH_STR "4:4"
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iter_a_before;
    GtkTreeIter iter_b_before;
    GtkTreeIter iter_a_after;
    GtkTreeIter iter_b_after;
    gint i;
    gint j;
    GValue iter_a_val_before = {0,};
    GValue iter_b_val_before = {0,};
    GValue iter_a_val_after = {0,};
    GValue iter_b_val_after = {0,};
    guint iter_a_uint_value_before;
    guint iter_b_uint_value_before;
    guint iter_a_uint_value_after;
    guint iter_b_uint_value_after;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_before, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_before, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_before, 0, &iter_a_val_before);
    iter_a_uint_value_before = g_value_get_uint (&iter_a_val_before);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_before, 0, &iter_b_val_before);
    iter_b_uint_value_before = g_value_get_uint (&iter_b_val_before);
    
    gtk_tree_store_swap (store, &iter_a_before, &iter_b_before);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_a_after, A_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", A_PATH_STR);
        return;
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &iter_b_after, B_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: can't get iter pointing to '%s'", B_PATH_STR);
        return;
    }
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_a_after, 0, &iter_a_val_after);
    iter_a_uint_value_after = g_value_get_uint (&iter_a_val_after);
    
    gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iter_b_after, 0, &iter_b_val_after);
    iter_b_uint_value_after = g_value_get_uint (&iter_b_val_after);
    
    if (iter_a_uint_value_before != iter_b_uint_value_after)
    {
        TEST_FAIL ("iter_a_uint_value_before = %d (path = '%s'), iter_b_uint_value_after = %d (path = '%s'), they should be the same", 
                   iter_a_uint_value_before, 
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_a_before)),
                   iter_b_uint_value_after,
                   gtk_tree_path_to_string (gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter_b_after)));
    }
    
    if (iter_b_uint_value_before != iter_a_uint_value_after)
    {
        TEST_FAIL ("iter_b_uint_value_before = %d, iter_a_uint_value_after = %d, they should be the same", 
                   iter_b_uint_value_before, iter_a_uint_value_after);
    }
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef A_PATH_STR
    #undef B_PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_90()
{
    #define PARENT_PATH_STR "0"
    #define NEW_ORDER_ARRAY {0, 1, 2, 3, 4}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_91()
{
    #define PARENT_PATH_STR "0"
    #define NEW_ORDER_ARRAY {1, 0, 2, 3, 4}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_92()
{
    #define PARENT_PATH_STR "0"
    #define NEW_ORDER_ARRAY {0, 1, 2, 4, 3}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_93()
{
    #define PARENT_PATH_STR "0"
    #define NEW_ORDER_ARRAY {4, 0, 1, 2, 3}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_94()
{
    #define PARENT_PATH_STR "4"
    #define NEW_ORDER_ARRAY {4, 2, 0, 1, 3}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_95()
{
    #define PARENT_PATH_STR "3"
    #define NEW_ORDER_ARRAY {2, 4, 1, 3, 0}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_96()
{
    #define PARENT_PATH_STR "2"
    #define NEW_ORDER_ARRAY {0, 4, 3, 1, 2}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_97()
{
    #define PARENT_PATH_STR "2"
    #define NEW_ORDER_ARRAY {1, 4, 2, 0, 3}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_store_reorder
 * Params: GtkTreeStore *tree_store, GtkTreeIter *parent, gint *new_order
 * Returns: void
 * Assertion: Reorders the children of parent in tree_store to follow the order indicated by new_order.
 * Params Values: newly created GtkTreeStore*, gint*
 * Expected: void  
 */
static void test_purpose_98()
{
    #define PARENT_PATH_STR "2"
    #define NEW_ORDER_ARRAY {4, 1, 2, 3, 0}
    
    GtkTreeStore* store;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter iters_before[NUM_TREE_ELEMS];
    GtkTreeIter iters_after[NUM_TREE_ELEMS];
    guint iters_uint_value_before[NUM_TREE_ELEMS];
    guint iters_uint_value_after[NUM_TREE_ELEMS];
    GValue iters_val_before[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GValue iters_val_after[] = {{0,}, {0,}, {0,}, {0,}, {0,}};
    GtkTreeIter parent_iter;
    gboolean iter_valid;
    gint i;
    gint j;
    gint new_order_array[] = NEW_ORDER_ARRAY;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (sizeof (new_order_array) / sizeof (new_order_array[0]) != 5)
    {
        TEST_FAIL ("new_order_array should contain 5 elements", NULL);
        return;
    }
    
    store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (store, &iter, NULL);
        gtk_tree_store_set (store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (store, &child_iter, &iter);
            gtk_tree_store_set (store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                      &parent_iter, 
                                                      PARENT_PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("0: can't get iter pointing to '%s'", PARENT_PATH_STR);
        return;
    }
    
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_before[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("1: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_before[i], 0, 
                                  &iters_val_before[i]);
        iters_uint_value_before[i] = g_value_get_uint (&iters_val_before[i]);
    }
    
    gtk_tree_store_reorder (store, &parent_iter, new_order_array);
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        char* child_path_str = (char*) calloc (sizeof (char), strlen (PARENT_PATH_STR) + 3);
        
        sprintf (child_path_str, "%s%s%d", PARENT_PATH_STR, ":", i);
        iter_valid = gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), 
                                                          &iters_after[i], 
                                                          child_path_str);
        if (!iter_valid)
        {
            TEST_FAIL ("2: can't get iter pointing to '%s', i = %d", 
                       child_path_str, i);
            return;
        }
        gtk_tree_model_get_value (GTK_TREE_MODEL (store), &iters_after[i], 0, 
                                  &iters_val_after[i]);
        iters_uint_value_after[i] = g_value_get_uint (&iters_val_after[i]);
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        for (j = 0; j < NUM_TREE_ELEMS && new_order_array[j] != i; j++)
            ;
    
        if (iters_uint_value_before[i] != iters_uint_value_after[j])
        {
            TEST_FAIL ("iters_uint_value_before[%d] = %d, iters_uint_value_after[%d] = %d, they should be equal", i, iters_uint_value_before[i], j, iters_uint_value_after[j]);
        }
    }
    
    gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), 
                              gtk_tree_path_new_from_string (PARENT_PATH_STR), 
                              TRUE);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PARENT_PATH_STR
    #undef NEW_ORDER_ARRAY
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkTreeStore ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {test_purpose_41, 1},
    {test_purpose_42, 1},
    {test_purpose_43, 1},
    {test_purpose_44, 1},
    {test_purpose_45, 1},
    {test_purpose_46, 1},
    {test_purpose_47, 1},
    {test_purpose_48, 1},
    {test_purpose_49, 1},
    {test_purpose_50, 1},
    {test_purpose_51, 1},
    {test_purpose_52, 1},
    {test_purpose_53, 1},
    {test_purpose_54, 1},
    {test_purpose_55, 1},
    {test_purpose_56, 1},
    {test_purpose_57, 1},
    {test_purpose_58, 1},
    {test_purpose_59, 1},
    {test_purpose_60, 1},
    {test_purpose_61, 1},
    {test_purpose_62, 1},
    {test_purpose_63, 1},
    {test_purpose_64, 1},
    {test_purpose_65, 1},
    {test_purpose_66, 1},
    {test_purpose_67, 1},
    {test_purpose_68, 1},
    {test_purpose_69, 1},
    {test_purpose_70, 1},
    {test_purpose_71, 1},
    {test_purpose_72, 1},
    {test_purpose_73, 1},
    {test_purpose_74, 1},
    {test_purpose_75, 1},
    {test_purpose_76, 1},
    {test_purpose_77, 1},
    {test_purpose_78, 1},
    {test_purpose_79, 1},
    {test_purpose_80, 1},
    {test_purpose_81, 1},
    {test_purpose_82, 1},
    {test_purpose_83, 1},
    {test_purpose_84, 1},
    {test_purpose_85, 1},
    {test_purpose_86, 1},
    {test_purpose_87, 1},
    {test_purpose_88, 1},
    {test_purpose_89, 1},
    {test_purpose_90, 1},
    {test_purpose_91, 1},
    {test_purpose_92, 1},
    {test_purpose_93, 1},
    {test_purpose_94, 1},
    {test_purpose_95, 1},
    {test_purpose_96, 1},
    {test_purpose_97, 1},
    {test_purpose_98, 1},
    {NULL,0}
};
